/* ==== ai.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */

#include <stdlib.h>
#include "C-foogrid.h"
#include "porrog_defs.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h"
#include "ai_t.h"
#include "player_t.h"

/* protos */
#include "ai_p.h"
#include "ai_wpt.h" /* Also has some types specific to itself */
#include "monster.h"
#include "dXrng.h"
#include "player_p.h"
#include "makemap_p.h"
#include "mg_utils.h"
#include "ui.h"
#include "crash.h"

#define sgn(n) (n>0?1:(n<0?-1:0))

void ai_act(monster_t *mob)
{
    int mstype; /* Type of tile stood upon */
    int routetype;/* Type of tiles route passes through */
    int bstype; /* Type of tile where checking to go */
    int mx=mob->X, my=mob->Y; /* Mob's coords. For ease of code */
    int Dx=0,Dy=0; /* Used in calculations of dx,dy */
    int dx=0,dy=0; /* Direction we will go in */
    int ex,ey; /* Used in scanning for target */
    int ox,oy; /* A random offset to make scans of surroundings unpredictable*/
    int do_open=0; /* Set if we need to open a door */
    int ubcount=0; /* Number of unblocked spaces around mob */
    int ubtarget; /*Aha! This is how we decide where to go at
       those intersections- just pick one randomly... */
    int ubtotal=0; /* This should replace ubcount, to mean how
       many clear spaces total. Count will now be a counter variable
       when counting up to the target */
    int doortotal=0; /* How many doors, whilst finding unblocked squares*/
    int passing=0; /* If set to 1, check for differnt waypoint after moving*/
    int mroomno;
    region_t *mregion;
    aicache_rooment_t *rooment;
    aicache_tunnelent_t *regent;
    intersection_t *inter; /* Used in finding point for monsters to pass*/
    maploc_t *mapstart=map->Data;
    maploc_t *elem;
    tile_t blockage; /* Tile in monster's path */
    entrance_t *target; /* Temporary var for ease of code */
    /* First: Is the monster in a tunnel region, room, doorway? 0,1.2
    NB: If mob can somehow be in a wall, don't know what to do.
    Should prolly check for this at the moment, and either crash
    (reporting it), or simply return straight away- where could
    it go? */
    aicache_updatetimes(mob);
    mstype=m_whatamion(mob);

    /* Then: Is the player in a tunnel region, room, doorway? 0,1,2
    Actually, this value should be global, and calculated once
    per turn.
    Issues of being in walls apply here, too- how could the
    mob reach the player? */
    /* pstype is player's equivalent */

/*  if(mstype==0||mstype==2) mregion=find_region(mx,my);
    if(mstype==1||mstype==2) mroomno=find_room(mx,my); */
    switch(mstype) {
       case 0:
         mregion=find_region(mx,my);
         break;
       case 1:
         mroomno=find_room(mx,my); /* this being so slow, mebbe
         we should store it with the mob and work out when it
         needs to be updated instead */
         break;
       case 2:
         mregion=find_region(mx,my);
         mroomno=find_room(mx,my);
         /* Now update cache stuff!
         Used to be done later on, but meant it got missed out
         when tracking the player. */
         rooment=aicache_roomcache(mob,mroomno,0);
         regent=aicache_regcache(mob,mregion,0);
         if (rooment==NULL) rooment=aicache_trycacheroom(mob,mroomno);
           else rooment->Age=0; /* Reset the timestamp of the room-
           -but not of its doors! */
         if (regent==NULL) regent=aicache_trycacheregion(mob,mregion);
           else regent->Age=0; /* As with rooment */
         /* We should now be GUARANTEED to have a valid pointer,
         since changes to cache code (to implement buffers) */
         if(rooment==NULL||regent==NULL) crash(INCONSISTENT);
         if (rooment!=NULL) aicache_tryresetdoor(mob,rooment->Paths);
         if (regent!=NULL) aicache_tryresetdoor(mob,regent->Paths);
         /* End of cache update code */

         break;
    }

    /* Count surrounding free spaces. Needed in many eventualities
    to figure out if mob is obstructed by other monsters.
    NB- The count variable used to be ubcount, but is now ubtotal,
    as ubcount will be needed elsewhere. */
    for(ey=-1;ey<=1;ey++)
      for(ex=-1;ex<=1;ex++)
      {
        if(ey==0&&ex==0)continue; /* Don't count this space!! */
        elem=(mapstart+FOOGRID_IDX(map,my+ey,mx+ex));
        blockage=elem->Tiletype;
        ubtotal+=(blockage>=PTL_FLOOR &&elem->Mptr==NULL);
        /* Yes, remember we're counting how many are UNblocked */
        doortotal+=((blockage==PTL_DOOR ||blockage==PTL_ODOOR)&&elem->Mptr==NULL);
        /* Count unblocked doors */
      }
    if (ubtotal==8&&mob->Annoyance>0) mob->Annoyance--;
    /* ubtarget is set later if needed */

    /* Now, work out if mob is in same place as player */
    if ( (0==mstype &&(0==pstype ||2==pstype) &&mregion==psregion &&(routetype=0,1))
    ||((1==mstype||2==mstype) &&(1==pstype||2==pstype) &&mroomno==psroomno &&(routetype=1,1))
    ||(2==mstype && 0==pstype &&mregion==psregion &&(routetype=0,1)) ) {  /* If mob AND player are on doors, we only check if they're
    in the same room. Too hard to check *everything* */
        /* In this case, mob and player are in the same room or region,
        so we just navigate within that room/region.
        Also, we don't need to worry about which doors to go through,
        as there can be no doors between the mob and player. Hence,
        we can use the obstacle avoidance routine later without having
        to tell it that it's OK to go through doors, because we
        shouldn't need to. It's still possible we might later
        decide that passing through doors is appropriate when a
        location gets overcrowded. */

        /* routetype is set within that if statement above! */
        if (routetype) /*0==mstype || 0==pstype ||(2==mstype&&2==pstype))*/ {
           /* Room? */
           /* Should really have the old behaviour, of
           try to go straight towards player, but if that path is
           blocked, try other ways around.
           Oh, and if on a door and path is blocked, try to find
           any surrounding spaces that are room type tiles */
           ai_set_waypoint(mob,routetype,player.X,player.Y,1);
           Dx=mob->Wx-mx;
           Dy=mob->Wy-my;
           dx=sgn(Dx);dy=sgn(Dy);
        } else {
           /* Tunnel region? */
           /* used to pass mstype, but that can be 2,
           which ai_set_waypoint wont accept. */
           ai_set_waypoint(mob,routetype,player.X,player.Y,1);

           Dx=mob->Wx-mx; /* Whee! */
           Dy=mob->Wy-my;
           dx=sgn(Dx);dy=sgn(Dy);
           /* Want to check for path blockages as elsewhere */
        }
        /* We ought to remember that in this mode, we don't check for
        mob changing location, and therefore dont update the cache or
        buffers... If we end up changing back to the other mode
        (because the player escapes beyond where the mob can detect
        him), then we may find ourselves with a stale target (such
        as a door from a totally different location), or trying
        to make use of a NULL cache response...*/

    } else { /* So *not* in the same room/region. Nearby? */
        if( (0==mstype &&1==pstype && NULL!=find_common_door(mregion,psroomno))
        || (0==pstype &&1==mstype && NULL!=find_common_door(psregion,mroomno)) )
        {
             /* Target the door between mob+player */
             target=pstype? find_common_door(mregion,psroomno) : find_common_door(psregion,mroomno);
             mob->Tx=target->Xcoord; mob->Ty=target->Ycoord;
             mob->Ttype=pstype;
             /* We want to remember when we set a target door, that
             we have to go to the square BEYOND the door; the square
             must be a different type area to the one the mob is
             in when the target is set. In this way, the mob continues
             going from region to room to region, continuously. */
             ai_set_waypoint(mob,mstype,mob->Tx,mob->Ty,1);
             /* The only door it's acceptable to go through is the
             one targeted- so we don't have to tell the obstacle avoidance
             block that it's should allow passing through doorways
             (it permits it when the doorway is at the target */
        } else { /* So mob is not near enough to player. Just wander */
            /* To work out if we've reached the target: simply
             check mstype==mob->Ttype! */
            if (mstype==mob->Ttype) {/* We must have hit the target!!*/
                ai_acquire_target(mob,mstype);
                ai_set_waypoint(mob,mstype,mob->Tx,mob->Ty,1);
            }
            /* And then make a step towards new target this turn */
            /* There is no else. Whether we got a new target or not,
            we still continue moving! */
        }
        /* We should have some sort of target. Get direction/action
        appropriate to following that target. */
        /* Method: If mstype == tile target was set from, go to door
                   If mstype == opposite tile type, are at target
                   (so should have been caught earlier)
                   If mstype == 2 (door), must make step towards
                   tile of opposite type.
                   If target door is closed and adjacent, dont
                   move, but open the door ;) */
        /* Perhaps this could be a switch? */
        if (2==mstype) {
           /* Also remember this is a good time to update cache stuff. */
           /* First check room and region caches, to find if either has
           any hits. If either does, reset this doorway in it. If
           they're *not* in the cache, try entering them in it, and
           if successful, reset this doorway in it. */
           /* cache checks here ignore the buffer, as in at least one
           case, it'll be for a different location, and for the other,
           we won't have lost anything... 
           Could actually change this so we consider using the buffer
           for the location we were just in, but not for the location
           we're stepping into. use mob->Ttype to determine this. */

           /* CACHE UPDATE NOW DONE EARLIER  */

           /* Go towards opposite type of tile */
           ox=(roll_dX(d6set[0])%3);oy=(roll_dX(d6set[1])%3);
           for (dy=0;dy<3;dy++)
              for(dx=0;dx<3;dx++)
              {
                 ex=((dx+ox)%3)-1; ey=((dy+oy)%3)-1;
                 elem=(mapstart+FOOGRID_IDX(map,my+ey,mx+ex));
                 blockage=elem->Tiletype;
                 bstype=( (blockage<PTL_FLOOR /* Must be wall, or door */
                 ||elem->Mptr!=NULL /* Dont want to walk into another mob */)
                 ? 2 /* Otherwise means door, but here includes walls */
                 : ((blockage==PTL_TUNNEL) ?0:1));
                 if (bstype==mob->Ttype) {
                    dx=ex;dy=ey;
                    goto gottarget;
                 }
              }
           if (mob->Ttype) {/* Cant get into room! Wait.*/
              dx=0;dy=0; /* If we couldn't find a way, stay still. */
              sprintf(strbuf,"Monster at %d,%d cant move off door.",mx,my);
              bugger(REPTYPE_AI,LOGLEVEL_COMMENTARY,strbuf);
              /* Is that appropriate severity? */
           } else { /* Cant get into tunnel- try returning! */
           /* Would be a useful thing for monsters trying to get into
           tunnels to be able to wander back into the room to let
           monsters out of the tunnel. This can be done! */
              if (ubtotal==0) {
                 dx=0;dy=0;goto gottarget;
              }
              /* Let's go back. */
              /* First get a waypoint somewhere within the room */
              mob->Wx=(tls->Room[mroomno].Xstart+1+
                ((tls->Room[mroomno].Width-2)*(roll_dX(d20set[0]))/20));
              mob->Wy=(tls->Room[mroomno].Ystart+1+
                ((tls->Room[mroomno].Height-2)*(roll_dX(d12set[0]))/12));
              /* But we don't change the target, nor the target type */
              /* Randomize the offsets */
              ox=(roll_dX(d6set[0])%3);oy=(roll_dX(d6set[1])%3);
              for (dy=0;dy<3;dy++)
                 for(dx=0;dx<3;dx++)
                 {
                    ex=((dx+ox)%3)-1; ey=((dy+oy)%3)-1;
                    elem=(mapstart+FOOGRID_IDX(map,my+ey,mx+ex));
                    blockage=elem->Tiletype;
                    bstype=( (blockage<PTL_FLOOR /* Must be wall, or door */
                    ||elem->Mptr!=NULL /* Dont want to walk into another mob */)
                    ? 2 /* Otherwise means door, but here includes walls */
                    : ((blockage==PTL_TUNNEL) ?0:1));
                    if (bstype==1) {
                       dx=ex;dy=ey;
                       goto gottarget;

                    }
                 }
              crash(FUNCTION_ERROR); /* If we couldn't get back to the room,
              ubtotal should have been 0. */
           }
           gottarget: Dx=dx;Dy=dy;
        } else {

           /* SOMEWHERE AROUND HERE -
           Adjust the block structure such that what follows
           is executed no matter what- mobs should be able
           to avoid obstacles no matter what.
           I think here is prolly good. */

           /* Check if entrance is adjacent.
              If mstype==1, can go roughly straight line to door.
              If mstype==0, can work out some path there.
              Actually, NO! not so. It's possible for a mob to
              be hugging the walls of a room to get to a door,
              and it will then go through the first door it
              sees with this algorithm... */
           if(mx==mob->Wx&&my==mob->Wy)
              ai_set_waypoint(mob,mstype,mob->Tx,mob->Ty,1);
           Dx=(mob->Wx-mx);Dy=(mob->Wy-my);
           dx=sgn(Dx);dy=sgn(Dy);
           /* 2==t_whatishere(mx+dx,my+dy) might perhaps fail
           in very odd circumstances. Could instead try
           mob->Tx-mx=dx, etc. */
        }
    }
    /* RIGHT. We splice HERE. */

           elem=(mapstart+FOOGRID_IDX(map,my+dy,mx+dx));
           blockage=elem->Tiletype;
           if ((dx==Dx)&&(dy==Dy)&&(elem->Mptr==NULL)) {
              /* The target door is right next to us! We can either
              go through it if its open (so change nothing), or
              open it if it's closed (so just set do_open=1) */
              if( blockage==PTL_DOOR)
                 do_open=1;
              /* If a monster's on the door, we move away a bit- may give
              them a chance to get through */
              if (elem->Mptr!=NULL) crash(FUNCTION_ERROR);
           } else {
              /* Check if the exact direction is blocked.
              If so, try shifting angle by 45-90 degrees */
              /* NOTE: WE SHOULD INSTEAD TREAT BLOCKAGE BY WALLS/DOORS
              DIFFERENTLY TO BLOCKAGE BY OTHER MONSTERS!!! */
              if (PTL_PWALL==blockage ||PTL_ROCK==blockage||PTL_HWALL==blockage
              ||PTL_VWALL==blockage||PTL_CORNER==blockage||PTL_DOOR==blockage||PTL_ODOOR==blockage)
              {
                 /* Tile isn't suitable to walk into. *Could*
                 walk into door tiles, but it would be inappropriate
                 behaviour- taking monster to a quite different path */
                 if (dx!=0&&dy!=0) { /* Was trying to go diagonal*/
                    /* Try zeroing one of the axes, in order to go
                    orthogonal. This should work in many cases-
                    I have no idea what sort of cases it wouldnt work with*/
                    elem=(mapstart+FOOGRID_IDX(map,my+dy,mx));
                    blockage=elem->Tiletype;
                    if(blockage!=PTL_PWALL&&blockage!=PTL_ROCK&&blockage!=PTL_HWALL&&blockage!=PTL_VWALL&&blockage!=PTL_CORNER&&blockage!=PTL_DOOR&&blockage!=PTL_ODOOR &&elem->Mptr==NULL) {
                       dx=0;goto jumpout; /* Ouch */
                    }
                    elem=(mapstart+FOOGRID_IDX(map,my,mx+dx));
                    blockage=elem->Tiletype;
                    if(blockage!=PTL_PWALL&&blockage!=PTL_ROCK&&blockage!=PTL_HWALL&&blockage!=PTL_VWALL&&blockage!=PTL_CORNER&&blockage!=PTL_DOOR&&blockage!=PTL_ODOOR &&elem->Mptr==NULL) {
                       dy=0;goto jumpout;
                    }
                    /* Well that didn't help. Perhaps it was because there
                    was another monster in the way, in which case we could
                    move round it, or we could wait a turn. */
                    sprintf(strbuf,"Monster at %d,%d found no clear way round blocked route.",mx,my);
                    bugger(REPTYPE_AI,LOGLEVEL_COMMENTARY,strbuf);
                    /* Sufficient severity? */
                    if (roll_dX(d6set[0])<4) {dx=0;dy=0;goto jumpout;}
                    /* Otherwise scan area for alternative directions */
                    ox=(roll_dX(d6set[0])%3);oy=(roll_dX(d6set[1])%3);
                    for(dy=0;dy<3;dy++)
                       for(dx=0;dx<3;dx++)
                       {
                          ex=((dx+ox)%3)-1; ey=((dy+oy)%3)-1;
                          elem=(mapstart+FOOGRID_IDX(map,my+ey,mx+ex));
                          blockage=elem->Tiletype;
                          if(blockage>=PTL_FLOOR&&elem->Mptr==NULL){
                            dx=ex;dy=ey;goto jumpout;
                          }
                       }
                    dx=0;dy=0;
                 }
              }
              /*Should this test go first instead? */
              /*else
              Aargh! There's cases where target is a door,
              or we had some other obstacle but couldn't avoid it by
              going orthogonally rather than diagonally, and it'd
              drop through to here, in which case finding the *else*
              would leave no solution whatsoever!*/
              if (elem->Mptr!=NULL) {
                 /* Better mechanism:
                 Count how many *unblocked* squares
                 there are around. Possibly shouldn't count entrances.
                  -Done at start now.
                 Then, if that count is only 1, there must be some
                 monster in the way (we don't have dead-end tunnels).
                 At this point, we increment our "annoyance" counter.
                 If the annoyance counter is higher than our respect
                 for the monster directly in our way, we hit it;
                 otherwise, we work out the closest point at which we
                 could pass each other, and set that as the waypoint.
                 Hopefully, the mob in the way will do the same.
                 If the unblocked count is 0, then we're totally
                 hemmed in!!! We increase our annoyance, and then
                 either fight our way out, or wait patiently...
                 NB- We will consider the respect for both the
                 monster directly in the way of where we want to
                 go, and the monster we respect least. If we lose
                 patience with one first, we fight that one. If
                 we lose patience with 2 at once, we fight
                 the one directly in the way.
                 If the unblocked count is 2 or more, we still
                 increase our annoyance counter, but very little,
                 and then alter our path to choose one of the
                 alternative squares, just to temporarily step
                 out of the way. Hopefully this will allow the
                 other mob to go past and stop the blockage. Or
                 maybe we will end up where it wants to go...

                 Also, the algo described in IDEA_AI2 allows the
                 mob to choose another entrance when in a 2-door
                 tunnel, thereby backing out of the tunnel, and
                 giving the mob in the way time to get through. */

                 /* Diagonal->Orthogonal modification moved
                 to earlier, wall avoidance block */

                 /* Count free spaces now done earlier too */
                 /* Now have ubtotal spaces (was ubcount originally). */
#ifndef CALM
                 if (ubtotal<4) mob->Annoyance+=(4-ubtotal);
#endif
                 switch (ubtotal)
                 {
                    case 0: /* Wait for it to clear, or fight */
                      /* This case can crop up when the only way out
                      is through a door- which we would normally not
                      want to go through. BUT. It has occurred that
                      the path the monster would like to go through
                      also has a monster stuck there, each monster
                      trying to get past the other, and each only
                      having a door to escape through. So what
                      happens then? Both hit this case and think
                      they're trapped!!!
                      So, if one of the spaces around is a door,
                      then there should be some possibility of
                      trying to go through it here. 1/3?
                      Rest of the time, should have default
                      behaviour of either starting a fight or
                      staying put. */
                      if(doortotal!=0 &&(roll_dX(d6set[0])<2)) {
                         /* Scan for the doorway, and choose it*/
                         for(ey=-1;ey<2;ey++)
                           for(ex=-1;ex<2;ex++)
                           {
                              elem=(mapstart+FOOGRID_IDX(map,my+ey,mx+ex));
                              blockage=elem->Tiletype;
                              if((blockage==PTL_DOOR||blockage==PTL_ODOOR)&&elem->Mptr==NULL) {
                                dx=ex;dy=ey;
                                if(blockage==PTL_DOOR) do_open=1;
                                doortotal=0;
                                break;
                              }
                           }
                           if (doortotal==0) break;
                         crash(FUNCTION_ERROR);
                      }
                      sprintf(strbuf,"Monster at %d,%d trapped!",mx,my);
                      bugger(REPTYPE_AI|REPTYPE_MAPGEN,LOGLEVEL_DEBUG,strbuf);
#ifndef CALM
                      if (m_respect(mob,elem->Mptr)>=mob->Annoyance) {
#endif
                        dx=0;dy=0; /* Wait rather than fighting */
#ifndef CALM
                      }
#endif
                      break;
                    case 1:
                    default: /* AHA! Have now generalised algorithm! I hope!*/
                      /* ONE way out. Work out which is the closest
                      intersection, at which both monsters can pass.*/
                      if (ubtotal==1) ubtarget=0;
                      else ubtarget=(ubtotal*roll_dX(d12set[0]))/12;
                        /* Should give target<total */
                      /* Now, subtly different behaviour depending on
                      whether in a tunnel or a room */
                      if (mstype==0) {
                        inter=nearest_intersection(mx,my);
                        if(inter!=NULL) {
                          mob->Wx=inter->Xcoord;mob->Wy=inter->Ycoord;
                        } else { /* No intersections?*/
                          /* Just back out again */
                          ai_target_last_entrance(mob,mregion);
                          mob->Wx=mob->Tx;mob->Wy=mob->Ty;
                        }
                        /* It'd be nice to remember that we're making
                        space, in case the room there is of order 1*/
                        Dx=mob->Wx-mx; Dy=mob->Wy-my;
                        dx=sgn(Dx);dy=sgn(Dy);
                      }
                      if((mstype)||(dx==Dx &&dy==Dy)) {
                        /* We're right next to that intersection.
                        If it's clear, go to it!
                        If not, then the one clear space might be
                        *adjacent* to it, which is just as good for
                        passing the other monster.
                        If it isn't adjacent to it, then the whole
                        intersection might be jammed up, in which
                        case, maybe we should move away? */
                        elem=(mapstart+FOOGRID_IDX(map,my+dy,mx+dx));
                        if(elem->Mptr!=NULL&&(dx!=0||dy!=0)) {
                          /* Pick a clear space*/
                          /* In this case, we should have a certain
                          probability of looking first just in the squares
                          around the intersection, as we'd prefer them-
                          but doing this adds predictable behaviour (IE-
                          it'll always choose the same one position over
                          another given the same state), which I'd say is
                          pretty undesirable. */
                          for(ey=-1;ey<2;ey++)
                            for(ex=-1;ex<2;ex++)
                            {
                              elem=(mapstart+FOOGRID_IDX(map,my+ey,mx+ex));
                              blockage=elem->Tiletype;
                              if(blockage<PTL_FLOOR) continue;
                              if(elem->Mptr==NULL) {
                                /* Heres a space, but do we choose it? */
                                ubcount++;if(ubcount>ubtarget)
                                  goto thisisclear1;
                              }
                            }
                          crash(FUNCTION_ERROR); /* Should never happen*/
                          thisisclear1:;
                          if((abs(ex-dx)<2)&&(abs(ey-dy)<2) ) {
                             /* Next to the intersection, so just as good.*/
                             dx=ex;dy=ey;
                          } else { /* Not a suitable place. Ooo! */
                             sprintf(strbuf,"Monster at %d,%d can't reach intersection.",mx,my);
                             bugger(REPTYPE_AI,LOGLEVEL_COMMENTARY,strbuf);
                             elem=(mapstart+FOOGRID_IDX(map,my+dy,mx+dx));
#ifndef CALM
                             /* Consider fighting monster at intersection*/
                             if(m_respect(mob,elem->Mptr)>=mob->Annoyance) {
                               mob->Annoyance+=3; /* Grrr*/
#endif
                               if (roll_dX(d6set[0])<2) {dx=ex;dy=ey;}
                               else {dx=0;dy=0; /* Wait for it to clear */}
#ifndef CALM
                             } /* Otherwise, kill them! kill them all! */
#endif
                          }
                        }
                        /* We move to dx,dy now ? */
                        if (mstype==0&&(dx!=0||dy!=0)) passing=1;
                          /*check afterwards if we're closer to true target.
                          Only makes sense for tunnels, but I only put
                          this restriction in because ai_set_waypoint
                          doesn't want to be called from a doorway. */
                      } else {/* On the way to the intersection.
                      If this way is blocked, wait a bit, as blocking
                      monster is hopefully going to move next.*/
                        elem=(mapstart+FOOGRID_IDX(map,my+dy,mx+dx));
                        if(elem->Mptr!=NULL) {
#ifndef CALM
                           if(m_respect(mob,elem->Mptr)>=mob->Annoyance) {
                             mob->Annoyance++;
#endif
                             dx=0;dy=0; /*breathe deeply..*/
#ifndef CALM
                           } /* Else fight them */
#endif
                        }
                      }
                      break;
                 /* default: Now, case 1 is generalised instead.
                    show_messages("Haven't figured this one out!"); 
                      break; */

                 } /* End of switch block */
              }
              jumpout: ;
           }
           /*
        }
    } */
    if (1==do_open) { /* Open door rather than moving */
        m_open_door(mob,dx,dy);/* Existing function is different currently*/
    } else { /* Just move. Well, if possible */
        if (dx!=0 || dy!=0) move_mob(dx,dy,mob);
        /* move_mob() should have no bad effects if dx,dy walks
        the mob into a wall... But it would be better to have
        dx,dy adjusted to have mob walk around obstacles... */
        if (passing) ai_set_waypoint(mob,mstype,mob->Tx,mob->Ty,1);
    }
}

void ai_acquire_target(monster_t *mob,int stype)
{
    int mx=mob->X,my=mob->Y,incache=0;
    int target=0,total=0,count;
    long oldest=0;
    region_t *reg;
    int room;
    entrancelist_t *elist;
    entrancelist_t *escan;
    aicache_tunnelent_t *tc;
    aicache_rooment_t *rc;
    aicache_doorlist_t *dlist;
    aicache_doorlist_t *dscan;
    if (2==stype) { /* In a doorway!! */
        bugger(REPTYPE_AI,LOGLEVEL_FOOBAR,"AI tried to pick new path from a door.");
        crash(INCONSISTENT); /* SHOULD NOT happen. */
    }
    if (0==stype) { /* In a tunnel */
        reg=find_region(mx,my);
        /* Check cache for region */
        tc=aicache_regcache(mob,reg, 1); /* Accept buffer entries, which
          will have been validated by now */
        mob->Ttype=1; /* Aim for a room */
        /* If region in cache, return one of least recently visited doors*/
        if (tc!=NULL)  {incache=1;dlist=tc->Paths; }
        else elist=reg->Firstentrance;
    } else { /* In a room */
        room=find_room(mx,my);
        /* Check cache for room, accept buffer entries */
        rc=aicache_roomcache(mob,room,1);
        mob->Ttype=0; /* Aim for a tunnel */
        if (rc!=NULL) {incache=1;dlist=rc->Paths;}
        /* Shouldn't happen now 
        else elist=tls->Room[room].Firstentrance; */
        else crash(INCONSISTENT);
    }
    /* Now we work out what to do with all that */
    if (incache) {
       /* Pick one at random from set of doors with highest
       age from doorlist */
       for(dscan=dlist;dscan!=NULL;dscan=dscan->Next)
         if (dscan->Thisdoor->Age>oldest) oldest=dscan->Thisdoor->Age;

       /* Now count how many have that age */
       for(dscan=dlist;dscan!=NULL;dscan=dscan->Next)
         if (dscan->Thisdoor->Age==oldest) total++;
       /* Now pick some random number from 0 to total */
       target=(total*roll_dX(d20set[0]))/20; /* Should give target<total */
       /* Now pick the target'th door that has oldest age! */
       dscan=dlist;
       if (target>0) {
         for(count=0;count+=(dscan->Thisdoor->Age==oldest), count<=target;dscan=dscan->Next);
         /* Odd stuff. Is it right?? Possibly noott..
         OW! It wasn't. Now changed, I'm sure it's OK*/
       } else { /* DOH! If target is 0, dscan is NOT dlist, as
         dlist->Age might NOT be oldest!!! */
         for(;;dscan=dscan->Next)
         {
            if(dscan->Thisdoor->Age==oldest) break;
            /* Should really be merged back to the general case. */
         }
       }
       if (dscan==NULL) crash(FUNCTION_ERROR); /* Shouldn't happen, but..*/
       mob->Tx=dscan->Thisdoor->Xcoord;
       mob->Ty=dscan->Thisdoor->Ycoord;
    } else {
       /* Pick one arbitrarily from entlist */
       /* Count them first */
       for(escan=elist;escan!=NULL;escan=escan->Next,total++);
       /* Now pick random number 0 to total-1 */
       target=(total*roll_dX(d20set[0]))/20; /*Should give target<total */
       /* Now just pick the target'th door of the list! */
       for(escan=elist,count=0;count<target;count++,escan=escan->Next);
       /* Is that right?? Looks odd */
       mob->Tx=escan->Entrance->Xcoord; mob->Ty=escan->Entrance->Ycoord;
    }
    /* Cache should hopefully get updated outside here...
    because we're not doing it here! */
}

void ai_target_last_entrance(monster_t *mob,region_t *reg)
{
    int newest=10000;
    aicache_doorlist_t *dlist;
    aicache_doorlist_t *dscan;
    aicache_tunnelent_t *tc;
    if(reg==NULL) crash(INCONSISTENT);
    /* Find cache element for this region */
    tc=aicache_regcache(mob,reg, 1);
    mob->Ttype=1;
    if (tc==NULL) crash(INCONSISTENT); /* Shouldnt happen */
   /* incache=1; */
    dlist=tc->Paths;

    /* Find entrance with LOWEST timestamp, which is
    the one we passed through last. */
    for(dscan=dlist;dscan!=NULL;dscan=dscan->Next)
      if (dscan->Thisdoor->Age<newest) newest=dscan->Thisdoor->Age;

    for(dscan=dlist;dscan!=NULL;dscan=dscan->Next)
      if (dscan->Thisdoor->Age==newest) {
        mob->Tx=dscan->Thisdoor->Xcoord;
        mob->Ty=dscan->Thisdoor->Ycoord;
        return;
      }
    crash(FUNCTION_ERROR);
}
