/* ==== ajoin.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h"

/* protos */
#include "makemap_p.h"
#include "memutils.h"
#include "mg_utils.h"
#ifdef MARK_INTERSECTIONS
#include "item_p.h"
#endif
#include "crash.h"
#include "dXrng.h"
#include "ui.h"
#include "ajoin.h" /* Needs to be put elsewhere */

/* extern roomdata_t room[]; REPLACING WITH tls->Room[] */

/* makes more sense than using a function */
#define CLASSIFY_RPOS(sx1,sy1,sx2,sy2,tx1,ty1,tx2,ty2) ( (sy2-1<ty1+1 || ty2-1<sy1+1)? ( (sx2-1<tx1+1 || tx2-1<sx1+1)? 0:1) :2)

/* When going from a tunnel to a room */
#define CLASSIFY_TRPOS(sx,sy, tx1,ty1, tx2,ty2) ( (1*(sx>tx1&&sx<tx2))+ (2*(sy>ty1&&sy<ty2))+ (3*(sx==tx1||sx==tx2))+ (4*(sy==ty1||sy==ty2)))
/* if sx==tx1 or sx==tx2, give 3.
   if sy==ty1 or sy==ty2, give 4.
   if (sx>sx1 && sx<sx2)  give 1
   if (sy>sx1 && sy<sy2)  give 2. Else give 0. */
/* The converse (from a room to a tunnel) needs no special treatment:
   the previous sx,sy (from the room source) needs no changing, and
   the point found with a tunnel is an acceptable target.*/

/* We will have to be able to deal the fact that tunnels can join 2 areas
  by merely running *adjacent* to another tunnel.
  More disturbingly, can join MORE than 2 areas at once!!!
  Potentially 4 areas.
  Does it actually *MATTER* if we join more than 2? It merely makes it
  a bit more connected. The map is not going to become 100% tunnel
  just by this, and we wont have the problem of unaccessible areas.*/


void attempt_joining(int source,int target)
{
    /* starts trying to connect source to target.
       If it hits something else on the way, then will
       either try to connect source to it, or it to target.
       If a tunnel, connect by coords. If a room, connect
       by room value.*/
    int sx,sy; /* Points to try to send tunnel from*/
    int tx,ty; /* Points to try to send tunnel to*/
    int mx,my; /* Points to try to place corner at*/
    int px,py,p; /* temp vars used in probing */
    int stype=2; /* 0 if source is tunnel, 1 if source is room, 2 if new room*/
    int ttype=2; /* 0 if target is tunnel, 1 if target is room, 2 if new room*/
    int sx1,sx2,sy1,sy2; /* Corners of source room*/
    int tx1,tx2,ty1,ty2; /* Corners of target room*/
    int xstart,xend,ystart,yend; /*constraints for x and y values of tunnels*/
    direction_t dir; /* a temporary variable used in probing.*/
    direction_t dir1,dir2; /* Direction of tunnels used for type 0 paths*/
    int pathstate=0; /* 1 indicates we've scanned sx,sy->mx,my, and have
      begun scanning mx,my->tx,ty */
    area_t *source_area=tls->Room[source].Area; /*source_area never changes*/
#ifndef PLUNK_PATHS
    area_t *target_area=tls->Room[target].Area; /*target_area may change*/
#endif
    area_t *probe_area;
    region_t *source_region=NULL; /* Rooms don't have regions. */
    region_t *target_region=NULL;
#ifndef PLUNK_PATHS
    region_t *tun_region; /* The region that new tunnel gets put in */
#endif
    int relat;
    int strike=0; /*When we've found a clear path from somewhere
        to somewhere else, strike=1*/

    if(target==-1) {show_messages("Target=-1!");crash(OUT_OF_BOUNDS);}
    if(source==-1) {show_messages("Source=-1!");crash(OUT_OF_BOUNDS);}
    if (tls->Room[source].Xcentre==-1){
      sprintf(strbuf,"Room %d doesnt exist!",source);
      show_messages(strbuf);
      crash(INCONSISTENT);
    }
    if (tls->Room[target].Xcentre==-1){
      sprintf(strbuf,"Room %d doesnt exist!",target);
      show_messages(strbuf);
      crash(INCONSISTENT);
    }
    while(0==strike)
    {
        if(2==stype)
        {
           sx1=tls->Room[source].Xstart; sx2=sx1+(tls->Room[source].Width)-1;
           sy1=tls->Room[source].Ystart; sy2=sy1+(tls->Room[source].Height)-1;
        }
        if(2==ttype)
        {
           tx1=tls->Room[target].Xstart; tx2=tx1+(tls->Room[target].Width)-1;
           ty1=tls->Room[target].Ystart; ty2=ty1+(tls->Room[target].Height)-1;
        }

        if(3<=(stype+ttype))
        { /* Plan course from new room to some other room */
            relat=CLASSIFY_RPOS(sx1,sy1,sx2,sy2, tx1,ty1, tx2,ty2);
            switch (relat) {
               case 0:
                 /* We need a corner(mx,my) and 2 tunnels to it*/
                 if(roll_dX(d20set[0])<10)
                 {/* Move horizontal first, then vertical*/
                    dir1=(sx1<tx1)?FACE_E:FACE_W;
                    dir2=(sy1<ty1)?FACE_S:FACE_N;
                    sx=(FACE_W==dir1)?sx1:sx2;
                    ty=(FACE_N==dir2)?ty2:ty1;
                    xstart=tx1+1;xend=tx2-1;
                    ystart=sy1+1;yend=sy2-1;
                    sy=ystart+((yend-ystart)*roll_dX(d20set[0])/19);
                    tx=xstart+((xend-xstart)*roll_dX(d20set[0])/19);
                    mx=tx;   my=sy;
                    pathstate=0;
                 } else { /* Move vertical first, then horizontal*/
                    dir1=(sy1<ty1)?FACE_S:FACE_N;
                    dir2=(sx1<tx1)?FACE_E:FACE_W;
                    sy=(FACE_N==dir1)?sy1:sy2;
                    tx=(FACE_W==dir2)?tx2:tx1;
                    xstart=sx1+1;xend=sx2-1;
                    ystart=ty1+1;yend=ty2-1;
                    sx=xstart+((xend-xstart)*roll_dX(d20set[0])/19);
                    ty=ystart+((yend-ystart)*roll_dX(d20set[0])/19);
                    mx=sx;  my=ty;
                    pathstate=0;
                 }
                 break;
               case 1:
                 /* source is n or s of target*/
                 xstart=MAX(sx1,tx1)+1;xend=MIN(sx2,tx2)-1;
                 sx=xstart+((xend-xstart)*roll_dX(d20set[0])/19);/*Somewhere between*/
                 /* We ought to make sure that sx,sy is not *next to*
                 an existing door. Can be ON one tho.
                 Ditto tx,ty! Is this possible? OHO! If tx,ty is a door,
                 there will be a tunnel already coming in the opposite
                 direction, and our new tunnel will meet this and connect
                 to it BEFORE we reach our target!!!
                 Furthermore, I realise now, that if sx,sy is next to
                 a door, the tunnel from that will run next to where
                 our new tunnel wants to, so we will actually start the
                 tunnel where the previous one ended!*/
                 tx=sx;
                 dir=(sy1<ty1)?FACE_S:FACE_N;
                 sy=(FACE_N==dir)?sy1:sy2;
                 ty=(FACE_N==dir)?ty2:ty1;
                 break;
               case 2:
                 /* source is w or e of target*/
                 ystart=MAX(sy1,ty1)+1;yend=MIN(sy2,ty2)-1;
                 sy=ystart+((yend-ystart)*roll_dX(d20set[0])/19);
                 ty=sy;
                 dir=(sx1<tx1)?FACE_E:FACE_W;
                 sx=(FACE_W==dir)?sx1:sx2;
                 tx=(FACE_W==dir)?tx2:tx1;
                 break;
            }
        }
        if (ttype==2 &&!stype)
        { /* Working out a path from part of a tunnel to a new room */
            relat=CLASSIFY_TRPOS(sx,sy, tx1,ty1,tx2,ty2);
            if (!relat) relat=(roll_dX(d20set[0])<10)?3:4;
            switch (relat) {
               case 1:
                 /* source n or s of target */
                 dir=(sy<ty1)?FACE_S:FACE_N;
                 ty=(FACE_N==dir)?ty2:ty1;
                 tx=sx; /*Oh. That's it??*/
                 break;
               case 2:
                 /* source w or e of target */
                 dir=(sx<tx1)?FACE_E:FACE_W;
                 tx=(FACE_W==dir)?tx2:tx1;
                 ty=sy;
                 break;
               case 3:
                 /* Must go H before V to target */
                 /* Pick a point on target wall */
                 xstart=tx1+1;xend=tx2-1;
                 tx=xstart+((xend-xstart)*roll_dX(d20set[0])/19);
                 dir1=(sx<tx)?FACE_E:FACE_W;
                 dir2=(sy<ty1)?FACE_S:FACE_N;
                 ty=(FACE_N==dir2)?ty2:ty1;
                 mx=tx;   my=sy;
                 pathstate=0;
                 break;
               case 4:
                 /* Must go V before H to target */
                 /* Pick a point on target wall */
                 ystart=ty1+1;yend=ty2-1;
                 ty=ystart+((yend-ystart)*roll_dX(d20set[0])/19);
                 dir1=(sy<ty)?FACE_S:FACE_N;
                 dir2=(sx<tx1)?FACE_E:FACE_W;
                 tx=(FACE_W==dir2)?tx2:tx1;
                 mx=sx;  my=ty;
                 pathstate=0;
                 break;
            }

        }
        if (stype) stype=1;
        if (ttype) ttype=1;
        /* Now we know *where* we want to look, we look! */
        switch (relat){
           case 3:
           case 4:
           case 0: /* For probing+building, 0,3,4 are equivalent.*/
             /* We need a corner (mx,my), and 2 tunnels to it*/
             /* probe direction 1 first. If probe goes past mx,my,
             then change to phase 2.
             If phase 2, probe direction 2. If probe goes past tx,ty,
             strike!*/
             dir=(pathstate)?dir2:dir1;
             px=(pathstate)?mx:sx;
             py=(pathstate)?my:sy;
             p=probe_path(px,py,dir);
             px=(pathstate)?tx:mx;
             py=(pathstate)?ty:my;
             if (target_reached(p,px,py,dir,!pathstate))
             { /* reached whichever our target was */
                if (pathstate) strike=1; /* We have a total path!*/
                else pathstate=1; /* Check the next half of path */
             } else {/* hit an obstacle.*/
                /* If it was already reachable, update stype,
                then "source" if a room, or "sx,sy" if a tunnel.
                pathstate, relat, etc SHOULD update themselves
                next iteration.
                If NOT yet reachable, we try to connect to it. By, er...*/
                if(dir==FACE_N||dir==FACE_S) /* As case 1*/
                { if(probe_status(px,p,dir)==0)
                  { p=find_room(px,p);
                    if (p==-1) {show_messages("find_room=-1!");crash(NO_MEM);}
                    if(tls->Room[p].Area==source_area)
                    { source=p;stype=2; /* We don't change pathstate,etc*/
                    } else { /* as they will get altered by new room*/
                      target=p;ttype=2; /* being discovered*/
                    }
                  } else { /* We found tunnels */
                    probe_area=get_area(px,p,source_area);
                    if (probe_area==source_area)
                    { sy=p;stype=0; /* Source is tunnel */
                      /* Set source_region */
                      source_region=get_region(px,p,NULL); /* What should
                          we have set as "this"? */
                      if (pathstate) {pathstate=0;sx=px;relat=1;}
                    } else {
                       /* set target_region */
                       target_region=get_region(px,p,NULL); /*
                        What should we have set as "this"? */
                       ty=p;ttype=0; /* Ah. Now what sort of path is it?
                         For tunnels, we *don't* make a further pass!*/
                       if (pathstate)
                        strike=1; else{relat=1;tx=sx;strike=1;}
                    }
                  }
                } else { /* Facing w or e. As case 2*/
                  if(probe_status(p,py,dir)==0)
                  { p=find_room(p,py);
                    if(p==-1){show_messages("find_room=-1!");crash(NO_MEM);}
                    if(tls->Room[p].Area==source_area)
                    { source=p;stype=2;
                    } else {
                      target=p;ttype=2;
                    }
                  } else { /* found tunnels */
                    probe_area=get_area(p,py,source_area);
                    if (probe_area==source_area)
                    {  sx=p;stype=0;
                       /* set source_region */
                       source_region=get_region(p,py,NULL);
                       if(pathstate) {pathstate=0;sy=py;relat=2;}
                    } else {
                       tx=p; ttype=0;
                       /* get target_region */
                       target_region=get_region(p,py,NULL);
                       if (pathstate)
                        strike=1;else {relat=1;ty=sy;strike=1;}
                    }
                  }
                }
             }
             break;
           case 1: /* source is n or s of target */
             p=probe_path(sx,sy,dir);
             if(target_reached(p,tx,ty,dir,0))
             {  strike=1; /* yay!*/
             } else{
                if(probe_status(sx,p,dir)==0)
                {  p=find_room(sx,p);
                   if(p==-1){show_messages("find_room=-1!");crash(NO_MEM);}
                   if(tls->Room[p].Area==source_area)
                   {  source=p;
                      stype=2;/* New source room*/
                   } else {
                      target=p;
                      ttype=2;/* Next iteration will get the proper tx,ty*/
                   }
                } else { /* We found tunnels around */
                   probe_area=get_area(sx,p,source_area);
                   if (probe_area==source_area)
                   {  sy=p;stype=0; /* Source is tunnel */
                      /* get source_region */
                      source_region=get_region(sx,p,NULL);
                   } else {
                      /* get target_region */
                      target_region=get_region(sx,p,NULL);
                      ty=p;ttype=0;strike=1;
                   }
                }
             }
             break;
           case 2: /* source is w or e of target */
             /* Probe dir . If probe goes past tx,ty, strike!*/
             p=probe_path(sx,sy,dir);
             if(target_reached(p,tx,ty,dir,0))
             {  strike=1;
             } else {
                if(probe_status(p,sy,dir)==0)
                {  p=find_room(p,sy);
                   if(p==-1){show_messages("find_room=-1!");crash(NO_MEM);}
                   if(tls->Room[p].Area==source_area)
                   {  source=p;
                      stype=2;/* New source room*/
                   } else {
                      target=p;
                      ttype=2;
                   }
                } else { /* found tunnels */
                   probe_area=get_area(p,sy,source_area);
                   if(probe_area==source_area)
                   {   sx=p;stype=0;
                       /* get source_region */
                       source_region=get_region(p,sy,NULL);
                   } else {
                       /* get target_region */
                       target_region=get_region(p,sy,NULL);
                       tx=p;ttype=0;strike=1;
                   }
                }
             }
             break;
        }

    }
/* == we have identified source *and* target, each of which may =====
=====         be either a room, or a point in a tunnel          == */
#ifndef PLUNK_PATHS
    if (ttype)target_area=tls->Room[target].Area;
    else target_area=get_area(tx,ty,source_area);
#endif
    /* We should instead have this done by plunk_path1() and plunk_path2() */


#ifndef PLUNK_PATHS
    if ((stype)&&(ttype)) { /* Two rooms. Create new region. */
       tun_region=(region_t *)counting_malloc(MEM_REGION,sizeof(region_t));
       /* We also want to fill in its details. */
       tun_region->Area=source_area; /* Could also be target_area?? <-!->*/
       tun_region->Firsttunnel=NULL; /* Will get filled in... */
       tun_region->Firstentrance=NULL; /* Ditto. */
       tun_region->Intersections=NULL;
       /* If we have ANY sort of list of regions, we'd want more here. */
       tun_region->Nextregion=NULL; /* That'll do! */
       /* NYEARGHH No it won't. */
       tun_region->Nextregion=source_area->Firstregion;
       source_area->Firstregion=tun_region;
       /* ...
          ...
          ...
          */
    } else if (!stype) {
       /* The source is a tunnel. What of target? */
       if (!ttype) { /* Source AND target are tunnels, so ... */
         /* Main ramification of this is that AFTER creating tunnel,
          we do merge_regions(). For now, we just have tun_region=source_reg*/
         tun_region=source_region; /* In short, doesn't matter about ttype yet*/
       } else { /* Target is room. */
         tun_region=source_region;
       }
    } else {
       /* The source is a room.Target cannot be (would have been caught)*/
       tun_region=target_region;
    }
    /* Region for tunnel established */
#endif

    /* Now build the tunnels. Should use plunk_pathN() */

    if(relat>0&&relat<3) /*3 and 4 are now as 0!!*/
    { /* 1 straight path */
         /* Now we pass the region instead, and we must
         decide whether to create the region. If we do so,
         we set region->Area=source_area */
         /* If stype and ttype, then we're joining 2 rooms
         directly, which means we're creating one new region.
         If we're joining a room to a tunnel, the region is the
         same as the tunnel's region. But the code depends
         on whether the tunnel is source or target.
         If both source AND target are tunnels, then we simply
         use the source region, and then do merge_regions(). */
         /* Was using source_area. Now tun_region. */
#ifdef PLUNK_PATHS
         plunk_path1(sx,sy,tx,ty,dir,stype,ttype,source_region,source_area);
#else
/*       if ( attempt_tunext(&sx,&sy,&tx,&ty)) {
           sprintf(strbuf,"Straight tunnel extended, src=%d, targ=%d.",sorig,torig);
           bugger(REPTYPE_MAPGEN,LOGLEVEL_WARNING,strbuf);
         }    Actually, this turned out to be dire */
         build_tunnel(tun_region,sx,sy,tx,ty);
#endif
    } else { /* 2 tunnels joined with a corner at (mx,my)*/
#ifdef PLUNK_PATHS
         plunk_path2(sx,sy,mx,my,tx,ty,dir1,dir2,stype,ttype,source_region,source_area);
#else
         /* if (attempt_tunext(&sx,&sy,&mx,&my)) {
           sprintf(strbuf,"JOINED tunnel extended! EEEK! src=%d, targ=%d.",sorig,torig);
           bugger(REPTYPE_MAPGEN,LOGLEVEL_WARNING,strbuf);
         }
         if (attempt_tunext(&mx,&my,&tx,&ty)) {

           sprintf(strbuf,"JOINED tunnel extended! EEEK! src=%d, targ=%d.",sorig,torig);
           bugger(REPTYPE_MAPGEN,LOGLEVEL_WARNING,strbuf);
         }  This was even worse here... */
         build_tunnel(tun_region,sx,sy,mx,my);
         add_intersection(mx,my,tun_region);
         build_tunnel(tun_region,mx,my,tx,ty);
#endif
    }
    /* NOW, if stype and ttype are tunnels, we do merge_regions().
       But would we want to then not call merge_areas()- IE, do
       we want merge_regions to incorporate its own
       implementation of merge_areas? */
    /* I think this should be done by plunk_paths() */
#ifndef PLUNK_PATHS
    if(stype) {build_door(sx,sy,source,tun_region);}/* build a door at sx,sy*/
    else {add_intersection(sx,sy,tun_region);}/* Note an intersection */
    if(ttype) {build_door(tx,ty,target,tun_region);}/* build a door at tx,ty*/
    else {add_intersection(tx,ty,tun_region);}/* Note an intersection */
    /* If we leave a tunnel tile where a door would be, it'll confuse
      functions that classify squares. But will it matter? Will
      anything find that square? Oh yeah, it could. In which case, lets
      have the option of placing a room tile there? */
#endif

    /* I think this should be done by plunk_paths... */
#ifndef PLUNK_PATHS
    merge_areas(source_area,target_area);
    if((!stype)&&(!ttype)) merge_regions(source_region,target_region); /*That's OK, yes? */
#endif

    /* If we have a type 0 relation (needs a corner), then source
      sends a tunnel to somewhere (mx,my) parallel with target.
      If it hits something already connected to source, tries to 
      send tunnel from that something to (mx,my) and continue.
      If it hits something not yet connected to source, send
      tunnel to that something and return.
      If it hits nothing, then try searching from (mx,my) to
      target. If it hits something already connected to source,
      then DONT send anything from (sx,sy) to (mx,my), but try
      to send tunnel from (something) to target.
      So, once we find something between (mx,my) and (tx,ty),
      if it is in source, we change relat to 1 or 2 accordingly
      and set sx=mx and sy=my. DOH! No, set sx,sy to obstacle.
      */
    /* End of function!*/
}

#define LOOK_FOR_TCELL(tilevar,x,y) tilevar=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype; if(tilevar==PTL_TUNNEL||tilevar==PTL_DOOR||tilevar==PTL_ODOOR)

void plunk_path1(int sx, int sy, int tx, int ty, direction_t dir, int stype,
  int ttype,region_t *source_region,area_t *source_area)
{
    /* This (and its sibling function) will be pretty painful too */
    int soverlap,toverlap; /* Do the end points already overlap? */
    int sinline=0,tinline=0; /* has the tunnel been extended either end
                                 due to an in-line tunnel cell? */
    int six=sx,siy=sy,tix=tx,tiy=ty;  /* coords of source+target taking
      in-line tunnel cells into account; used for build_tunnel() */
    int sXtunnel=0; /* Number of extra cells for any crossing tunnel at
       source end (to join main tunnel with any side-tunnel cells) */
    int sXt1x=sx,sXt1y=sy,sXt2x=sx,sXt2y=sy; /* Coords for ends of
       any crossing tunnel at source end. */
    int tXtunnel=0; /* Number of extra cells for crossing tunnel (target end)*/
    int tXt1x=tx,tXt1y=ty,tXt2x=tx,tXt2y=ty; /* Coords for ends of any
       crossing tunnel at target end. */
    area_t /* *source_area=NULL,*/ *target_area=NULL;
    region_t /* *source_region=NULL,*/ *target_region=NULL;
    /* I now think we should work out tun_region ourselves */
    region_t *tun_region;
    maploc_t *mapstart=map->Data;
    tile_t stile,ttile;
    stile=(mapstart+FOOGRID_IDX(map,sy,sx))->Tiletype;
    ttile=(mapstart+FOOGRID_IDX(map,ty,tx))->Tiletype;
    soverlap=(stype||stile==PTL_TUNNEL||stile==PTL_DOOR||stile==PTL_ODOOR);
    toverlap=(ttype||ttile==PTL_TUNNEL||ttile==PTL_DOOR||ttile==PTL_ODOOR);
    if (soverlap) {
       /* These should be passed by args?
       if (stype) source_area=tls->Room[find_room(sx,sy)].Area;
       else {
         source_region=find_region(sx,sy);
         source_area=source_region->Area;
       } */
    } else {
       /* No overlap. Check if the/a tunnel to join to is behind
       the start of the tunnel (an in-line tunnel cell). */
       switch(dir) { /* side tunnels found in *next* switch statement */
         case FACE_N:
            LOOK_FOR_TCELL(stile,sx,sy+1) {
               sinline=1;
               siy=sy+1; /* Extend */
            }
            break;
         case FACE_E:
            LOOK_FOR_TCELL(stile,sx-1,sy) {
               sinline=1;
               six=sx-1;
            }
            break;
         case FACE_S:
            LOOK_FOR_TCELL(stile,sx,sy-1) {
               sinline=1;
               siy=sy-1;
            }
            break;
         case FACE_W:
            LOOK_FOR_TCELL(stile,sx+1,sy) {
               sinline=1;
               six=sx+1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       switch(dir) { /* NOW check for side tunnels */
         case FACE_N:
         case FACE_S:
            LOOK_FOR_TCELL(stile,sx-1,sy) {
               sXtunnel++;
               sXt1x=sx-1;
            }
            LOOK_FOR_TCELL(stile,sx+1,sy) {
               sXtunnel++;
               sXt2x=sx+1;
            }
            break;
         case FACE_W:
         case FACE_E:
            /* stuff */
            LOOK_FOR_TCELL(stile,sx,sy-1) {
               sXtunnel++;
               sXt1y=sy-1;
            }
            LOOK_FOR_TCELL(stile,sx,sy+1) {
               sXtunnel++;
               sXt2y=sy+1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       /*
       if (sinline) {   we can get new point as source area/region 
          source_region=find_region(sx,sy);
          source_area=source_region->Area;
       } else {
           malloc one? or not?
       } */
    }
    if (toverlap) {
       /* We should indeed be getting these ourselves, IMO */
       if (ttype) target_area=tls->Room[find_room(tx,ty)].Area;
       else {
         target_region=find_region(tx,ty);
         target_area=target_region->Area;
       }
    } else {
       /* Do these things happen though? I thought not...
       IIRC, the target area would only connect to the sides
       (or by direct overlap)... No, whilst writing out
       NOTES/Tunnels3, I realised target could be caught by
       a side tunnel, and have an in-line tunnel as well. */
       switch(dir) { /* Get side tunnels next switch statement */
         case FACE_N:
            LOOK_FOR_TCELL(ttile,tx,ty-1) {
               tinline=1;
               tiy=ty-1; /* Extend */
            }
            break;
         case FACE_E:
            LOOK_FOR_TCELL(ttile,tx+1,ty) {
               tinline=1;
               tix=tx+1;
            }
            break;
         case FACE_S:
            LOOK_FOR_TCELL(ttile,tx,ty+1) {
               tinline=1;
               tiy=ty+1;
            }
            break;
         case FACE_W:
            LOOK_FOR_TCELL(ttile,tx-1,ty) {
               tinline=1;
               tix=tx-1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       if(tinline) {
         target_region=find_region(tix,tiy);
         target_area=target_region->Area;
       }
       switch(dir) {
         case FACE_N:
         case FACE_S:
            /* stuff */
            LOOK_FOR_TCELL(ttile,tx-1,ty) {
               tXtunnel++;
               tXt1x=tx-1;
            }
            LOOK_FOR_TCELL(ttile,tx+1,ty) {
               tXtunnel++;
               tXt2x=tx+1;
            }
            break;
         case FACE_W:
         case FACE_E:
            /* stuff */
            LOOK_FOR_TCELL(ttile,tx,ty-1) {
               tXtunnel++;
               tXt1y=ty-1;
            }
            LOOK_FOR_TCELL(ttile,tx,ty+1) {
               tXtunnel++;
               tXt2y=ty+1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       if (tXtunnel) { /* We can use one or the other of these */
          target_region=find_region(tXt1x,tXt1y);
          if(find_region(tXt2x,tXt2y)!=NULL) target_region=find_region(tXt2x,tXt2y);
          target_area=target_region->Area;
       }
       if (tinline) { /* Can use new point for area details. */
         sprintf(strbuf,"plunk_paths(): managed to extend target end of a tunnel at %d,%d",tx,ty);
         bugger(REPTYPE_MAPGEN,LOGLEVEL_WARNING,strbuf);
         /* Perhaps get these from get_region()?
         target_region=find_region(tx,ty);
         target_area=target_region->Area; */
       } else {
         /* dont know */
       }
    }
    /* Get tun_region */
    if (stype&&ttype) { /* 2 rooms, so we must create it */
       tun_region=(region_t *)counting_malloc(MEM_REGION,sizeof(region_t));
       tun_region->Area=source_area; /* must already have this! */
       tun_region->Nextregion=source_area->Firstregion;
       source_area->Firstregion=tun_region;
       /* blanks for the rest */
       tun_region->Firsttunnel=NULL;
       tun_region->Firstentrance=NULL;
       tun_region->Intersections=NULL;
    } else if (!stype) { /* Source is tunnel. Target is ??? */
       /* if (!ttype) ...either way, we could do the same thing! */
       tun_region=source_region;
    } else { /* source must be room, but target can't be */
       tun_region=target_region;
    }

    /* Having got that info, we can build the tunnel! */
    build_tunnel(tun_region,six,siy,tix,tiy); /* We could also sort the
    tunnels to face the "nice" way round based on dir. */

    if(sXtunnel) {
      build_tunnel(tun_region,sXt1x,sXt1y,sXt2x,sXt2y);
      /* Also the intersections for it */
      add_intersection(sXt1x,sXt1y,tun_region);
      add_intersection(sXt2x,sXt2y,tun_region);
    } /* if sXtunnel and tXtunnel were to overlap (would happen if
      source&target the same), it shouldn't matter; even the AI should
      be able to deal with it, as the algorithm remembers tunnels
      it's checked already, so there shouldn't be infinite loops */
    if(tXtunnel) {
      build_tunnel(tun_region,tXt1x,tXt1y,tXt2x,tXt2y);
      /* Plus intersections */
      add_intersection(tXt1x,tXt1y,tun_region);
      add_intersection(tXt2x,tXt2y,tun_region);
    }

    /* Now, work out if we (still) have to place intersections at
     sx,sy and tx,ty. In some cases, there shouldn't be any there
     at all! */

    if(!stype) { /* source is tunnel */
       add_intersection(six,siy,tun_region);
       if(sinline &&sXtunnel==2) add_intersection(sx,sy,tun_region);
    } else {
       build_door(sx,sy,find_room(sx,sy),tun_region); /* doors must be at the exact coords */
    }
    if(!ttype) { /* target is tunnel */
       add_intersection(tix,tiy,tun_region);
       if(tinline &&tXtunnel==2) add_intersection(tx,ty,tun_region);
    } else {
       build_door(tx,ty,find_room(tx,ty),tun_region);
    }
    /* If !soverlap, must do list merges */
    merge_areas(source_area,target_area);
    if((!stype)&&(!ttype)) merge_regions(source_region,target_region);

    /* If soverlap, then we have nothing to do for extending source end
    of tunnel or adding any extra intersections there.
    If toverlap, then we have nothing to do for extending target end
    of tunnel or adding any extra intersections there.
    Otherwise, it is *guaranteed* that we will have to(?) */

    /* Never seemed to actually need to merge more than one area/region
    at once. But... We'd need the tunnel pointers etc still intact
    to get them for further merging. So I think this has to be done
    before crossing tunnels are placed... and if there's in-line tunnels,
    should be done before the main tunnel is built...
    Meh, my brain hurts now */


    if(!soverlap) { /* WOOP!WOOP! There must have been tunnel squares
    around sx,sy */
       sprintf(strbuf,"plunk_path1() had to play about to join up source end of tunnel at %d,%d.",sx,sy);
       bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,strbuf);
       /* Now use   dir  to work out which cells have tunnels to
       connect to */
    } /* else... do nothing? */
    if(!toverlap) {
       sprintf(strbuf,"plunk_path1() had to play about to join up target end of tunnel at %d,%d.",tx,ty);
       bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,strbuf);
       /* and the rest */
    }
    /* We should also do merge_areas() and that for where soverlap, etc,
    were true... */
}

void plunk_path2(int sx,int sy,int mx,int my,int tx,int ty,direction_t dir1,
  direction_t dir2, int stype, int ttype, region_t *source_region,area_t *source_area)
{
    /* The sibling function to plunk_path1(). Called for tunnels that
    have a corner, whilst plunk_path1() is for straight-line tunnels. */
    int soverlap,toverlap; /* Do the end points already overlap? */
    int sinline=0,tinline=0;
    int six=sx,siy=sy,tix=tx,tiy=ty;
    int sXtunnel=0;
    int sXt1x=sx,sXt1y=sy,sXt2x=sx,sXt2y=sy;
    int tXtunnel=0;
    int tXt1x=tx,tXt1y=ty,tXt2x=tx,tXt2y=ty;
    area_t /* *source_area=NULL,*/ *target_area=NULL;
    region_t /* *source_region=NULL,*/ *target_region=NULL;
    region_t *tun_region;
    maploc_t *mapstart=map->Data;
    tile_t stile,ttile;
    stile=(mapstart+FOOGRID_IDX(map,sy,sx))->Tiletype;
    ttile=(mapstart+FOOGRID_IDX(map,ty,tx))->Tiletype;
    soverlap=(stype||stile==PTL_TUNNEL||stile==PTL_DOOR||stile==PTL_ODOOR);
    toverlap=(ttype||ttile==PTL_TUNNEL||ttile==PTL_DOOR||ttile==PTL_ODOOR);
    if (soverlap) {
    /* snipped */
    } else {
       /* No overlap. Check if the/a tunnel to join to is behind
       the start of the tunnel (an in-line tunnel cell). */
       switch(dir1) {
         case FACE_N:
            LOOK_FOR_TCELL(stile,sx,sy+1) {
               sinline=1;
               siy=sy+1;
            }
            break;
         case FACE_E:
            LOOK_FOR_TCELL(stile,sx-1,sy) {
               sinline=1;
               six=sx-1;
            }
            break;
         case FACE_S:
            LOOK_FOR_TCELL(stile,sx,sy-1) {
               sinline=1;
               siy=sy-1;
            }
            break;
         case FACE_W:
            LOOK_FOR_TCELL(stile,sx+1,sy) {
               sinline=1;
               six=sx+1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       switch(dir1) {
         case FACE_N:
         case FACE_S:
            LOOK_FOR_TCELL(stile,sx-1,sy) {
               sXtunnel++;
               sXt1x=sx-1;
            }
            LOOK_FOR_TCELL(stile,sx+1,sy) {
               sXtunnel++;
               sXt2x=sx+1;
            }
            break;
         case FACE_W:
         case FACE_E:
            LOOK_FOR_TCELL(stile,sx,sy-1) {
               sXtunnel++;
               sXt1y=sy-1;
            }
            LOOK_FOR_TCELL(stile,sx,sy+1) {
               sXtunnel++;
               sXt2y=sy+1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
    }
    if (toverlap) {
       /* We should indeed be getting these ourselves, IMO */
       if (ttype) target_area=tls->Room[find_room(tx,ty)].Area;
       else {
         target_region=find_region(tx,ty);
         target_area=target_region->Area;
       }
    } else {
       switch(dir2) {
         case FACE_N:
            LOOK_FOR_TCELL(ttile,tx,ty-1) {
               tinline=1;
               tiy=ty-1;
            }
            break;
         case FACE_E:
            LOOK_FOR_TCELL(ttile,tx+1,ty) {
               tinline=1;
               tix=tx+1;
            }
            break;
         case FACE_S:
            LOOK_FOR_TCELL(ttile,tx,ty+1) {
               tinline=1;
               tiy=ty+1;
            }
            break;
         case FACE_W:
            LOOK_FOR_TCELL(ttile,tx-1,ty) {
               tinline=1;
               tix=tx-1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       if(tinline) {
         target_region=find_region(tix,tiy);
         target_area=target_region->Area;
       }
       switch(dir2) {
         case FACE_N:
         case FACE_S:
            LOOK_FOR_TCELL(ttile,tx-1,ty) {
               tXtunnel++;
               tXt1x=tx-1;
            }
            LOOK_FOR_TCELL(ttile,tx+1,ty) {
               tXtunnel++;
               tXt2x=tx+1;
            }
            break;
         case FACE_W:
         case FACE_E:
            LOOK_FOR_TCELL(ttile,tx,ty-1) {
               tXtunnel++;
               tXt1y=ty-1;
            }
            LOOK_FOR_TCELL(ttile,tx,ty+1) {
               tXtunnel++;
               tXt2y=ty+1;
            }
            break;
         default:
            bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"plunk_paths(): tunnel direction was very wrong.");
            crash(OUT_OF_BOUNDS);
       }
       if(tXtunnel) { /* We can use one or the other of these */
          target_region=find_region(tXt1x,tXt1y);
          if(find_region(tXt2x,tXt2y)!=NULL) target_region=find_region(tXt2x,tXt2y);
          target_area=target_region->Area;
       }
    }
    /* Get tun_region */
    if (stype&&ttype) { /* 2 rooms, so we must create it */
       tun_region=(region_t *)counting_malloc(MEM_REGION,sizeof(region_t));
       tun_region->Area=source_area;
       tun_region->Nextregion=source_area->Firstregion;
       source_area->Firstregion=tun_region;
       tun_region->Firsttunnel=NULL;
       tun_region->Firstentrance=NULL;
       tun_region->Intersections=NULL;
    } else if (!stype) {
       tun_region=source_region;
    } else {
       tun_region=target_region;
    }

    /* Here's the main difference against plunk_path1()... */
    build_tunnel(tun_region,six,siy,mx,my);
    add_intersection(mx,my,tun_region);
    build_tunnel(tun_region,mx,my,tix,tiy);

    /* And the cross-tunnels */
    if(sXtunnel) {
      build_tunnel(tun_region,sXt1x,sXt1y,sXt2x,sXt2y);
      add_intersection(sXt1x,sXt1y,tun_region);
      add_intersection(sXt2x,sXt2y,tun_region);
    }
    if(tXtunnel) {
      build_tunnel(tun_region,tXt1x,tXt1y,tXt2x,tXt2y);
      add_intersection(tXt1x,tXt1y,tun_region);
      add_intersection(tXt2x,tXt2y,tun_region);
    }

    if(!stype) { /* source is a tunnel */
       add_intersection(six,siy,tun_region);
       if(sinline &&sXtunnel==2) add_intersection(sx,sy,tun_region);
    } else {
       build_door(sx,sy,find_room(sx,sy),tun_region);
    }
    if(!ttype) { /* target is tunnel */
       add_intersection(tix,tiy,tun_region);
       if(tinline &&tXtunnel==2) add_intersection(tx,ty,tun_region);
    } else {
       build_door(tx,ty,find_room(tx,ty),tun_region);
    }
    merge_areas(source_area,target_area);
    if((!stype)&&(!ttype)) merge_regions(source_region,target_region);

    /* left out the merging with extra tunnels bit */



    /* end of plunk_path2() */
}


int target_reached(int probe,int targetx,int targety,direction_t dir, int mp)
{
    /* If probe has got to or past targetx,targety,
    if midpoint, returns 1 if probe is PAST target.
    if not midpoint, returns 1 if probe has REACHED target. */
    if (targetx<2||targetx>=MAPWID-2||targety<2||targety>=MAPHGT-2)
    {
      sprintf(strbuf,"Targetting bad coords?");
      show_messages(strbuf);
      crash(OUT_OF_BOUNDS);
    }
    switch (dir) {
       case FACE_N:
         return ((probe<=(targety-mp))?1:0);
         break;
       case FACE_E:
         return ((probe>=(targetx+mp))?1:0);
         break;
       case FACE_S:
         return ((probe>=(targety+mp))?1:0);
         break;
       case FACE_W:
         return ((probe<=(targetx-mp))?1:0);
         break;
       default:
         crash(INCONSISTENT);
         break;
    }
    show_messages("Somehow reached the end of target_reached() function.");
    crash(FUNCTION_ERROR);
    return(0); /* Shut up, gcc... */
}

int probe_status(int x,int y, direction_t dir)
{
   /* When a probe returns, you don't know what it found to make it return.
   pass the returned coords (+ direction) to this to find out if it was:
   -A tunnel directly below the cursor (means nothing adjacent can be
   connected to a different area),
   -A room type square (will really mean either a perimeter wall, a door,
     or a corner, unless I've made a foobar)
   -Next to exactly one tunnel square
   -Next to more tunnels .
   What then?
   If a tunnel, would find its area ptr.
   If a room, would find its number
   If next to a tunnel, would find its area ptr.
   If next to more tunnels, would JOIN THEM first, then find the NEW area
   ptr for them. Well, if choosing to tunnel to/from there! If not, then...
   Aha! Could call a function to see if current area is one of those.
   Such a function could be used also for the other cases!*/
   /* 0=room. 1=tunnel. 2=adjacent tunnel. 3=adjacent tunnelS */
   maploc_t *mapstart=map->Data;
   tile_t ttile;
   int count=0;
   ttile=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
   if (ttile==PTL_CORNER||ttile==PTL_DOOR||ttile==PTL_VWALL||ttile==PTL_HWALL) return(0);
   if (ttile==PTL_FLOOR) return (0); /*This shoud NOT happen! But seems to*/
   if (ttile==PTL_TUNNEL) return(1);
   switch(dir) {
      case FACE_N:
      case FACE_S:
         ttile=(mapstart+FOOGRID_IDX(map,y,x-1))->Tiletype;
         if(ttile==PTL_TUNNEL) count=1;
         ttile=(mapstart+FOOGRID_IDX(map,y,x+1))->Tiletype;
         if(ttile==PTL_TUNNEL) count++;
         return(count);
      case FACE_W:
      case FACE_E:
         ttile=(mapstart+FOOGRID_IDX(map,y-1,x))->Tiletype;
         if(ttile==PTL_TUNNEL) count=1;
         ttile=(mapstart+FOOGRID_IDX(map,y+1,x))->Tiletype;
         if(ttile==PTL_TUNNEL) count++;
         return(count);
      default:
         crash(INCONSISTENT);
         break;
   }
   show_messages("Somehow reached the end of probe_status() function.");
   crash(FUNCTION_ERROR);
   return(0); /* Shut up again, gcc! */
}

/* if either (x,y) or at least one of its orthogonal neighbours has a
   different area to this, return such area. Otherwise, return this
   if any of them has this area, or NULL if no areas found. */
/* Why? Because if we find a point bordered by an already connected
   area and a not-yet-connected area, then we would be better off
   building a tunnel TO it, not from it. Trying to build from it
   would not automatically link the areas (as far as the algorithm's
   concerned)*/
area_t *get_area(int x,int y,area_t *this)
{
   /* Search for other. */
/*   tile_t n,s,w,e;  The tiles around the centre. Which aren't checked.*/
   tile_t centre;
   region_t *nr;
   region_t *sr;
   region_t *wr;
   region_t *er;
   maploc_t *mapstart=map->Data;
   if (y<3||x<3||x>=MAPWID-2||y>=MAPHGT-2)
   {
      show_messages("Get_area for dubious coords!");
      crash(OUT_OF_BOUNDS);
   }
   centre=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
   if (centre!=PTL_ROCK)
   {
   /* We've found *something* on the target square. Anything
     else around must be connected to it.*/
      if (centre==PTL_TUNNEL) return(find_region(x,y)->Area); /* Works for tunnels*/
      else return (tls->Room[find_room(x,y)].Area); /* Works for rooms!*/
   }
   /* NOW we're sure that there could be something adjacent that's
      not yet connected. find_area now doesn't consider rooms, thank
      goodness. */
   nr=(find_region(x,y-1)); /*Hmm. This got a lot more messy since regions..*/
   sr=(find_region(x,y+1));
   wr=(find_region(x-1,y));
   er=(find_region(x+1,y));
   if(NULL!=nr&&this!=nr->Area) return(nr->Area);
   if(NULL!=sr&&this!=sr->Area) return(sr->Area);
   if(NULL!=wr&&this!=wr->Area) return(wr->Area);
   if(NULL!=er&&this!=er->Area) return(er->Area);
   /* No "other" areas found.*/
   if(NULL!=nr) return(nr->Area);
   if(NULL!=sr) return(sr->Area);
   if(NULL!=wr) return(wr->Area);
   if(NULL!=er) return(er->Area);
   crash(INCONSISTENT);
   return(NULL);/*SURELY there MUST have been a more concise way to say that?*/
}

/* Checks if there's tunnel tiles one cell further than the tunnel bounds.
   If so, extends the bounds to cover them, and later returns 1. */
int attempt_tunext(int *xs,int *ys,int *xe,int *ye)
{
   int changed=0;
   maploc_t *mapstart=map->Data;
   if (*xs==*xe) { /* vertical tunnel */
      if (*ys<*ye) {
       if( (mapstart+FOOGRID_IDX(map,(*ys)-1,*xs))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel upwards.");
         (*ys)--;
         changed=1;
       }
       if( (mapstart+FOOGRID_IDX(map,(*ye)+1,*xs))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel downwards.");
         (*ye)++;
         changed=1;
       }
      } else {
       if( (mapstart+FOOGRID_IDX(map,(*ys)+1,*xs))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel downwards.");
         (*ys)++;
         changed=1;
       }
       if( (mapstart+FOOGRID_IDX(map,(*ye)-1,*xs))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel upwards.");
         (*ye)--;
         changed=1;
       }
      }
   } else { /* horizontal tunnel */
      if (*xs<*xe) { /* start less than end */
       if( (mapstart+FOOGRID_IDX(map,*ys,(*xs)-1))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel left.");
         (*xs)--;
         changed=1;
       }
       if( (mapstart+FOOGRID_IDX(map,*ys,(*xe)+1))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel right.");
         (*xe)++;
         changed=1;
       }
      } else {
       if( (mapstart+FOOGRID_IDX(map,*ys,(*xs)+1))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel right.");
         (*xs)++;
         changed=1;
       }
       if( (mapstart+FOOGRID_IDX(map,*ys,(*xe)-1))->Tiletype==PTL_TUNNEL)
       {
         bugger(REPTYPE_MAPGEN,LOGLEVEL_DEBUG,"Extending tunnel left.");
         (*xs)--;
         changed=1;
       }
      }
   }
   return(changed);
}

region_t *get_region(int x,int y, region_t *this)
{
   tile_t centre;
   region_t *nr;
   region_t *sr;
   region_t *wr;
   region_t *er;
   maploc_t *mapstart=map->Data;

   if (y<3||x<3||x>=MAPWID-2||y>=MAPHGT-2)
   {
      show_messages("Get_region for dubious coords!");
      crash(OUT_OF_BOUNDS);
   }
   centre=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
   if (centre==PTL_TUNNEL)/* || centre==PTL_DOOR */
   {
      return(find_region(x,y));
   }
   nr=(find_region(x,y-1));
   sr=(find_region(x,y+1));
   wr=(find_region(x-1,y));
   er=(find_region(x+1,y));
   if(NULL!=nr&&this!=nr) return(nr);
   if(NULL!=sr&&this!=sr) return(sr);
   if(NULL!=wr&&this!=wr) return(wr);
   if(NULL!=er&&this!=er) return(er);
   /* No "other" regions found */
   if(NULL!=nr) return(nr);
   if(NULL!=sr) return(sr);
   if(NULL!=wr) return(wr);
   if(NULL!=er) return(er);
   crash(INCONSISTENT); /* There SHOULD have been a tunnel around here to
     cause this, so we should have found a region. */
   return(NULL); /* Cheer up GCC */
}

void build_door(int x,int y,int roomno, region_t *region)
{
   entrance_t *door=(entrance_t *)counting_malloc(MEM_ENTRANCE,sizeof(entrance_t));
   entrancelist_t *regdoor=(entrancelist_t *)counting_malloc(MEM_ENTLIST,sizeof(entrancelist_t));
   entrancelist_t *roomdoor=(entrancelist_t *)counting_malloc(MEM_ENTLIST,sizeof(entrancelist_t));
   maploc_t *mapstart=map->Data;
   if((mapstart+FOOGRID_IDX(map,y,x))->Tiletype!=PTL_TUNNEL){
     /* We expect the space to alrady have a tunnel there */
     crash(INCONSISTENT);
   }
   if(find_region(x,y)==NULL) {
     /* The tunnel must be part of a region!!!  But sometimes isnt???*/
     crash(INCONSISTENT);
   }
   (mapstart+FOOGRID_IDX(map,y,x))->Tiletype=PTL_DOOR;
#ifdef NEW_DOOR
   /* preallocated dummy item */
   (mapstart+FOOGRID_IDX(map,y,x))->Itemlist=getItemtypeByName("Closed Door",IG_DUMMY)->Typedata.DummyInst;
#endif
   door->Roomno=roomno;
   door->Region=region;
   door->Xcoord=x; door->Ycoord=y;
   door->Open=0; /* Initially closed. This is required for map regen. */
   regdoor->Entrance=door;
   roomdoor->Entrance=door;

   regdoor->Next=region->Firstentrance;
   region->Firstentrance=regdoor;
   roomdoor->Next=tls->Room[roomno].Firstentrance;
   tls->Room[roomno].Firstentrance=roomdoor;
}

void add_intersection(int x,int y,region_t *region)
{
   intersection_t *head=region->Intersections;
   intersection_t *scan;
#ifdef MARK_INTERSECTIONS
/* Markers to show every single intersection. */
/* static itemtype_t *corner_marker=NULL; Now in globals.c and item_p.h */
/* Now in i_init.c 
   if(corner_marker==NULL) {
      corner_marker=(itemtype_t *)counting_malloc(MEM_ITYPE,sizeof(itemtype_t));
      newItemtype(corner_marker,'+',"Intersection marker",IG_STONE);
   } */
#endif

   for(scan=head;scan!=NULL;scan=scan->Next)
      if(scan->Xcoord==x && scan->Ycoord==y) return; /*Already there! */
   scan=(intersection_t *)counting_malloc(MEM_INTERSECTION,sizeof(intersection_t));
   scan->Xcoord=x;scan->Ycoord=y;
   scan->Next=head;
   region->Intersections=scan;

#ifdef MARK_INTERSECTIONS
/* place the marker. */
   placeItem(tls,x,y,newItem(corner_marker,1));
#endif
}

