/* ==== apply.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


/* #include "syscurses.h" */
/* defs */
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "crash.h"
/*#include "item_t.h" */

/* protos */
#include "ui.h"
#include "item_p.h"
#include "player_p.h" /* Now only has the protos, we don't need the types */
#include "equip.h"
#include "apply.h"

void apply_main()
{
    /* Pick an item to apply */
    /* call whichever function is appropriate to item's type */
    item_t *choice=prompt_for_inv_item(0);
    if (NULL==choice) return; /* Cancelled */
    switch (choice->Itemtype->Type)
    {
       case IG_CONTAINER:
          clearPrompt();
          apply_container(choice);
          break;
       case IG_TOOL:
          show_messages("No tools are usable yet!");
          break;
       case IG_WAND:
       case IG_STAFF:
       case IG_ROD:
          show_messages("Zappables don't work yet! When they do, it'll be with 'z'ap");
          break;
       case IG_SCROLL:
       case IG_BOOK:
          show_messages("Readables don't work yet! When they do, it'll be with 'r'ead");
          break;
       case IG_WEAPON:
       case IG_RANGED:
          show_messages("You would 'w'ield that.");
          break;
       case IG_SUIT:
       case IG_SHIELD:
       case IG_HEADGEAR:
       case IG_GLOVES:
       case IG_FOOTWEAR:
       case IG_RING:
       case IG_NECKWEAR:
          show_messages("You would 'W'ear that.");
          break;
       case IG_MISSILE:
          show_messages("You would 'f'ire that. But you can't yet.");
          break;
       case IG_FOOD:
       case IG_CORPSE:
          show_messages("You would 'e'at that (if your version of the game supports it).");
          break;
       case IG_CONDIMENT:
          show_messages("This would 'a'pply the condiment to some food, if it was implemented! You would also 't'hrow it at monsters.");
          break;
       case IG_POTION:
          /* Should also be able to apply potion of ink, to refill a pen */
          show_messages("Hey, potions do now work! But generally you would 'q'uaff them. Possibly throw them (not yet, sorry). You'll only be able to apply some of them, and you can't do that yet.");
          break;
       default:
          show_messages("You can't apply that! Silly person.");
          break;
    }
    /* Er... I think we now return!
       It's important to note that some things will use up the item
       when you apply it (perhaps it breaks) and we'll prolly need
       to do that by having the called function notify us that the
       item was destroyed, and we do the actual inventory/heap
       reordering and memory freeing *here*. */
}

/* This now moved to ui.c, where it can just call the
small helper functions made here. */

/*void apply_container(item_t *container) {}  */
    /* NOTES on apply_container(), for... somewhere */
    /* Remember, containers have weight limits, and some
       have further restrictions- eg a purse can only carry coins
       and stones, a quiver can only contain missiles, a fridge
       can only contain comestibles, a cruet can only contain
       condiments ;)
       Containers tend to be split into bags and boxes (of sorts).
       Boxes generally can't fit into other containers, but it
       depends on size.
       Objects have to be given sizes. Must remember this. */
    /* Prompt the user for an object to go in or come out of the
       container. Inventory has limited space, you can only have
       52 (!!!) items, which each have inventory letters that tend
       to remain, WHEREAS containers have potentially infinite
       space, so you can't just respond with a letter for item
       choices from containers. When selecting an item from a
       container, you see lists of their contents (possibly by
       category like in NH) screen by screen, each screenfull
       having items with one of a small range of letters to
       identify them. */

/* Returns 1 if could succeed, else  0. */
int can_go_in_container(item_t *container, item_t *item)
{
   /* Should this instead return 0? */
   char cname[80];
   char nstring[80];
   if(item==NULL) {
      bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Tried to place NULL item in container!");
      crash(INCONSISTENT);
   }
   /*describe_item(cname,container); */
   /*describe_item(nstring,item); */
   item_basename(cname,container);
   item_basename(nstring,item);
   if (p_has_equipped(item)) { /* Can't do that! */
       sprintf(strbuf,"You can't put the %s in the %s whilst you've got it equipped!",nstring,cname);
       show_messages(strbuf);
       return(0);
   }
   if ((!isinclass(item->Itemtype,container->Itemtype->Typedata.Container.AcceptsClass)) && (item->Itemtype->Type!=container->Itemtype->Typedata.Container.AcceptsSpec)) {
       sprintf(strbuf,"The %s won't go in the %s.",nstring,cname);
       show_messages(strbuf);
       return(0);
   }
   if ((item->Itemtype->Weight)>(container->Itemtype->Typedata.Container.Wlim-container->Instdata.Container.Currentweight)) {
       sprintf(strbuf,"The %s is too full for the %s to go in.",cname,nstring);
       show_messages(strbuf);
       return(0);
   }
   return(1);
}

void put_in_container(item_t *container, item_t *item)
{
   item_t *contents=container->Instdata.Container.Contents;
   /* Oh. Thats a lot shorter... */
   if (item==NULL) {
      bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Tried to put NULL item in container.");
      crash(INCONSISTENT);
   }
   container->Instdata.Container.Contents=item;
   item->Next=contents;
}

void remove_from_container(item_t *container, item_t *item)
{
   item_t *prev;
   item_t *heap=container->Instdata.Container.Contents;
   if (NULL==item) {
      bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Tried to remove NULL item from container.");
      crash(INCONSISTENT);
   }
   if (NULL==heap) {
      bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Tried to remove item from empty container.");
      crash(INCONSISTENT);
   }
   if (heap==item) {
      container->Instdata.Container.Contents=item->Next;
   } else { /* item must be further on */
      for(prev=heap;prev->Next!=item&&prev->Next!=NULL; prev=prev->Next);
      if(prev->Next==NULL) {
         bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Item to retrieve from container was not there.");
         crash(INCONSISTENT);
      }
      prev->Next=item->Next;
   }
   /*Unlike skim_item(), we don't do anything about the level-wide item list.*/
}

