/* ==== config.h ==== */

/* Compile-time options. Moving these #defines from the Makefile
to an actual source file means that make can tell when you've
changed these options. */
/* To use an option, uncomment it. To not use it, comment it again. */

/* TODO: Explain how to comment things in C; explain the differences
between setting defines in C source compared to in Makefiles */

/* # For normal game, define   CAREFUL   and  PLUNK_PATHS.
 You might also want to define RANDOMGAME  */

/* Various other potential combinations you might choose:
# CAREFUL  FATAL_PATHS  and  STEALTHY
# FATAL_PATHS  STEALTHY  AUTOPILOT  FEW_MON  CALM  and SEE_ALL
*/
/* NB: Aug 2004, note that these recommendations are pretty well
   out of date now! Defaults should be good. */


/* -----  OPTIONS START FROM HERE  ----- */


/* SEE_ALL lets you see through walls (useful for testing the AI code,
   but also is a PITA because you get messages about nearly everything). */
/* #define SEE_ALL */

/* COMBATSTATS lets you see stats of hit probabilities, evasion
   probabilities, damage, etc during combat. Just for debugging
   game balance stuff and all that */
/*#define COMBATSTATS */

/* CAREFUL ensures foogrid arrays are not accessed out of bounds.
   may be given further effects some time. Such out-of-bounds
   accesses can't possibly be worked around, they're a truly
   fatal bug, so this option is IMPORTANT. Luckily I've not seen
   any yet.
   The only reason not to use this option is that it could slow
   down access to the map a bit, but it doesn't seem to make
   much difference: just set it. */
#define CAREFUL


/* MARK_INTERSECTIONS  places little markers at every tunnel intersection
   that is created. This is useful for debugging the map generation code.
   If you get messages "Monster at (x,y) can't get to (x2,y2)" in your
   debug logs, then that means there's an intersection either missing
   or misplaced. The markers this option places are items that you
   can pick up. Currently, they are 'X' symbols.
   Without this option set, you would see tunnels meeting and would have
   to just assume there was an intersection where they meet. But Porrog
   might not know this! */
#define MARK_INTERSECTIONS

/* OLDINVENT makes porrog use the old inventory code, where items you
   pick up appear at the top of your inventory (so the items at the
   end of the inventory list are the ones you've held longest). This
   has never seemed to have any bugs. HOWEVER:
   If you undefine (or comment out) OLDINVENT, porrog will use the new
   inventory code, which puts items you pick up at the *end* of your
   inventory list, making it look nicer (IMO). This has not been
   extensively tested yet, and might be buggy. Hence you have this
   option to give you the choice, just in case.
   ...Note, this isn't to do with whether to use the new pretty inventory
   *listing* code or not, it just affects where items go when you get them.*/
/* #define OLDINVENT */

/* TIERED_MONSTERS makes sure that monsters generated are of a suitable
   level (an average of the dungeon level and your level) */
#define TIERED_MONSTERS

/* PLUNK_PATHS makes the map generation code use the new  plunk_paths1()
   and plunk_paths2()  functions instead of the old code that was
   made part of attempt_joining(). The old code was pretty broken,
   but the new code hasn't had much testing so far. This option is
   only here so that I don't have to take out the old code completely.
   When you define this, the majority of attempt_joining continues to
   work, but the part that turned out to have been buggy is moved to
   the separate functions that I've made more robust.   Just keep
   it defined, don't comment it if you know what's good for you. */
#define PLUNK_PATHS

/* NEW_DOOR makes closed doors be represented by a dummy item, such that
   if you see a door is closed, the code will show what item you remember
   being there (the dummy item) when you've gone, and if the door then
   opens, this will not be shown until you return to see it.
   Similarily, if a door you remember as open is closed, you'll not
   remember the dummy item being there, so you'll remember the door as
   still open.
   Apart from changes to draw_map(), this requires changes to the code
   for opening and closing doors (particularly, it requires that doors
   may not be blocked by items when they are closed, as such items
   would be destroyed), and for setting out the doors when entering
   a level.
   (Note, that though there is also new code so opening/closing doors
   has a new UI and you no longer simply open/close all orthogonally
   adjecent doors, the NEW_DOOR option does not affect *that* at all) */
#define NEW_DOOR

/* ROGUE_TUNNELS makes tunnels render as just isolated lines of colons
   like      :    :
     :::::::::::::::
         +
   etc, like in roguelikes such as Rogue, Hack, Nethack (although those
   use different symbols) RATHER THAN showing the dungeon walls around
   the tunnels, like  #########:#####:#
                      :::::::::::::::::
                      ####+############
    like in many others such as ADOM, *Angband, and probably others.
   Some people would prefer the walled, Adom-style tunnels, because they
   can see the dungeon structure in terms of where the walls are, but
   the main reason *I* liked that style is that I'm used to "#" being
   walls/rock, and the Nethack approach used it as the tunnels themselves!
   As Porrog uses a different symbol for the tunnels, I don't think that's
   a problem, and using ROGUE_TUNNELS has various nice advantages-
   for one thing, you don't see where the walls of rooms are before you go
   into them. NB: I might add an option "ROGUE_ROOMS" that might give
   line-based symbols to the walls of rooms, like in Rogue and Nethack. */
/* OK, in retrospect it seems a tad ugly. It may take some getting used to!*/
/* #define ROGUE_TUNNELS */

/* ITEM_PROBABILITY makes the item generation code use the probability
   data to pick item types. Items with very low probability will be rare.
   It's possible some items will get missed out by this code, or that
   the scanning will overflow somehow. Otherwise, you should define it. */
#define ITEM_PROBABILITY

/* FATAL_PATHS makes the waypoint finding code crash if a path cant be found.

   I've now changed the code to just log this serious error so that
   normal players can see it in a log file dumped after the game (and
   people running in DEBUG mode can see it whenever they want).

   So, I'm pretty sure this option is no longer used ANYWHERE. */
/* #define FATAL_PATHS */


/* VERBOSE_AI makes the AI routine mention when monsters have difficulty
   getting about in whatever ways. Can be useful for investigating the AI.
   can also be a massive PITA.
   Don't think this has ANY effect now, as I'm changing those debug
   messages to use the newly made debug logging functions instead,
   which doesn't need any compile-time options. */
/* #define VERBOSE_AI */


/* RANDOMGAME makes the game's RNG be initialised with different random
   numbers each time. Without this, the game should run the same each
   time (*very* useful for debugging!).
   At some point, I'd like to change the RNG to be able to derive
   entropy from processing a single string of characters, as the only
   alternative to using the system's random() function. Advantages to
   that? Well, AFAICT, random() is not guaranteed to produce the same
   sequence of numbers from a given seed across different systems. I'd
   quite like to be able to give character strings that (combined
   with the same user input) will result in the same end situation.
   The original idea was that using such a scheme, you could get a
   form of savegame that could be exchanged between people in newsgroups
   and forums and things. I forget the exact details of how I intended
   to deal with the vast list of keypresses that would end up written
   in such a thing... :P */
#define RANDOMGAME  /* */

/* NO_REGEN disables the new (not THAT new) player regeneration code, if
   you don't want that for some reason. */
/* #define NO_REGEN */

/* NO_EAT disables the new satiation/hunger/eating code, which is probably
   in need of tweaking (insofaras it doesn't really make the game
   noticably harder) */
/* #define NO_EAT */

/* AUTOPILOT makes the game ignore the keyboard, and simply
   watches one monster wander the map indefinitely.
   To stop the autopilot you must interupt it with ctrl-C.
   The autopilot is often combined with ONE_MON.
   NOTE: currently you can only stop the program from AUTOPILOT mode
   by pressing ctrl-C, and that currently stops the buglog being dumped. */
/* #define AUTOPILOT */


/* ONE_MON makes sure only one monster is ever created. */
/* #define ONE_MON */


/* FEW_MON makes sure very few monsters are created.
    NOTE: This suddenly seems to cause compilation errors???
    If anyone can see why, I'd love to know. */
/* #define FEW_MON */


/* STEALTHY makes you unseen by monsters (they may still hit you
   if you get in their way). Useful for debugging. */
/* #define STEALTHY */


/* CALM stops monsters getting so angry that they fight each other.
   This is particularly useful for testing how well monsters in
   tunnels can extricate themselves from traffic jams (they seem to do
   quite well now, if they're allowed to take their time about it). */
/* #define CALM */


/* ----- There are no more options. That's all. ----- */




/* END of config.h */














/* -----  No, really, stop reading already.  ----- */














/* -----  There really is NO MORE for you to see!!  ----- */














/* -----  You're wasting your time...  ----- */

















/* -----  Pfft, some people just won't listen.  ----- */





















































/* Told you. */
