/* ==== ctrl.c ==== */
/* runs the main loop: getting keypresses, doing actions
   corresponding to the keypresses, updating other things,
   then displaying the map */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include "porrog_defs.h"
#include <math.h>   /* FOR TESTING */
#include "syscurses.h"
#include <unistd.h>
#include <stddef.h>
#ifdef AUTOPILOT
#include    <time.h>
#endif
#include "C-foogrid.h" /* Used for foogrid*/

/*#include "mapgen_types.h" */
#include "maparray.h"
#include "crash.h"
#include "dXrng.h"
#include "memutils.h"

#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "player_t.h"
#include "ai_t.h"

/* protos? */
#include "makemap_p.h"
#include "graphics_p.h"
#include "help.h"
#include "item_p.h"
#include "player_p.h"
#include "monster.h"
#include "version.h"
#include "ui.h"
#include "apply.h"
#include "potions.h" /* for quaff_main*/
#include "ai_p.h"
#include "mg_utils.h" /* For find_region */

/* We'll put the code for adjusting viewport coords here in macros */
#define xvc_adj(vx,px) (px<(vx+16))?(px-26):((px>(vx+32))?(px-24):vx)
#define yvc_adj(vy,py) (py<(vy+7 ))?(py-13):((py>(vy+15))?(py-10):vy)
/* Forgot, we also have to clip the viewport to the proper coords */
#define xvc_clip(vx) (vx<0)?0:((vx>(MAPWID-51))?(MAPWID-50):vx)
#define yvc_clip(vy) (vy<0)?0:((vy>(MAPHGT-24))?(MAPHGT-24):vy)

extern unsigned int randCycle; /* so we can set it from main() and not
                             need to pass the silly thing */
void free_everything();

/* Returns different values depending on why the game exited,
   for main() to deal with appropriately */
int ctrl_loop() /* Should this take any args? */
{
    /* Initial coordinates of the top-left corner of the display */
#ifndef AUTOPILOT
/*  int x=(MAPWID/2)-25,y=(MAPHGT/2)-13; */
    int x=player.X-25,y=player.Y-11;

    /* used to be -10, but changed due to change in display height */
#else
/*  struct timespec req={0, 500000};   V fast but visible */
    struct timespec req={0,40000000};  /* Reasonable */
/*  struct timespec req={0,75000000};  Slow enough to pause in GDB*/
    struct timespec rem;
    int x=(tls->Moblist->X)-25,y=(tls->Moblist->Y)-11; /* again, was -10 */
#endif
    monster_t *mob;
    int cycledur; /* duration of each cycle of loop */
    int pdur; /* duration of player's action (or main monster's
      action if using autopilot */
    int croom; /* Room used for trying to add monsters */
    maploc_t *mapstart=map->Data;
/*   intersection_t *inter; */
    char k;

/*  make_map();  Should get called when you enter a new level */

    /* quit is now in globals.c and player_t.h */
    while(quit==0)
    {
      draw_status();/* HP, etc */
#ifndef AUTOPILOT
      draw_map(x,y,player.X,player.Y);
        /* Somewhere in the middle */
#else
      /*
      draw_map(x,y,tls->Moblist->X,tls->Moblist->Y); */
      draw_map(x,y,player.X,player.Y);
#endif

#ifndef AUTOPILOT
      /* Get user input */
      /* check we can move already! */
      if(player.Ticker<=0) {
       k=getch();
       show_messages(NULL); /* Allow messages buffer to reset */
       switch (k){
         case '?': /* Help! */
            help_main();
       /* show_messages("This should give help (but that's not done yet)."); */
            /* Haha! It is now! :) */
            pdur=0;
            break;
         case 'h':
            p_move(-1,0); /* Now p_move();*/
            if (player.X<(x+16)) {x=(player.X-26);}
            if(x<0) x=0;
            pdur=1000; /* get from func */
            break;
         case 'y': /* -1,-1 */
            p_move(-1,-1);
            if (player.X<(x+16)) {x=(player.X-26);}
            if (player.Y<(y+7))  {y=(player.Y-13);}
            if(x<0) x=0;
            if(y<0) y=0;
            pdur=1000;
            break;
         case 'u': /* 1,-1 */
            p_move(1,-1);
            if (player.X>(x+32)) {x=(player.X-24);}
            if (player.Y<(y+7))  {y=(player.Y-13);}
            if(x>(MAPWID-51)) x=(MAPWID-50);
            if(y<0) y=0;
            pdur=1000;
            break;
         case 'j':
            p_move(0,1);
            if (player.Y>(y+15)) {y=(player.Y-10);}
            if(y>(MAPHGT-24)) y=(MAPHGT-24); /* was -23, -23 */
            pdur=1000;
            break;
         case 'k':
            p_move(0,-1);
            if (player.Y<(y+7)) {y=(player.Y-13);}
            if(y<0) y=0;
            pdur=1000;
            break;
         case 'b':
            p_move(-1,1);
            if (player.X<(x+16)) {x=(player.X-26);}
            if (player.Y>(y+15)) {y=(player.Y-10);}
            if(x<0) x=0;
            if(y>(MAPHGT-24)) y=(MAPHGT-24);
            pdur=1000;
            break;
         case 'n':
            p_move(1,1);
            if (player.X>(x+32)) {x=(player.X-24);}
            if (player.Y>(y+15)) {y=(player.Y-10);}
            if(x>(MAPWID-51)) x=(MAPWID-50);
            if(y>(MAPHGT-24)) y=(MAPHGT-24);
            pdur=1000;
            break;
         case 'l':
            p_move(1,0);
            if (player.X>(x+32)) {x=(player.X-24);}
            if(x>(MAPWID-51)) x=(MAPWID-50);
            pdur=1000;
            break;
         case '<': /* Up stairs */
            p_try_upstairs(); /* Should this function return some value
               for use here (eg to detect change in level)? */
            x=xvc_adj(x,player.X);x=xvc_clip(x);
            y=yvc_adj(y,player.Y);y=yvc_clip(y);
            pdur=3000; /* get from func */
            break;
         case '>': /* Down stairs */
            p_try_downstairs();
            x=xvc_adj(x,player.X);x=xvc_clip(x);
            y=yvc_adj(y,player.Y);y=yvc_clip(y);
            pdur=3000; /* get from func */
            break;

         case 'o':
            /* open doors*/
            p_open_door(); /* now p_open_door()*/
            pdur=1200;
            break;
         case 'c':
            /* close doors */
            p_close_door();
            pdur=1200;
            break;

         case 'a':
            /* Apply something! New code. */
            apply_main();
            pdur=400; /* get from func */
            break;
         case 'q': /* quaffing at last! */
            quaff_main(); /* in potions.c */
            pdur=400; /* get from func */
            break;
         case 'Q': /* Should set return status */
            quit=1;
            pdur=0;
            break;
         case 'r': /* Should soon change this to do reading */
            if (find_region(player.X,player.Y)==NULL) { /* inside room */
              sprintf(strbuf,"Room %d",find_room(player.X,player.Y));
              show_messages(strbuf);
            } else { /* In corridor */
              show_messages("You're in a tunnel/doorway!");
            }
            pdur=0; /* NAUGHTY! */
            break;
         case 'v':
            version(strbuf);
            show_messages(strbuf);
            pdur=0;
            break;
         case 'R':
            show_messages("Refreshing screen.");
            wrefresh(curscr); /* Try to refresh screen too.Put this elsewhere */
            pdur=0;
            break;
         case 'i':
            /* display_heap("Your inventory",player.Inventlist);  OLD one */
            /* this version is experimental! */
            displayHeapV2(PLISTMODE_SHOWINV,"Your inventory",player.Inventlist);
            pdur=0;
            break;
         case ';':
            /* display_heap("On the floor",((maploc_t *)(mapstart)+FOOGRID_IDX(map,player.Y,player.X))->Itemlist); */
            displayHeapV2(PLISTMODE_SHOWHEAP,"On the floor",((maploc_t *)(mapstart)+FOOGRID_IDX(map,player.Y,player.X))->Itemlist);
            pdur=0;
            break;
         case ',': /* Will now choose an item to pick up */
            p_pickup(map); /* Does it all now */
            pdur=700; /* get from func */
            break;
         case 'd': /* Will now choose an item to drop */
            p_drop(map);
            pdur=300; /* get from func */
            break;

         case 'w': /* Wield summat */
            show_messages("wield what?");
            /*
            p_wield(prompt_for_inv_item()); */
            p_trywield();
            pdur=500; /* get from func */
            break;
         case 'W': /* Wear summat */
            show_messages("wear what?");
            p_trywear();
            pdur=4000; /* get from func */
            break;
         case 'T': /* Take off something. */
            show_messages("Take off what?");
            p_tryunwear();
            pdur=4000; /* get from func */
            break;
         /* unwielding done differently? */

         case 'e': /* eat: unimplemented */
#ifdef NO_EAT
            show_messages(roll_dX(d4set[0])?"You're not hungry right now.":"Your jaw seems strangely stuck!");
            pdur=90; /* get from func, when done! */
#else /* initial code */
            pdur=p_tryEat();
#endif
            break;
         case 'M':
            show_messages("This is a very very long message just to test the function show_messages(), OKay");
            pdur=0; /* naughty! */
            break;
         case 16: /* THIS SHOULD BE ^P... but I'm not certain it's portable!*/
            show_mrb(); /* Show old messages buffer! */
            pdur=0;
            break;
         case 'm':
            /*
            show_messages("Creating new map.");
            make_dmap(firstlevel,1); NAUGHTY! */
            show_messages("The \'m\' key no longer heals your wounds. Such a feature may be put in a \"debug\" menu later.");
            /*
            show_messages("You heal all your wounds!");
            player.Var_stats.Hp=player.Var_stats.MaxHp; -* NAUGHTY!*/
            pdur=0; /* Naughty! */
            break;
         case 'x':
            show_messages("Trying to REGENERATE map.");
            remake_dmap(thislevel); /* NAUGHTY! */
            pdur=0; /* naughty */
            break;
         case 'D': /* NAUGHTY! */
            show_messages("Dumping buglog. This should actually put you in DEBUG mode, and prevent you winning, but meh. Not that yet.");
            dump_buglog();
            pdur=0;
            break;
         case 'p': /* SILLY */
            loadPBMFile("arranged.pbm");
            displayBitmap();
            sleep(8);
            clear(); wrefresh(curscr);
            pdur=0;
            break;
         case '.': /* wait */
            pdur=250; /* should be calculated 1/4 of normal move,
             or less if concentrating. Means we should have a function
             to calculate it really */
            break;
         default:
            show_messages("Unknown key!");
            pdur=0;
       }
       player.Ticker+=pdur;
      } /* else keep old value of pdur... */
#else
      nanosleep(&req,&rem); /* Just assume it works. */
      pdur=1000; /* for autopilot, remember?*/
#endif
      /* Now update environment */
      cycledur=MIN(50,pdur); /* could be less? */

      /* Get details of player position */
#ifndef STEALTHY
      pstype=p_whatamion();
      if(pstype==1||pstype==2) psroomno=find_room(player.X,player.Y);
      if(pstype==0||pstype==2) {
        psregion=find_region(player.X,player.Y);
/*      for(inter=psregion->Intersections;inter!=NULL;inter=inter->Next)
           (mapstart+FOOGRID_IDX(map,inter->Ycoord,inter->Xcoord))->Itemlist
           ->Itemtype=first_its[IG_STONE]; */
      }
#else
/* Pretend the player is somewhere no monster can reach-
 makes it as though player is invisible */
      pstype=0;psregion=NULL;
#endif

#ifndef AUTOPILOT
      /* Occasionally produce a new mob. */
      tls->Mobticker+=cycledur; /* NOTE: 1000 moments = 1 old move.
       Make sure this can't overflow!!!!*/
      /* limit was: 100+(tls->Nmobs*25),
      then 300+(tls->Nmobs*40),
      then 100+(tls->Nmobs*6),
      then to  75+((tls->Nmobs/2)^2) Still a bit too slow.
      Trying 35+((tls->Nmobs/2)^2). Base generation should be very fast
      The important thing is for map to initially be sparsely populated */
      if ((tls->Mobticker/1000)>(35+pow(tls->Nmobs/2,2) )) {
         /* Lets make a new mob!*/
         croom=pick_room(tls->Totalrooms); /* Some random room*/
         if (0==pstype|| ((1==pstype||2==pstype)&&psroomno!=croom) ) {
            add_room_mobs(croom); /* Put em where player isn't */
            tls->Mobticker=0; /* Reset the counter if we managed */
            show_messages("You hear a *plop* noise"); /* how naughty */
         }
      }
#endif /* AUTOPILOT */
      /* All monsters act */
      for (mob=tls->Moblist;mob!=NULL;mob=mob->Next) {
         mob->Ticker-=cycledur;
         if(mob->Ticker<=0) {
           ai_act(mob);
           mob->Ticker+=1000; /* get from func... */
           /* And they should also regenerate, but slower than player */
           if(mob->Var_stats.Hp<mob->Var_stats.MaxHp && !roll_dX(d20set[0]))
              mob->Var_stats.Hp++;
           /* actually, regeneration should be independent of movement
           speed. Perhaps increment HP when cycledur>large random number? */
         }
      }
#ifdef AUTOPILOT
      x=xvc_adj(x,tls->Moblist->X);x=xvc_clip(x);
      y=yvc_adj(y,tls->Moblist->Y);y=yvc_clip(y);
#endif

      randCycle+=731;randCycle-=(randCycle>999)?1000:0;
#ifndef NO_REGEN
      /* Regenerate player. */
      /* was &&roll_dX(d6set[0])<2, ==1/3; changed to !roll_dX(d8set[0]),=1/8*/
      /*if (player.Var_stats.Hp<player.Var_stats.MaxHp && !roll_dX(d8set[0]))*/
      if (cycledur!=0 \
         && player.Var_stats.Hp<player.Var_stats.MaxHp \
         && cycledur> ( (200*roll_dX(d20set[0])) \
                       +(10*roll_dX(d20set[1])) \
                       +roll_dX(d10set[0])) \
         &&(2>roll_dX(d4set[3]))) /* every 8 moves?*/
         player.Var_stats.Hp++;
#endif
#ifndef NO_EAT
      /* digest food/ get hungrier */
      if (cycledur>randCycle /* roughly once per move */ \
         && player.Var_stats.Satiation>0)
         /* player.Var_stats.Satiation--; */
         p_decSatiation(1);
      if(categorizeSatiation(player.Var_stats.Satiation)==SAT_STARVING && \
         cycledur>(((randCycle+300)%1000)+player.Var_stats.Satiation) && \
         roll_dX(d20set[0])+roll_dX(d20set[1])==0) {
            show_messages("You starve to death!");
            quit=2;
      }
#endif
      /* now advance player time, bringing us closer to next move */
      player.Ticker-=cycledur;
    }
    /* We've exited. Either quitted, saved, died, or won. Other options? */
    bugger(REPTYPE_CODE,LOGLEVEL_DEBUG,"Game loop ended.");
    /* Now show appropriate screen? */
    switch(quit) {
    /* if (quit==2) {  dead */
      case 1: /* quitting */
        loadPBMFile("quitter.pbm");
        displayBitmap();
        move(21,29);addstr("You throw in the towel");
        break;
      case 2: /* dead */
        loadPBMFile("skel.pbm");
        displayBitmap();
        move(20,34);addstr("You are dead");
        move(21,26);addstr("Sucks to be you, doesn't it?");
        break;
      case 3: /* save */
        loadPBMFile("saved.pbm");
        displayBitmap();
        move(21,35);addstr("Game saved");
        break;
      case 4: /* win */
      default:
        /* */
        break;
    }
        move(23,0); /* move cursor away */
        refresh();
        sleep(4); /* should wait for a key instead */
    /* } */
    bugger(REPTYPE_CODE,LOGLEVEL_DEBUG,"Game ending normally; dumping buglog.");
    /* Try to free everything? */
    dump_buglog(); /* THIS SHOULD BE CALLED AT THE END instead. But...
     I might forget myself and have free_Everything free the ringbuffers! */
    free_everything();

/*  endwin(); */
/*  printf("%ld unfreed allocations made.\n",check_allocs()); */
    return(0); /* Change to whatever our return status is */
}

void free_everything()
{
    monster_t *mob;
    monster_t *mnx;
    mobtype_t *mt;
    mobtype_t *mtnx;
    item_t *item;
    item_t *inx;
    itemgroup_t grp;
    itemtype_t *itype;
    itemtype_t *next_it;
    aicache_memory_t *mmem;
    levelmap_t *nextlev;
    int i;
    fprintf(stderr,"Freeing level data...\n");
    for (thislevel=firstlevel;thislevel!=NULL;thislevel=nextlev)
    {
      tls=thislevel->Unpacked;
      free_areas();
      for(i=0;i<217;i++) {
          if(tls->Room[i].Xcentre==-1) continue;/* not a real room */
          free_entrancelist(tls->Room[i].Firstentrance,0);
          /* The 0 shows the entrances themselves are already
          freed. Only free the *list*. */
      }
      for(item=tls->Itemlist;item!=NULL;item=inx) {
          inx=item->Next2;
          freeItem(item);
      }
      for(mob=tls->Moblist;mob!=NULL;mob=mnx) {
          mmem=mob->Memory;
          aicache_freerooments(mmem->Roombuffer);
          aicache_freetunnelents(mmem->Tunnelbuffer);
          aicache_freerooments(mmem->Roomcache);
          aicache_freetunnelents(mmem->Tunnelcache);
          counting_free(MEM_MMEM,mmem);                      
          /* Should also free monster inv here */
          mnx=mob->Next;
          counting_free(MEM_MONSTER,mob);
      }
      free_features();
      nextlev=thislevel->Nextlevel;
      counting_free(MEM_LVLSTRCT,tls);
      counting_free(MEM_LVLMAP,thislevel);
    }
/* === Freed all levels  === */
    fprintf(stderr,"Level data freed.\n");
    /* also free item types. */
    counting_free(MEM_ITEM,phonyitem); /* mustnt use counting_free for this */
    /* free player inventory */
    for(item=player.Inventlist;item!=NULL;item=inx) {
       inx=item->Next;
       freeItem(item);
    }
    #ifdef MARK_INTERSECTIONS
    counting_free(MEM_ITYPE,corner_marker);
    #endif
    for(grp=0;grp<IG_LASTGROUP   ;grp++) {
      if(grp==IG_CORPSE) continue; /* mobtype freeing code does these,
      because they have names mallocced by mobtype creation code. */
      for(i=0,itype=first_its[grp];i<it_nums[grp];i++) {
         if(grp==IG_DUMMY) counting_free(MEM_ITEM,itype->Typedata.DummyInst);
         next_it=itype->Next;
         counting_free(MEM_ITYPE,itype);
         itype=next_it;
      }
    } /* end freeing item types */
    /* Still need to free monster types, and STRINGS!?*/
    for(mt=first_mt;mt!=NULL;mt=mtnx) {
       mtnx=mt->Next;
       counting_free(MEM_STRING,mt->Corpsetype->Name);
       counting_free(MEM_ITYPE,mt->Corpsetype); /* Make sure the itemtype
        freeing code doesnt free that? */
       counting_free(MEM_MTYPE,mt);
    }
    /* Free our strings. Hope dump_buglog() has already been called by now. */
    free_mrb();
    free_buglog();
    /* Is that everything now? */
    fprintf(stderr,"Remaining data freed.\n");
}

