/* ==== dungeon.c ==== */
/* general functions for controlling dungeony things */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */



#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h"
/* protos */
#include "crash.h"
#include "item_p.h"
#include "mg_utils.h"
#include "dungeon_p.h"
#include "dXrng.h"



direction_t checkForTile(int x,int y,tile_t target_tile,int consider_here) {
    /* int scx,scy,tx,ty;*/
    direction_t current_status=FACE_NONE,scandir;
    maploc_t *mapstart=map->Data;
    /*maploc_t *tile;*/
    /* tx=x+1; ty=y+1;
    for(scx=(x-1);scx<=tx;scx++) {
       for(scy=(y-1);scy<=ty;scy++) {
          =(mapstart+FOOGRID_IDX(map,y,x));
       }
    } */
    if(consider_here && target_tile==(mapstart+FOOGRID_IDX(map,y,x))->Tiletype)
        current_status=FACE_HERE;
    for(scandir=FACE_N;scandir<=FACE_NW;scandir++) {
       if(target_tile==(mapstart+FOOGRID_IDX(map,(y+FACING_TO_Y(scandir)),(x+FACING_TO_X(scandir))))->Tiletype) {
          /* got a match */
          if(current_status==FACE_NONE) current_status=scandir;
          else return(FACE_ERROR);
       }
    }
    return(current_status);
}

int isUnopenable(int x,int y) { /* UNopenable, not openable...*/
    maploc_t *mapstart=map->Data;
    maploc_t *tile=(mapstart+FOOGRID_IDX(map,y,x));
    if(tile->Tiletype==PTL_ODOOR) return 2; /* already open */
    if(tile->Tiletype!=PTL_DOOR) return 1; /* Not a door */
    /* if( door->locked)   return 3; */
    if(!roll_dX(d8set[0])) return 4; /* stuck! */
    return 0;  /* we can in fact open it, hence NOT unopenable... */
}

int isUncloseable(int x,int y) { /* UNcloseable, not closeable */
    maploc_t *mapstart=map->Data;
    maploc_t *tile=(mapstart+FOOGRID_IDX(map,y,x));
    if(tile->Tiletype==PTL_DOOR) return 2; /* already closed */
    if(tile->Tiletype!=PTL_ODOOR) return 1; /* Not a door */
    if(tile->Itemlist!=NULL||tile->Mptr!=NULL) return 3; /* occupied! */
    if(!roll_dX(d12set[0])) return 4; /* stuck! less probable for closing */
    return 0; /* can close it */
}

void open_door(x,y) {
    entrance_t *door;
    maploc_t *mapstart=map->Data;
    maploc_t *tile=(mapstart+FOOGRID_IDX(map,y,x));
    if (tile->Tiletype!=PTL_DOOR) {
       bugger(REPTYPE_CODE|REPTYPE_MAPGEN,LOGLEVEL_WARNING,"open_door(): tried to open something other than a closed door");
       return;
    }
    door=find_door(find_region(x,y),x,y);
    if(door==NULL) {
       bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"open_door(): door not found in doors list for region");
       crash(INCONSISTENT);
    }
    door->Open=1; /* for regenerating map */
#ifdef NEW_DOOR
    tile->Itemlist=NULL; /* clear away the "Closed Door" dummy item */
#endif
    tile->Tiletype=PTL_ODOOR;
}

void close_door(x,y) {
    entrance_t *door;
    maploc_t *mapstart=map->Data;
    maploc_t *tile=(mapstart+FOOGRID_IDX(map,y,x));
    if (tile->Tiletype!=PTL_ODOOR) {
        bugger(REPTYPE_CODE|REPTYPE_MAPGEN,LOGLEVEL_WARNING,"close_door(): tried to close something other than an open door");
        return;
    }
    if (tile->Itemlist!=NULL) {
       bugger(REPTYPE_CODE|REPTYPE_ITEMS|REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"close_door(): doorway occupied by items");
       crash(INCONSISTENT);
    }
    if (tile->Mptr!=NULL) {
       bugger(REPTYPE_CODE|REPTYPE_MONSTERS,LOGLEVEL_FOOBAR,"close_door(): doorway occupied by monster");
       crash(INCONSISTENT);
    }
    door=find_door(find_region(x,y),x,y);
    if(door==NULL) {
       bugger(REPTYPE_MAPGEN,LOGLEVEL_FOOBAR,"close_door(): door not found in doors list for region");
       crash(INCONSISTENT);
    }
    door->Open=0; /* store for regenerating map */
#ifdef NEW_DOOR
    tile->Itemlist=getItemtypeByName("Closed Door",IG_DUMMY)->Typedata.DummyInst;
#endif
    tile->Tiletype=PTL_DOOR;
}

