/* ==== equip.c ==== */
/* Code for items of equipment. May be merged in with some other file */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include "C-foogrid.h"
#include "porrog_defs.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
/* #include "maparray.h" */
#include "player_t.h"

/* protos */
#include "ui.h"
#include "item_p.h"
#include "crash.h"

/* Try to equip a weapon (either hand-to-hand or ranged) */
equipspace_t classify_equipspace(item_t *item)
{
    switch (item->Itemtype->Type) {
        case IG_WEAPON:
           return(EQUIP_WEAPON);
        case IG_RANGED:
           return(EQUIP_RANGED);
        case IG_MISSILE:
           return(EQUIP_NONE);
        case IG_SUIT:
           return(EQUIP_SUIT);
        case IG_SHIELD:
           return(EQUIP_SHIELD);
        case IG_HEADGEAR:
           return(EQUIP_HEADGEAR);
        case IG_GLOVES:
           return(EQUIP_GLOVES);
        case IG_FOOTWEAR:
           return(EQUIP_FOOTWEAR);
        case IG_RING:
           return(EQUIP_RING1); /* Caller should check and consider RING2*/
        case IG_NECKWEAR:
           return(EQUIP_NECKWEAR);
        case IG_SCROLL:
        case IG_BOOK:
        case IG_FOOD:
        case IG_CORPSE:
        case IG_CONDIMENT: /* Will be able to throw them. But wield?? */
        case IG_POTION: /* Ditto. */

           return(EQUIP_NONE);
        case IG_WAND:
        case IG_ROD:
           return(EQUIP_RANGED);
        case IG_TOOL:
           /* Some of these are weapon-tools. Can we check these? */
           /* Consider screwdrivers, shovel... bottle? */
           return(EQUIP_NONE);
        case IG_CONTAINER:
           return(EQUIP_NONE);
        case IG_STONE:
           return(EQUIP_WEAPON); /* Heavy blunt objects */
        case IG_COINAGE:
           return(EQUIP_NONE);

        case IG_LASTGROUP:
           crash(INCONSISTENT); /* NOTHING should have that!! */
/*      default:
           return(EQUIP_NONE); */
    }
    return(EQUIP_NONE);
}

/* Returns 1 if item has been equipped */
int p_has_equipped(item_t *item)
{
   equipspace_t space=classify_equipspace(item);
   switch (space) {
      case EQUIP_NONE:
         return(0); /* Can't be equipped! */
      case EQUIP_RING1: /* Check RING1 AND RING2 */
         if(player.Equipment[EQUIP_RING1]==item
         || player.Equipment[EQUIP_RING2]==item)
           return(1);
         else return(0);
      default:
         if (player.Equipment[space]==item) return(1);
         else return(0);
   }
   crash(FUNCTION_ERROR);
   return(1);
}

/* Was p_wield(weapon), is now a generic p_equip(item).
Will equip any equippable item if there isn't already an
item equipped there. If space is RING1, will consider RING2
as well. */
void p_equip(item_t *item)
{
    int cov;
    char nstring[80];
    equipspace_t space=classify_equipspace(item);
/*  if (space>EQUIP_RANGED) return;  Can't wield that! */
    if (space==EQUIP_NONE) crash(INCONSISTENT); /* Shouldn't have happened */

    item_basename(nstring,item);
    if (space==EQUIP_RING1 &&player.Equipment[EQUIP_RING1]!=NULL)
       space=EQUIP_RING2; /* Alternative finger! Might allow more */

    if (player.Equipment[space]!=NULL) {
       item=player.Equipment[space]; /* To give names */
       item_basename(nstring,item);
       switch(space) {
         case EQUIP_WEAPON:
         sprintf(strbuf,"You're already wielding a %s!",nstring);
         break;
         case EQUIP_RANGED:
         sprintf(strbuf,"You've already readied a %s!",nstring);
         break;
         case EQUIP_SUIT:
         case EQUIP_SHIELD:
         case EQUIP_HEADGEAR:
         case EQUIP_GLOVES:
         case EQUIP_FOOTWEAR:
         case EQUIP_NECKWEAR:
         sprintf(strbuf,"You're already wearing a %s!",nstring);
         break;
         case EQUIP_RING1: /* Should *this* now be the freak case? */
         case EQUIP_RING2:
            /* We've already tried other slots. */
            sprintf(strbuf,"You can't wear any more rings!");
            break;
    /*   case EQUIP_RING2: */
         case EQUIP_NONE:
         crash(FUNCTION_ERROR); /* Those shouldnt happen */
       }
       show_messages(strbuf);
       return;
    }

    player.Equipment[space]=item;
    if (space>=EQUIP_SUIT &&space<EQUIP_NONE) { /* armour */
       cov=item->Itemtype->Typedata.Arm.Cov;
       player.Var_stats.Cov+=cov;
       player.Var_stats.Pravg+=(item->Itemtype->Typedata.Arm.Prot*cov); /* NOT RIGHT! Must recalc avg.*/
    }
    /* Won't do this any more. Caller can do it instead.
    if (space==EQUIP_WEAPON) {
       sprintf(strbuf,"You wield a %s.",weapon->Itemtype->Name);
    } else {
       sprintf(strbuf,"You ready a %s.",weapon->Itemtype->Name);
    }
    show_messages(strbuf); */
    return;
}

/* is this used any more? */
void p_unequip(item_t *item)
{
    int cov;
    equipspace_t space=classify_equipspace(item);
    if (space==EQUIP_NONE) crash(INCONSISTENT);
    if (player.Equipment[space]==item|| (space==EQUIP_RING1&&player.Equipment[space=EQUIP_RING2]==item)) {
       /* You were wearing item */
       player.Equipment[space]=NULL;
       if (space>=EQUIP_SUIT &&space<EQUIP_NONE) { /* armour */
          cov=item->Itemtype->Typedata.Arm.Cov;
          player.Var_stats.Cov-=cov;
          player.Var_stats.Pravg-=(item->Itemtype->Typedata.Arm.Prot*cov);
          /* Pravg is only for informing the user. It has no game effect */
       }
       return;
    }
    /* Couldn't wear it */
}

