/* ==== globals.c ==== */
/* Declarations for these will be in specific headers */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */



#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "dXrng.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "player_t.h" /* Just the typedefs, maam */


/* --- Would have been in ajoin.c --- */

#ifdef MARK_INTERSECTIONS
/* Used to be static, but then you can't free it so easily... */
/* Is *declared* in item_p.h */
itemtype_t *corner_marker=NULL;
#endif


/* --- Would have been in crash.c --- */

char *bugger_ringbuffer[200];
/* Each element is a pointer to a single bug (as logged by bugger()).
   When all elements are used up, the earliest are replaced. */

unsigned short brb_startidx=0; /* Index to earliest element in bugs
  ringbuffer. Start reading from here. */

unsigned short brb_endidx=0; /* Index to latest element in bugs ringbuffer.
  Add new debug messages here. */

unsigned short watchflags; /* Flags to indicate what subjects we're
  particularly interested in logging */

/* --- Would have been in ctrl.c --- */

unsigned int randCycle=999; /* acts as a random number generator purely
  for the purposes of things that happen with regular frequency, EG:
  getting hungry, regenerating. number gets incremented by largish number
  and is then %1000; value is then compared to cycledur, to see if the
  equivalent of 1 move has passed. */


/* ---  dXrng.c stuff should be mostly private/static... --- */
/* ---  except the new dice array vars ! --- */

dXrng_roller *d4set[8]; /* 8d4 gives avg ~16 */
dXrng_roller *d6set[5]; /* 5d6 gives avg ~15 */
dXrng_roller *d8set[4]; /* 4d8 gives avg ~16 */
dXrng_roller *d10set[3]; /* 3d10 gives avg ~15 */
dXrng_roller *d12set[3]; /* 3d12 gives avg ~18 */
dXrng_roller *d20set[2]; /* 2d20 gives avg 20 */


/* --- Were in item.c --- */

itemtype_t *first_its[IG_LASTGROUP];
/* An array of pointers to initial item types */

int it_nums[IG_LASTGROUP];
/* The number of different types in each group */

unsigned int it_freqtot[IG_LASTGROUP];
/* The sum of the frequencies of items, for each group. For
 calculating which items to randomly generate. */

char invletter_array[13];
/* Bits set in this array indicate letters that ARENT used */

item_t *phonyitem;
/* prompt_for_inv_item(1) returns this when player presses '-' */

/* --- Were in los.c --- */

char los_vistab[21*21];
/* Each element is 1 if visible, and 0 if not */

char los_blocktab[21*21];
/* Used in updating LOS each turn. Set up by init_los() (not yet implemented*/


/* --- Was briefly in main.c --- */

levelmap_t *firstlevel;
/* Um... First level, doesn't matter where it is as all levels are in
 a list and this will link to them? */

levelmap_t *thislevel;
/* The level we're on! */

levelmap_t *lastlevel=NULL;
/* Used by map generator in order to link back to where you were
 when you move into a new level */

levelstructs_t *tls;
/* The (unpacked) structs of the level we're on: thislevel->Unpacked */
/* WILL HAVE TO GO OR CHNAGE! */

leveltype_t thislevtype;
/* enumerated type for what SORT of level we're on. */

int dlev=0; /* (phony, temp) level of dungeon we're at. 0 before
 creating first level; 1 after that level's created. Result:
 initial batch of monsters should all be L0, but subsequent monsters
 should be mix of L0 and L1, until player gains levels.
 Eventually this'll be stored in area_t. Shouldn't be stored in
 level structure, as the separate areas thing should allow for monsters
 of different level on the same dungeon level */
int clev=0; /* level to create new monsters at. SHOULD be an
 average of player level and dungeon level.*/

/* --- Were in maparray.c --- */

foogrid *map=NULL;
/* The map array for current level (may change implementation) */

foogrid *tmap=NULL;
/* Array of pointers to tunnel sections */

/* roomdata_t room[217];   BEING PHASED OUT AND REPLACED
   The array of room data structures.
   REPLACING WITH tls->Room[] */

/* area_t *firstarea=NULL;   BEING PHASED OUT AND REPLACED
  Pointer to the first area of list.
  REPLACING WITH tls->Firstarea. */


/* --- (memutils.c has only *static* vars) --- */

/* --- From monster.c --- */

mobtype_t *first_mt=NULL;
int mt_nums=0;

/* Will need structures to store each monster instance so we
   can actually find them all to move them... */
/* Will be changed to array of lists for each monster symbol */


/* --- Were in player.c --- */

player_t player;
/* The player character's data */

int pstype;
/* Describes the type of thing the player is on:
 0=tunnel region, 1=room, 2=doorway.
 Set with p_whatamion() each turn, for use by
 the AI system, possibly other uses too */

region_t *psregion;
/* The region the player is in, if any */

int psroomno;
/* The room the player is in, if any */

int quit=0;
/* 0 whilst playing, set when game is ending */

/* --- Would be in race.c --- */

unsigned long  ex_nx[27]; /* actually stores 26 vals, but should start at 1*/
/* experience needed between L-1 and L, in calculating values for: */
unsigned long  ex_tot[27];
/* total experience needed to reach a level. DO NOT ACCESS DIRECTLY,
use global_scale_expreq() from race_p.h instead */

/* --- Were in ui.c --- */

char strbuf[160];
/* A buffer for sprintf() to write strings into */


char *messages_ringbuffer[200];
/* Each element is a pointer to a single message (as shown by show_messages).
   When all elements are used up, the earliest are replaced. */

unsigned short mrb_startidx=0; /* Index to earliest element in messages
  ringbuffer. Start reading from here. */

unsigned short mrb_endidx=0; /* Index to latest element in messages ringbuffer.
  Add messages here. */

int stalePrompt=1; /* when set, draw_status must draw/clear the prompt area*/

/* --- Think that's all now --- */
