/* ==== graphics.c ==== */
/* haha, fooled you. It's not really graphics, still ASCII. */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include "syscurses.h"
#include <string.h>
#include <stdio.h>
#include "porrog_defs.h"
#include "graphics_p.h"
#include "crash.h"

unsigned char bitblock[250]; /* gives 80*25 bits */
/* we don't get endianness issues within individual
*bytes*, do we? */

void loadPBMFile(char *filename) {
   /* NOTE: older version read PBMs minus the header. Now we use real ones.*/
   int i,ip;
   FILE *pbmfile;
   char charbuf[25];
   if(filename==NULL) {
      bugger(REPTYPE_CODE,LOGLEVEL_FOOBAR,"loadPBMFile() given NULL filename");
      return;
   }
   if(strlen(filename)>14) {
      bugger(REPTYPE_CODE,LOGLEVEL_FOOBAR,"loadPBMFile(): filename too long");
      return;
   }
   sprintf(charbuf,"screens/%s",filename);
   pbmfile=fopen(charbuf,"r");
   if (pbmfile==NULL) {
      bugger(REPTYPE_FILE,LOGLEVEL_WARNING,"loadPBMFile() couldn't load file! Making up nonsense image");
      /* now make up nonsense image. Make mostly blank, with
      some guff in the middle */
      for(i=0;i<122;i++) bitblock[i]=0;
      for(i=122;i<127;i++) bitblock[i]=255;
      for(i=127;i<240;i++) bitblock[i]=0;
   } else {
      /* get header, check it. */
      for(i=0;i<9;i++) {
         ip=getc(pbmfile);
         if(ip!=EOF) charbuf[i]=ip;
         else {
            bugger(REPTYPE_FILE,LOGLEVEL_WARNING,"loadPBMFile() read corrupted file-Under 9 bytes long!");
            return;
         }
      }
      if((strncmp(charbuf,"P4",2)!=0)||(strncmp(charbuf+3,"80 24",5)!=0)) {
         bugger(REPTYPE_FILE,LOGLEVEL_WARNING,"loadPBMFile() given unsuitable file. See screens/README for format.");
         return;
      }
      for(i=0;i<240;i++) {
         ip=getc(pbmfile);
         if(ip!=EOF) bitblock[i]=ip;
         else {
            bugger(REPTYPE_FILE,LOGLEVEL_WARNING,"loadPBMFile(): unexpected EOF in file!");
            return;
         }
      }
      ip=getc(pbmfile);
      if(ip!=EOF) bugger(REPTYPE_FILE,LOGLEVEL_WARNING,"loadPBMFile(): file appears unnecessarily long? Ignoring extra bytes.");
   }
}

#define GETBITFROMCHAR(pos,octet) (0<(octet & (128>>pos)))
/* MSB is leftmost, and therefore considered pos 0 here.
   LSB is rightmost, and therefore considered pos 7 here.
   Unusual, but right for us. */

#define GETBITFROMCOORDS(x,y) GETBITFROMCHAR(((x)&7),bitblock[(10*(y))+((x)>>3)])

char sym[256];

void displayBitmap() { /* just displays whats in bitblock[]. */
   int here,around[8];
   int x,y,i;
   unsigned int testblock;
   for(y=1;y<23;y++) {
      move(y,0);addch('|');
      move(y,79);addch('|');
   }
   for(x=1;x<79;x++) {
      move(0,x);addch('-');
      move(23,x);addch('-');
   }
   move(0,0);addch('+');move(0,79);addch('+');
   move(23,0);addch('+');move(23,79);addch('+');
   for(y=1;y<23;y++) {
      for(x=1;x<79;x++) {
         move(y,x);
         here=GETBITFROMCOORDS(x,y);
         if(!here) {addch(' ');continue;}
         testblock=0;
         /* otherwise, consider surroundings */
         /* remapped */
         around[7]=GETBITFROMCOORDS(x-1,y-1);
         around[6]=GETBITFROMCOORDS(x,y-1);
         around[5]=GETBITFROMCOORDS(x+1,y-1);
         around[4]=GETBITFROMCOORDS(x+1,y);
         around[3]=GETBITFROMCOORDS(x+1,y+1);
         around[2]=GETBITFROMCOORDS(x,y+1);
         around[1]=GETBITFROMCOORDS(x-1,y+1);
         around[0]=GETBITFROMCOORDS(x-1,y);
         for(i=0;i<8;i++) testblock|=((1&around[i])<<i);
         /* now, process...

         765
         4H3  the bit positions.
         210
          DOH! to allow shifts to rotate the mapping,
          the mapping must be *clockwise*.
         765
         0H4
         123

         ???
         010  - 2 or 3: V
         000

         100
         01? - 0,1,2 : \
         00?

         ?00
         ?10  - 2 or 3: >
         ?00

         ?10
         110  - /
         000

         101
         010  - V
         010

         ?1?
         111  - 2 or less: +  3,4: #
         ?1?

         1?1
         ?1?  - 2 or less: X  3,4: #
         1?1

         others???

         calculate symbols for all combinations with
         bits set in top left and/or top middle position.
         ALL other combinations can be considered a rotation
         of those by 90 degrees, = a shift of 2 bits. */
         addch(sym[testblock]);
      }
   }
   refresh();
}

void graphicsPrepareMapping() {
    /* takes no arguments, sets up a translation table */
    /* 765  7654 3210
       4H3
       210 */
    int i,shifting,shifted;
    int bitcount[256];
    for(i=0;i<256;i++) {
       bitcount[i]=(0<(i&128))+(0<(i&64))+(0<(i&32))+(0<(i&16))\
         +(0<(i&8))+(0<(i&4))+(0<(i&2))+(0<(i&1));
    }
    for(i=64;i<256;i++) { /* all with bits 6 and/or 7 set */
       /* remapped */
       if((i&85)==85) { /* 90= 0101 1010, =+ sign */
         if(bitcount[i]>6) sym[i]='#';
         else sym[i]='+';
         continue;
       }
       /* remapped */
       if((i&170)==170) { /* 165= 1010 0101, =X sign */
         if(bitcount[i]>6) sym[i]='#';
         else sym[i]='X';
         continue;
       }
       switch(bitcount[i]) {
          case 0:
            bugger(REPTYPE_CODE,LOGLEVEL_FOOBAR,"graphicsPrepareMapping() found a bitpattern with 0 bits?");
            crash(FUNCTION_ERROR);
            break;
          case 1:
            /* premapped */
            if(i==64) sym[i]='|';
            else sym[i]='\\';
            break;
          case 2:
            /* pos 7 set */ /* premapped+remapped */
            if(bitcount[i&224]==2) {sym[i]='V';break;}  /* 224= 1110 0000*/
            if(bitcount[i&156]==2) {sym[i]='\\';break;} /* 139= 1000 1011*/
            if(bitcount[i&131]==2) {sym[i]='>';break;}  /* 148= 1001 0100 */
            /* else pos 6 must be set */ /* remapped */
            if(bitcount[i&67]==2)  {sym[i]='/';break;}  /* 84= 0101 0100 */
            if(bitcount[i&88]==2)  {sym[i]='\\';break;} /* 73= 0100 1001 */
            if(bitcount[i&68]==2)  {sym[i]='|';break;}  /* 66= 0100 0010 */
            bugger(REPTYPE_CODE,LOGLEVEL_FOOBAR,"couldnt find 2bit bitpattern!");
            crash(FUNCTION_ERROR);
            break;
          case 3:
            /* premapped */
            switch(i&192) {
               case 64:   /* top */
                 if(bitcount[i&14]==2) {sym[i]='A';break;}/* 64|7= 0100 0111 */
                 /* 73:\  82: /  98: /  104: \??  112:\ */
                 if(i==81) {sym[i]='-';break;} /* inv T:alt= +? */
                 if(i==88||i==73||i==104||i==112)  {sym[i]='\\';break;}
                 if(bitcount[i&3]==2)  {sym[i]='/';break;}
                 if(i==82||i==97||i==98) {sym[i]='/';break;}
                 if(i&4) {sym[i]='|';break;}
                 sym[i]='3';break;
               case 128:  /* corner */
                 /* 131: >  133: \  134: \  145: - 148: \
                    161: /  162: /  176: \  */
                 if((i&8)||i==133||i==134||i==148||i==176) {
                     sym[i]='\\';break;
                 } /* ALL such patterns?!*/
                 if(i==164) {sym[i]='V';break;}
                 if(i==161||i==162) {sym[i]='/';break;}
                 if(i==146||i==131) {sym[i]='>';break;}
                 if(i==145) {sym[i]='-';break;}
                 sym[i]='3';break;
                 break;
               case 192:  /* both: one bit somewhere else  */
                 /* 194: /  196: |  200: \  208: \ 224: V */
                 if(i==193||i==194) {sym[i]='/';break;}
                 if(i==200||i==208) {sym[i]='\\';break;}
                 if(i==196) {sym[i]='|';break;}
                 if(i==224) {sym[i]='V';break;}
                 sym[i]='3';break;
                 break;
            }
            break;
          case 4: /* remapped */
            if(i&136 && bitcount[i&85]==2) {sym[i]='\\';break;}
            /* lots of combinations give that */
            sym[i]='4'; break;
          case 5:
            /* THOUGHT: only combinations not already cought would
            EITHER: have two side bits missing, or two corner bits
            missing. Forget the last bit. ?*/
            /* remapped */
            if(bitcount[i&170]==2) {
               /* missing 2 corners */ /* remapped */
               if ((i&130)==0||(i&40)==0) {sym[i]='|';break;}
               if ((i&160)==0||(i&10)==0) {sym[i]='-';break;}
               if ((i&136)==0) {sym[i]='/';break;}
               if ((i&34)==0) {sym[i]='\\';break;}
            } else {
               /* missing 2 sides */ /* remapped */
               if((i&17)==0) {sym[i]='|';break;} /* or '#'? */
               if((i&68)==0) {sym[i]='-';break;} /* or '#'? or 'X'? */
               /* missing sides are not opposite. Treat 2 opposite
               corners as ends of a diagonal. */ /* remapped */
               if((i&136)==136) {sym[i]='\\';break;}
               if((i&34)==34) {sym[i]='/';break;}
               /* if(((i&18)==0||(i&72)==0)&&((i&129)==129)) */
            }
            sym[i]='5';break;
          case 6:
            sym[i]='#';break; /* well, good as */
          default:
            sym[i]='?';break; /* VERY iffy */
       }
       /* ends switch */
    }
#if 0
    /* debug tables */
    printf("graphicsPrepareMapping(): listing char mappings (for debugging)\n");
    for(i=64;i<256;i++) printf("%3d: %c  ",i,sym[i]);
    printf("\n");
#endif
       /* now work out versions for other rotations. */
       /* blah blah. */
    sym[0]='+';
    for(i=1;i<64;i++) {
       for(shifting=1;shifting<6;shifting++) {
          shifted=(i<<(shifting*2));
          if (0<(shifted&192)) {
             sym[i]=sym[shifted];
             if(shifting==2) {
                switch(sym[i]) {
                  case 'V': sym[i]='A';break;
                  case 'A': sym[i]='V';break;
                  case '<': sym[i]='>';break;
                  case '>': sym[i]='<';break;
                  default:break;
                }
                break; /* 180 degrees, usually do nothing */
             }
             switch(sym[i]) {
                case '|':sym[i]='-';break;
                case '-':sym[i]='|';break;
                case '/':sym[i]='\\';break;
                case '\\':sym[i]='/';break;
                case 'V':sym[i]=(shifting==1)?'<':'>';break;
                case 'A':sym[i]=(shifting==1)?'>':'<';break;
                case '<':sym[i]=(shifting==1)?'A':'V';break;
                case '>':sym[i]=(shifting==1)?'V':'A';break;
                default:break;
             }
             break;
          } /* else keep shifting! */
       }
       if(shifting>3) printf("%d had to be shifted more than 3...\n",i);
    }
#if 0
    /* debug tables continued */
    for(i=0;i<64;i++) printf("%3d: %c  ",i,sym[i]);
    printf("\n");
#endif
}
