/* ==== i_init.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */



/* This initialises the set of item types. Still not sure if such
   data should be inside the .c source or a header... No, it
   would make no sense to put it in a header, as it would only
   be #included once. */

/* Need helper functions that do more than just "new_object_type",
 and define type-specific things */

#include <stddef.h>
#include "syscurses.h" /* Shouldn't be here */
#include "porrog_defs.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "C-foogrid.h" /* dammit we HAVE to separate protos from typedefs. */

/* Protos */
#include "crash.h"
#include "item_p.h"
#include "potions.h"
#include "memutils.h"

#define DEF_WEAP(name,tht,dam) \
       td=push_itype(IG_WEAPON,name,100,1), \
       td->Typedata.Weapon.Tht=tht,td->Typedata.Weapon.Dmg=dam;
#define DEF_BOW(name) push_itype(IG_RANGED,name,80,1);
#define DEF_MISSILE(name) push_itype(IG_MISSILE,name,12,1);

/* Coverage percentages:
  SUIT=40%.   SHIELD=15%,   HEADGEAR=15%,
  GLOVES=8%   FOOTWEAR=20%, NECKWEAR=2%.
  Rings have no coverage (same place as gloves). */
/* Armour items */
#define DEF_SUIT(td,name,cov,prot) \
       td=push_itype(IG_SUIT,name,170,1), \
       td->Typedata.Arm.Cov=(cov*0.4), td->Typedata.Arm.Prot=prot;
#define DEF_SHIELD(td,name,cov,prot) \
       td=push_itype(IG_SHIELD,name,90,1), \
       td->Typedata.Arm.Cov=(cov*0.15), td->Typedata.Arm.Prot=prot;
#define DEF_HEADGEAR(td,name,cov,prot) \
       td=push_itype(IG_HEADGEAR,name,20,1), \
       td->Typedata.Arm.Cov=(cov*0.15), td->Typedata.Arm.Prot=prot;
#define DEF_GLOVES(name,cov,prot) \
       td=push_itype(IG_GLOVES,name,20,1), \
       td->Typedata.Arm.Cov=(cov*0.08), td->Typedata.Arm.Prot=prot;
#define DEF_FOOTWEAR(td,name,cov,prot) \
       td=push_itype(IG_FOOTWEAR,name,40,1), \
       td->Typedata.Arm.Cov=(cov*0.2), td->Typedata.Arm.Prot=prot;

/* Jewellery */
#define DEF_RING(name) push_itype(IG_RING,name,2,1);
#define DEF_NECKWEAR(name,cov,prot) \
       td=push_itype(IG_NECKWEAR,name,3,1), \
       td->Typedata.Arm.Cov=(cov*0.02), td->Typedata.Arm.Prot=prot;

/* Readables */
#define DEF_SCROLL(name)  push_itype(IG_SCROLL,name,10,1);
#define DEF_BOOK(name) push_itype(IG_BOOK,name,60,1);

/* Comestibles */
#define DEF_FOOD(name) push_itype(IG_FOOD,name,30,1);
#define DEF_CONDIMENT(name) push_itype(IG_CONDIMENT,name,20,1);
#define DEF_POTION(really,effect,descr,prerec,applic) \
       td=push_itype(IG_POTION,really,30,1), \
       td->Typedata.Potion.Effect=effect, \
       td->Typedata.Potion.Dname=descr, \
       td->Typedata.Potion.Prerec=prerec, \
       td->Typedata.Potion.CanApply=applic;
/* Zappables */
#define DEF_WAND(name)  push_itype(IG_WAND,name,10,1);
#define DEF_STAFF(name) push_itype(IG_STAFF,name,120,1);
#define DEF_ROD(name)   push_itype(IG_ROD,name,30,1);
/* weight default to 90 for typical tool */
#define DEF_TOOL(name,group,data,weight,prob)  \
       td=push_itype(IG_TOOL,name,weight,prob), \
       td->Typedata.Tool.Group=group, td->Typedata.Tool.Subtype=data;
#define DEF_CONTAINER(name,opens,wlim,iclass,extype) \
       td=push_itype(IG_CONTAINER,name,1,1), \
       td->Typedata.Container.Opens=opens, \
       td->Typedata.Container.Wlim=wlim, \
       td->Typedata.Container.AcceptsClass=iclass, \
       td->Typedata.Container.AcceptsSpec=extype;
#define DEF_STONE(name)  push_itype(IG_STONE,name,75,1);
/* say a coin is the lightest thing, and it weighs 10 grams */
#define DEF_COINAGE(name)  push_itype(IG_COINAGE,name,1,1);

void i_init()
{
    itemtype_t *td;
    int i;
    for(i=0;i<IG_LASTGROUP;i++) {/*Mar 2004- They were uninitialised before!*/
        if(i==IG_LASTGROUP) crash(FUNCTION_ERROR);
        it_nums[i]=0;
        it_freqtot[i]=0;
        first_its[i]=NULL;
    }
    phonyitem=(item_t *)counting_malloc(MEM_ITEM,sizeof(item_t));
    phonyitem->Itemtype=NULL; /* Number element used to
    describe whether to unwield or unready */
    phonyitem->Number=0; /* required by the wielding code to work out
    what item to unwield. Which is stupid. Going to change this system
    some time anyway. */
#ifdef MARK_INTERSECTIONS
    /* This is freed by free_everything() */
    corner_marker=(itemtype_t *)counting_malloc(MEM_ITYPE,sizeof(itemtype_t));
    newItemtype(corner_marker,'X',PCOL_BBLUE,"Intersection marker",IG_STONE,1000,0);
    /* WAS COLO:R_BLUE+9 */
#endif
    /* dummy items: only one created, not manipulatable */
    td=push_itype(IG_DUMMY,"Closed Door",0,0);
    td->Typedata.DummyInst=newItem(td,1);
    td->Symbol='+',td->Col=PCOL_BROWN;
      /* brown, not bright yellow. WAS COLO:R_YELLOW+1 */
    /* Wieldables/wearables */
    /* NOTE: Weapons should take target size into account:
    Some weapons, eg knife, are more manouevrable, so good against small,
    fast-moving targets, whereas others, eg sledgehammer, are harder to
    direct, but have very good reach, so are better against big slow targets.*/
    DEF_WEAP("sword",3,5);
    DEF_WEAP("axe",2,8);
    DEF_WEAP("spear",4,5);
    DEF_WEAP("club",3,4);
    DEF_WEAP("knife",2,3);
    DEF_WEAP("claw hammer",2,6);
    DEF_WEAP("mallet",1,5);
    DEF_WEAP("sledgehammer",3,10);
    DEF_WEAP("exacto knife",1,3);
    DEF_WEAP("penknife",2,2);
    DEF_WEAP("hatchet",2,5);
    DEF_WEAP("cleaver",3,4);
    DEF_WEAP("whip",4,3);

    DEF_BOW("bow");
    DEF_BOW("crossbow");

    DEF_MISSILE("arrow");
    DEF_MISSILE("bolt");

    DEF_SUIT(td,"pinstripe suit",99,1);
    DEF_SUIT(td,"boiler suit",90,2);
    DEF_SUIT(td,"gimp suit",99,2);
    DEF_SUIT(td,"leather armour",70,3);
    DEF_SUIT(td,"chain mail",90,4);
    DEF_SUIT(td,"plate mail",85,6);
    DEF_SUIT(td,"kevlar body armour",55,8);

    DEF_SHIELD(td,"small shield",30,4);
    DEF_SHIELD(td,"large shield",70,6);
    DEF_SHIELD(td,"riot shield",80,5);

    DEF_HEADGEAR(td,"paper hat",30,1);
    DEF_HEADGEAR(td,"paper crown",25,1);
    DEF_HEADGEAR(td,"flat cap",30,2);
    DEF_HEADGEAR(td,"beret",30,2);
    DEF_HEADGEAR(td,"tam-o'-shanter",30,2);
    DEF_HEADGEAR(td,"wooly hat",40,2);
    DEF_HEADGEAR(td,"bobble hat",40,2);
    DEF_HEADGEAR(td,"leather cap",30,3);
    DEF_HEADGEAR(td,"peaked cap",35,2);
    DEF_HEADGEAR(td,"skull cap",20,1);
    DEF_HEADGEAR(td,"baseball cap",35,1);
    DEF_HEADGEAR(td,"spiked helmet",45,6);
    DEF_HEADGEAR(td,"crash helmet",75,5);
    DEF_HEADGEAR(td,"hard hat",45,5);
    DEF_HEADGEAR(td,"mortarboard",30,1);
    DEF_HEADGEAR(td,"headscarf",40,1);
    DEF_HEADGEAR(td,"wig",45,1);
    DEF_HEADGEAR(td,"davy crocket hat",40,2);
    DEF_HEADGEAR(td,"deerstalker",50,2);
    DEF_HEADGEAR(td,"fez",38,2);
    DEF_HEADGEAR(td,"top hat",30,2);
    DEF_HEADGEAR(td,"trilby",32,1);
    DEF_HEADGEAR(td,"bowler hat",35,2);
    DEF_HEADGEAR(td,"steel helmet",85,6);
    DEF_HEADGEAR(td,"crown",40,4);

    DEF_GLOVES("fingerless gloves",55,1);
    DEF_GLOVES("mittens",90,2); /* Should reduce dex? Or evasion? */
    DEF_GLOVES("gloves",85,1);
    DEF_GLOVES("rigger's gloves",95,3);
    DEF_GLOVES("kevlar gloves",95,7);
    DEF_GLOVES("welder's gloves",100,4);/*should that be welder's gauntlets?*/
    DEF_GLOVES("gauntlets",100,6);

    /* Good thing armour doesnt stack because neither do these names */
    DEF_FOOTWEAR(td,"socks",35,1);
    DEF_FOOTWEAR(td,"stockings",70,1);
    DEF_FOOTWEAR(td,"slippers",25,3);
    DEF_FOOTWEAR(td,"slipper-socks",30,2);
    DEF_FOOTWEAR(td,"plimsolls",25,3);
    DEF_FOOTWEAR(td,"sandals",15,2);
    DEF_FOOTWEAR(td,"clogs",25,4);
    DEF_FOOTWEAR(td,"trainers",30,3);
    DEF_FOOTWEAR(td,"shoes",30,3);
    DEF_FOOTWEAR(td,"low boots",32,4);
    DEF_FOOTWEAR(td,"high boots",40,4);
    DEF_FOOTWEAR(td,"cowboy boots",50,4);
    DEF_FOOTWEAR(td,"thigh boots",65,4);
    DEF_FOOTWEAR(td,"waders",90,3);
    DEF_FOOTWEAR(td,"wellies",50,5); /* Should reduce AGI or evasion */
    DEF_FOOTWEAR(td,"roller skates",30,2); /* Hmm, AGI also represents
    balance,etc , as well as speed. Roller skates should make you a bit
    clumsy but sometimes faster. Don't know how to do that */
    DEF_FOOTWEAR(td,"roller blades",35,2); /* Like roller skates */
    DEF_FOOTWEAR(td,"ice skates",35,2); /* Ditto, but only any use on ice */
    DEF_FOOTWEAR(td,"steel toecapped boots",40,6);

    DEF_RING("ring");
    DEF_RING("magic ring"); /* what more do I do for these??? */

    DEF_NECKWEAR("necklace",0,0);
    DEF_NECKWEAR("amulet",0,0);
    DEF_NECKWEAR("torc",75,2);/* If neckwear is 2% covg, mebbe the
    coverage numbers should be out of 1000 instead to stop underflow */
    /* End of wieldables/wearables */

    /* Readables... Most of these are "flavoured" items */
    DEF_SCROLL("scroll"); /* er...*/

    DEF_BOOK("textbook");
    DEF_BOOK("novel"); /* just random stuff?*/
    DEF_BOOK("handbook"); /* gives some skills */
    DEF_BOOK("manual"); /* other skills*/
    DEF_BOOK("grimoire"); /* gives some spells */
    DEF_BOOK("codex"); /* other spells */
    /* End of readables */

    /* Comestibles!*/
    /* For most foods, thought must be given to how to number them */
    DEF_FOOD("sandwich");
    DEF_FOOD("scotch egg");
    DEF_FOOD("hard boiled egg");
    DEF_FOOD("pickle"); /* would this interfere with condiments?*/
    DEF_FOOD("fish"); /*should this be subdivided?*/
    DEF_FOOD("meat");
    DEF_FOOD("apple");
    DEF_FOOD("pancake");
    DEF_FOOD("orange");
    DEF_FOOD("doner kebab");
    DEF_FOOD("hamburger");
    DEF_FOOD("bread"); /* But *loaf* or *slice*? */
    DEF_FOOD("toast"); /* Must be slice, yes? */
    DEF_FOOD("banana");
    DEF_FOOD("shish kebab"); /* perhaps this could leave a skewer! */
    DEF_FOOD("noodles");
    DEF_FOOD("sushi");
    DEF_FOOD("meat pie");
    DEF_FOOD("quiche");
    DEF_FOOD("pizza");
    DEF_FOOD("pineapple");
    DEF_FOOD("soup"); /* things like these should have containers */
    DEF_FOOD("curry");
    DEF_FOOD("balti");
    DEF_FOOD("ice cream");
    DEF_FOOD("spam");
    DEF_FOOD("corned beef");
    DEF_FOOD("spaghetti");
    DEF_FOOD("rice");
    DEF_FOOD("packet of crisps");
    DEF_FOOD("banana chips");
    DEF_FOOD("bombay mix");
    DEF_FOOD("meatball");
    DEF_FOOD("haggis");
    DEF_FOOD("durian");
    DEF_FOOD("ham"); /* Specific meats */
    DEF_FOOD("gammon");
    DEF_FOOD("bacon");
    DEF_FOOD("beef");
    DEF_FOOD("steak");
    DEF_FOOD("lamb");
    DEF_FOOD("leg of lamb");
    DEF_FOOD("mutton");
    DEF_FOOD("pork chop");
    DEF_FOOD("lamb chop");
    DEF_FOOD("chicken breast");
    DEF_FOOD("turkey breast");
    DEF_FOOD("chicken leg");
    DEF_FOOD("turkey leg");
    DEF_FOOD("chicken wing");
    DEF_FOOD("turkey wing");
    DEF_FOOD("horse meat");  /* takes all sorts */
    DEF_FOOD("dog meat");  /* hmm */
    DEF_FOOD("dog food");
    DEF_FOOD("cat food");
    DEF_FOOD("soylent brown"); /* hehe... will have to include the
                                  story for this :) */

    /* Corpse types should be defined by monster definitions?
       Should corpses just be lumped in with food (which means normal food)?*/
    DEF_CONDIMENT("salt");
    DEF_CONDIMENT("black pepper");
    DEF_CONDIMENT("white pepper");
    DEF_CONDIMENT("szechuan pepper");
    DEF_CONDIMENT("cayenne pepper");
    DEF_CONDIMENT("malt vinegar");
    DEF_CONDIMENT("spirit vinegar");
    DEF_CONDIMENT("balsamic vinegar"); /* think that's all vinegars*/
    DEF_CONDIMENT("english mustard");
    DEF_CONDIMENT("french mustard");
    DEF_CONDIMENT("wholegrain mustard");
    DEF_CONDIMENT("dijon mustard"); /* Any other mustards I forgot?*/
    DEF_CONDIMENT("horseradish");
    DEF_CONDIMENT("japanese horseradish"); /* wasabi? */
    DEF_CONDIMENT("coriander");
    DEF_CONDIMENT("basil"); /* there are a few types of basil*/
    DEF_CONDIMENT("oregano");
    DEF_CONDIMENT("parsley");
    DEF_CONDIMENT("thyme");
    DEF_CONDIMENT("lemongrass");
    DEF_CONDIMENT("paprika");
    DEF_CONDIMENT("cinnamon");
    DEF_CONDIMENT("ginger");
    DEF_CONDIMENT("nutmeg");
    DEF_CONDIMENT("anise");
    DEF_CONDIMENT("cardamom");
    DEF_CONDIMENT("cloves"); /* do these really count as condiments? */
    DEF_CONDIMENT("garlic powder");
    DEF_CONDIMENT("powdered onion"); /* to distinguish from any food onion */
    DEF_CONDIMENT("grated cheddar");
    DEF_CONDIMENT("grated parmesan");
    DEF_CONDIMENT("sliced lemon peel");
    DEF_CONDIMENT("pesto"); /* yes I know theres lots of types */
    DEF_CONDIMENT("salsa");
    DEF_CONDIMENT("spaghetti sauce");
    DEF_CONDIMENT("tomato ketchup");
    DEF_CONDIMENT("brown sauce");
    DEF_CONDIMENT("HP sauce");
    DEF_CONDIMENT("soy sauce");
    DEF_CONDIMENT("worcestershire sauce");
    DEF_CONDIMENT("tabasco sauce");
    DEF_CONDIMENT("chilli sauce");
    DEF_CONDIMENT("lemon juice");
    DEF_CONDIMENT("lime juice");

    /* Some potions should start off known. The player would
    probably know only some. What he recognises would not be
    made clear to him. If the player starts off *not* recognising
    such a potion, that potion would take on a different name
    as given below. If the player DID recognise the potion from
    the beginning, those alternative names could be used by OTHER
    potions. You might only find out that a "clear potion" wasn't
    water when you find a "potion of water". */

    /* params:(realthing,effectfn,descr,prerec,applic)
    Where descr is what it (might) look like, prerec is probability
    (out of 10) that you already know the potion as itself (non-zero
    values of prerec mean the potion will *otherwise* be fixed with
    descr). applic is flag for if potion can be specially applied */
    DEF_POTION("water",water_potion_effect,"clear potion",5,0);
    DEF_POTION("orange juice",water_potion_effect,"orange potion",6,0);
    DEF_POTION("cola",water_potion_effect,"black potion",3,0);
    DEF_POTION("lemonade",water_potion_effect,"fizzy potion",5,0);
    DEF_POTION("booze",water_potion_effect,"clear brown potion",4,0);
    DEF_POTION("tea",water_potion_effect,"opaque brown potion",7,0);/*tan?*/
    DEF_POTION("coffee",water_potion_effect,"murky potion",8,0);
    /* NB: yes, everyone can recognise water or booze... but there'd
    be other clear liquids that you might not be able to differentiate
    from water, or alcohol-based potions you might not be able to
    differentiate from booze */

    /* Unknown types should (??) be distinguished by a flag that makes
    the word "potion" go either at the end, or at the beginning with
    the word "of" ... give this thought */
    DEF_POTION("magic potion",magic_potion_effect,"purple potion",0,0);
    /* should just be poison */
    DEF_POTION("potion of poison",poison_potion_effect,"green potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"blue potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"yellow potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"red potion",0,0);
    /* should be nasty poison */
    DEF_POTION("potion of nasty poison",poison2_potion_effect,"turquoise potion",0,0);
/*    DEF_POTION("orange potion"); Known potions now handled differently */
/*    DEF_POTION("brown potion"); Depends if we use "brown" for known potions*/
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"violet potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"pink potion",0,0);
    DEF_POTION("healing potion",healing_potion_effect,"viscous potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"oily potion",0,0);
    DEF_POTION("ink",ink_potion_effect,"inky potion",3,1);/* refill pens */
/*  DEF_POTION("fizzy potion"); */
    DEF_POTION("bubbling ineffectual potion",ineffectual_potion_effect,"bubbling potion",0,0);
    DEF_POTION("fuming ineffectual potion",ineffectual_potion_effect,"fuming potion",0,0);
    DEF_POTION("cloudy ineffectual potion",ineffectual_potion_effect,"cloudy potion",0,0);
/*  DEF_POTION("murky potion"); */
/*  DEF_POTION("clear potion"); */
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"opaque potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"grey potion",0,0);
/*  DEF_POTION("black potion"); */
    DEF_POTION("sparkling ineffectual potion",ineffectual_potion_effect,"sparkling potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"glowing potion",0,0);
    /* should be experience potion? */
    DEF_POTION("potion of experience",exp_potion_effect,"flashing potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"swirly potion",0,0);
    DEF_POTION("ineffectual potion",ineffectual_potion_effect,"pulsing potion",0,0);



    /* SHOULD PROLLY BE CALLED ELSEWHERE! */
    shuffle_potions(); /* Like AFTER class has chance to set preknown types*/



    /* Zappables... Again, flavoured items. */
    DEF_WAND("magic wand"); /* uh... */
    DEF_STAFF("magic staff"); /* :P */
    DEF_ROD("magic rod"); /* ... */
    /* end of zappables */

    /* The rest... */ 
    DEF_TOOL("broken key",      TOOL_KEY,0,7,12); /* was just key */
    DEF_TOOL("nondescript key", TOOL_KEY,1,7,18);
    DEF_TOOL("squiggly key",    TOOL_KEY,2,7,3);
    DEF_TOOL("zigzag key",      TOOL_KEY,3,7,7);
    DEF_TOOL("hooked key",      TOOL_KEY,4,7,3);
    DEF_TOOL("triangular key",  TOOL_KEY,5,7,6);
    DEF_TOOL("helical key",     TOOL_KEY,6,7,3);
    DEF_TOOL("notched key",     TOOL_KEY,7,7,5);
    DEF_TOOL("skeleton key",    TOOL_KEY,-1,7,1); /* unlocks anything, once */
    DEF_TOOL("remote control",  TOOL_REDHERRING,0,8,14); /* does nothing! :) */
    DEF_TOOL("pen",       TOOL_PEN,0,4,9);
    DEF_TOOL("screwdriver",TOOL_TOOL,0,10,5); /* repair stuff? untrap stuff? */
    DEF_TOOL("wire cutters", TOOL_TOOL,1,80,2); /* untrapping stuff */
    DEF_TOOL("scissors", TOOL_TOOL,2,15,8); /* shrug */
    DEF_TOOL("shovel", TOOL_TOOL,3,150,9);
    DEF_TOOL("bottle", TOOL_RECEPTACLE,0,20,3); /* Can be filled to get a potion! Or condiment? */
    DEF_TOOL("jar", TOOL_RECEPTACLE,1,20,1);
    DEF_TOOL("broken bottle", TOOL_RECEPTACLE,-1,18,19); /* could
           eventually be used for weapon?*/
    DEF_TOOL("xoanon", TOOL_REDHERRING,1,120,2); /* "Primitive, usu
      wooden carving of deity, believed to have fallen from heaven"
        - from my dictionary. Could eventually have religious use? */

    /* name, opens, wlim, iclass,extype */
    DEF_CONTAINER("bucket",0,100,IC_NOCLASSES,IG_STONE); /* for rocks! */
    DEF_CONTAINER("bag",0,30,IC_ALLCLASSES,IG_LASTGROUP);
    DEF_CONTAINER("paper bag",0,5,IC_COMESTIBLES,IG_COINAGE);
    DEF_CONTAINER("carrier bag",0,15,IC_ALLCLASSES,IG_LASTGROUP);
    DEF_CONTAINER("satchel",0,20,IC_READABLES,IG_FOOD);
    DEF_CONTAINER("rucksack",1,250,IC_ALLCLASSES,IG_LASTGROUP);
    DEF_CONTAINER("sack",0,75,IC_ALLCLASSES,IG_LASTGROUP);
    DEF_CONTAINER("quiver",0,20,IC_NOCLASSES,IG_MISSILE); /* for missiles */
    DEF_CONTAINER("wallet",1,10,IC_NOCLASSES,IG_COINAGE); /* for coinage */
    DEF_CONTAINER("purse",1,12,IC_VALUABLES,IG_LASTGROUP);  /* "  " */
    DEF_CONTAINER("money box",1,25,IC_NOCLASSES,IG_COINAGE); /* " " */
    DEF_CONTAINER("safe",1,70,IC_VALUABLES,IG_LASTGROUP); /* " " , heavy!*/
    DEF_CONTAINER("cardboard box",1,25,IC_ALLCLASSES,IG_LASTGROUP);
    DEF_CONTAINER("chest",1,400,IC_ALLCLASSES,IG_LASTGROUP); /* heavy! */
    DEF_CONTAINER("deposit box",1,15,IC_SMALL,IG_LASTGROUP); /* limited capacity! Not sure how to express that...*/
    DEF_CONTAINER("cruet",0,20,IC_NOCLASSES,IG_CONDIMENT); /* For condiments!!! */
    DEF_CONTAINER("mini fridge",1,40,IC_COMESTIBLES,IG_LASTGROUP); /* For food */
    DEF_CONTAINER("ice box",1,60,IC_COMESTIBLES,IG_LASTGROUP); /* " " , heavy! */
    DEF_CONTAINER("fridge freezer",1,200,IC_COMESTIBLES,IG_LASTGROUP); /* " " , V HEAVY! Hey, what about power?*/
    DEF_CONTAINER("chest freezer",1,400,IC_COMESTIBLES,IG_LASTGROUP); /* " ", V HEAVY! Ah, fudge it. */

    DEF_STONE("rock");
    DEF_STONE("pebble");
    DEF_STONE("flint");
    DEF_STONE("ore");
    DEF_STONE("gem");
    DEF_STONE("glass");

    DEF_COINAGE("gold coin"); /* gold coin */
    DEF_COINAGE("silver coin"); /* silver coin */
    DEF_COINAGE("brass coin"); /* brass coin */
    DEF_COINAGE("iron coin"); /* iron coin */
    DEF_COINAGE("thrupenny bit"); /* ;) */

}
