/* ==== ai_p.h ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */

/* Include ai_t.h before this. */

/* This header declares functions for the AI of Porrog's monsters. */

/* this header defines types used for the AI of Porrog's monsters.
   The AI basically works as a cache of data of which doors a
   monster has gone through, to speed up the monster's progress
   through the map. As such, it requires very very little processing,
   only data of which entrances lead where, etc, which gets produced
   when the map is made (since I changed the map generator to
   work in terms of regions and areas), but of course it also
   isn't effective in a map not built of rooms and tunnels
   (where some other AI function would need to be used instead).
   The scheme also doesn't really consitute an entire AI, only a
   means of getting each monster to cover as much of the dungeon
   as possible in a given amount of time, so they can find the
   player. Then they can do other stuff. */


/* Our lovely types are in ai_t.h */

/* The functions */

/* Try to put region/room into cache. If succeeds, return its pointer,
else return NULL.
 NB- If the cache element is changed/added, all the room/region's doors
 get timestamped to 1 or 2, and the room/region gets timestamped to 0.
 Any replaced entries have their doorlists and their own space freed.
 What happens if the region/room is already there? Return it anyways. */
aicache_tunnelent_t *aicache_trycacheregion(monster_t *mob,region_t *region);
aicache_rooment_t *aicache_trycacheroom(monster_t *monb, int roomno);

/* Return cache element containing specified region/room, if any.
  Is it necessary to have a version that doesn't replace elements?
  Yes! The version that doesnt replace elements is far far faster.
  'buf' flag accepts buffer entries as valid; if buf is unset, we
  ignore it (in some cases it's known to be wrong, but hasnt been
  unset yet). */
aicache_tunnelent_t *aicache_regcache(monster_t *mob,region_t *region,int buf);
aicache_rooment_t  *aicache_roomcache(monster_t *mob,int roomno,int buf);

/* Our routine- every tick:
   Find where player is (room or tunnel region: doorway counts as tunnel)
   Find where mob is, similarily.
   If player and mob are (both in room and room is the same)
    or (both in tunnel regions and region is the same)
    then mob should go straight to player!
   If (player is in tunnel and mob is in room)
    or (player is in room and mob is in tunnel)
    and region and room have a door in common,
    then mob should be in earshot, and take the route to that door
    and then to the player!
   If not, then mob should just keep going towards any target they
    may already have,
   If they don't yet have a target, they choose a door from their
    room/tunnel using the AI mechanism:
    If the room/tunnel they're in is in the AI cache, then they
    randomly pick a door from the set of doors with the highest
    timevalues. When the room/region is first entered (supposing
    it goes into the cache), the timestamp on its entry is set to
    0, and the same goes for all its doors. Every turn after that,
    all those timestamps are incremented. That set of highest
    timevalues really does mean as many doors as have the highest
    single value. When the door is gone through, its timestamp
    is reset again.
    In this way, for the places stored in the cache, the mob will
    choose to go through doors it hasn't gone through before, or
    those it went through longest ago. This should GENERALLY mean
    the mob takes less time to cover the level.
   The replacement algorithm for the cache entries is described
    well enough elsewhere. Basically it favours rooms/regions
    with more entrances, and rooms/regions the mob's been through
    most recently. It's most important to apply the caching to
    places where there are most ways to go. Where there are
    less choices, not remembering where you've been before is
    less painful.
   At all times, there should be a "monster pressure" effect, that
    acts when there are other monsters within a certain range.
    The more monsters there are, the higher the pressure. The
    position they are in makes a difference too (different squares
    are "weighted"). Squares blocked by walls, etc increase it as
    well. If this pressure is high enough (doesn't have to be very
    high) the mobs are likely to move randomly away from the
    source of the pressure. This has various good effects:
     -Mobs are unlikely to fill an area (especially stairs)
     -Mobs are unlikely to block each others' path (at least for long)
    The monster pressure should also deter breeding monsters.
    The effects of moving away from sources of monster pressure
    should last some random (but pretty small) amount of time, so
    that creatures try to get back on course at different times
    and are less likely to get in each other's way again.
   Generally, all targets a mob has should be doors (or the
    player, or sometimes items), but the mob should actually
    be aiming to go one step *past* that door, so that they
    go from a region to a room or vice-versa. If they stopped
    on the doors, they'd not be able to get a new target.
*/

void ai_act(monster_t *mob);
/* Called when it's mob's turn to do something. Basically,
   the wrapper that decides what to do, and does it. */

void ai_acquire_target(monster_t *mob, int stype);
/* Called when the mob cant find the player, and needs to
   find *some* place to go. Uses the AI to pick a random
   door that was hopefully picked least recently */

void ai_target_last_entrance(monster_t *mob, region_t *reg);
/* Called when the mob is jammed in a tunnel (not a room!?!)
   with no intersections to pass at.
   Simply find the entrance with the *lowest* timestamp
   rather than the highest!*/

void aicache_freerooments(aicache_rooment_t *head);
void aicache_freetunnelents(aicache_tunnelent_t *head);
/* Free all of a list of cache entries (or buffers) */

void aicache_freedoorlist(aicache_doorlist_t *list);
/* Clear the list of doors from a cache entry. */

void aicache_tryresetdoor(monster_t *mob, aicache_doorlist_t *list);
/* Goes through list to find door at mob's coords. There should
be one (and if not, crash) */

aicache_doorlist_t *aicache_convertdoors(entrancelist_t *list);
/* Converts a list of entrances to a list of aicache door entries,
each with age of 2, for the use of the functions that add/replace
cache entries */

void aicache_updatetimes(monster_t *mob);
/* Increment the timestamps on everything in memory */

tunnellist_t *get_tunnels(tunnellist_t *head,int x,int y);
/* Find first tunnel in list starting at head that (x,y) falls in.
If no more, return NULL */

intersection_t *get_intersections(tunnellist_t *section,intersection_t *head);
/* Finds first intersection in list starting from head that falls in
tunnellist section. If no more, return NULL. */

intersection_t *nearest_intersection(int mx,int my);
/* Works out which tunnels (mx,my) is in, then finds all intersections
in those tunnels, and returns (one of) the closest to (mx,my).
Used for finding points for monsters to pass each other.
Shouldn't return entrances if possible. */

