/* ==== enums.h ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */

/* Only has all the enumerated types. Depends on nothing else? */

typedef enum {
    PTL_PWALL, /* Permanent wall at the edge of the map*/
    PTL_ROCK, /* was PTL_TWALL*/
    PTL_HWALL, /* A horizontal perimeter wall (of a room)*/
    PTL_VWALL, /* " vertical    """""""*/
    PTL_CORNER,/* Corner of a room. Similar to PTL_PWALL.*/
    PTL_DOOR,
    PTL_ODOOR, /* A door that is open not closed. Doh.*/
    PTL_FLOOR,
    PTL_TUNNEL, /* Area of floor in a tunnel. Just for mapgen functions.*/
    PTL_STAIRSUP,
    PTL_STAIRSDOWN,
    PTL_ALTAR,
    PTL_SHOP, /* Shouldn't actually appear in any map cell */
    PTL_RMID
} tile_t;

typedef enum {
   EQUIP_WEAPON, /* Your wieldables */
   EQUIP_RANGED,

   EQUIP_SUIT, /* Your armour */
   EQUIP_SHIELD,
   EQUIP_HEADGEAR,
   EQUIP_GLOVES,
   EQUIP_FOOTWEAR,

   EQUIP_RING1, /* Your jewellery */
   EQUIP_RING2, /* Might also have 2 other rings? */
   EQUIP_NECKWEAR,
   EQUIP_NONE /* Not equippable */
}equipspace_t; /* Position that an item of equipment is at */

typedef enum {
   IG_WEAPON, /* Wieldables first */
   IG_RANGED,
   IG_MISSILE,

   IG_SUIT,  /* Wearables */
   IG_SHIELD,
   IG_HEADGEAR,
   IG_GLOVES,
   IG_FOOTWEAR,

   IG_RING, /*Including jewellery */
   IG_NECKWEAR,

   IG_SCROLL, /* Readables */
   IG_BOOK, /* spellbooks and mundane books together or not?? */

   IG_FOOD, /* Comestibles */
   IG_CORPSE,
   IG_CONDIMENT, /* Can be applied to other foods, thrown, etc */
   IG_POTION,

   IG_WAND, /* Zappables */
   IG_STAFF, /* are targetless */
   IG_ROD,  /* one-shot-but-slowly-recharge */

   IG_TOOL, /* Generic tools */
   IG_CONTAINER,
   IG_STONE, /* Gemstones and rocks and pebbles and so on */
   IG_COINAGE, /* That everything??*/
   IG_DUMMY, /* nope. NB- this is a type used by the game, for doors, etc*/
   IG_LASTGROUP /* Marker! */
} itemgroup_t;

typedef enum {
   IC_WIELDABLES,
   IC_WEARABLES,
   IC_WEAPONS,
   IC_ARMOURS,
   IC_JEWELLERY,
   IC_READABLES,
   IC_COMESTIBLES,
   IC_ZAPPABLES,
   IC_VALUABLES,
   IC_SMALL, /* Should perhaps be done differently??? */
   IC_ALLCLASSES,
   IC_NOCLASSES
} itemclass_t; /* Used for, eg, stating what items a container can carry */

typedef enum {
   TOOL_KEY, /* unlock doors, and maybe other things? */
   TOOL_PEN, /* for writing */
   TOOL_RECEPTACLE, /* bottles, jars, tubs (NOT to be confused w/containers!) */
   TOOL_TOOL, /* general stuff */
   TOOL_REDHERRING /* just for flavour */
} toolgroup_t; /* to subdivide tool items by general use */

/* Various ways potion can be used. Passed to potion functions */
typedef enum {
   PUSE_QUAFF, /* player quaffs it */
   PUSE_QUAFFM, /* monster quaffs it */
   PUSE_DIP, /* player (only) dips object in it */
   PUSE_THROWP, /* thrown AT player */
   PUSE_THROWM, /* thrown AT monster */
   PUSE_APPLY  /* player (only) applies it. Only for some potions. */
} potionuse_t;

/* mode argument for displayHeapV2 (or its successor, p'raps "itemList()") */
typedef enum {
   /* Just list the items. User can only page through any pages or quit */
   PLISTMODE_SHOWINV, /* show inventory listing */
   PLISTMODE_SHOWHEAP, /* show any other item list */

   /* pick a single stack, no count given. Caller *might* choose to just
     pick one item from the chosen stack, eg the (q)uaff function, but
     that's automatic and the displayHeapV2() or itemList() function
     won't know anything about that. */
   PLISTMODE_PICKALLINV, /* pick an inventory item, don't allow a count */
   PLISTMODE_PICKALLHEAP, /* pick a heap item, don't allow a count */

   /* pick a single stack, allowing user to specify a count, which
      requires displayHeapV2() or itemList() to consider splitting the
      stack! */
   PLISTMODE_PICKSPLINV, /* pick an inventory item, allow a count */
   PLISTMODE_PICKSPLHEAP  /* pick a heap item, allow a count */

   /* Potentially have mutliple-stack selections in future? But these
   will be HARD to work out how to do! */
} listmode_t;

typedef enum {
   CSTR_UNCODED,
   CSTR_29BLNK,
   CSTR_MOREORCANCEL,
   CSTR_HEAPORMORE,
   CSTR_HEAPOREND,
   CSTR_INVORMORE,
   CSTR_INVOREND
} commonstring_t; /* indices to commonstring[] array */

typedef enum {
   SAT_STARVING,
   SAT_FAINTING,
   SAT_WEAK,
   SAT_VERYHUNGRY,
   SAT_HUNGRY,
   SAT_NORMAL,
   SAT_SATIATED,
   SAT_BLOATED,
   SAT_BURSTING
} satiation_t;

/* use this enumeration for describing any colours */
typedef enum {
   PCOL_DWHITE=0, /* AKA pale grey, off-white, etc */
   PCOL_DRED,
   PCOL_DBLUE,
   PCOL_MAGENTA,
   PCOL_DGREEN,
   PCOL_BROWN,  /* ==dim yellow */
   PCOL_DCYAN,
   PCOL_INV, /* ODD COLOUR OUT! Is *reverse* attribute! */

   PCOL_BWHITE=8, /* 8+0 */
   PCOL_BRED,
   PCOL_BBLUE,
   PCOL_PINK,
   PCOL_BGREEN,
   PCOL_YELLOW, /* ==bright yellow */
   PCOL_BCYAN,
   PCOL_GREY /* dark grey, == bright black, just called grey */
} pcolour_t;


typedef enum {
   FACE_ERROR,
   FACE_NONE,
   FACE_N,
   FACE_NE,
   FACE_E,
   FACE_SE,
   FACE_S,
   FACE_SW,
   FACE_W,
   FACE_NW,
   FACE_UP,
   FACE_DOWN,
   FACE_HERE
} direction_t;

typedef enum {
   PGOD_ENPITSU,
   PGOD_EB,
   PGOD_GRUBBLE,
   PGOD_URANUS,
   PGOD_PUTPUT,
   PGOD_NAPKIN,
   PGOD_KECKS,
   PGOD_SWUUJ,
   PGOD_K,
   PGOD_TINSEL,
   PGOD_HAPPYDUDE, /* Working title... */
   PGOD_TVB,
   PGOD_ATHEIST, /* Specifically when one *denies* gods */
   PGOD_SECULAR /* Used for when one hasn't chosen yet? */
} deity_t; /* The pantheon of Porrog! See NOTES/IDEA_DEITIES*/

typedef enum {
   LEVEL_WILD, /* Wilderness level */
   LEVEL_DUN0, /* default dungeon level (the original) */
   LEVEL_DUNX, /* test level type (to be removed) */
   LEVEL_CAVE, /* a big gnarly cavern */
   LEVEL_LASTLEVEL /* Marker */
} leveltype_t; /* what sort of structure the levelmap is; gets loaded
 into a global variable is it will be used in various switches, and
 is only changed when you change levels. */

typedef enum {
   MEM_LVLMAP,
   MEM_LVLSTRCT,
   /* low-level structures inside level */
   MEM_AREA,
   MEM_REGION,
   MEM_TUNNEL,
   MEM_INTERSECTION,
   MEM_ENTLIST,
   MEM_ENTRANCE,
   /* Items+mobs */
   MEM_ITYPE,
   MEM_ITEM,
   MEM_MTYPE,
   MEM_MONSTER,
   /* AI stuff */
   MEM_MMEM,
   MEM_TUNMEM,
   MEM_ROOMMEM,
   MEM_DOORMEM,
   MEM_TSEARCH,
   MEM_ISEARCH,
   /* Obscure stuff */
   MEM_LOCID,
   /* More generic stuff */
   MEM_STRING,
   MEM_LASTTYPE /* Marks the end */
} strgrp_t; /* Used with memutils functions to state what sort of elements
               are being mallocced */
typedef enum {
   LOGLEVEL_COMMENTARY, /* Pretty trivial */
   LOGLEVEL_DEBUG,   /* Not necessarily *bad*, but could be consequential */
   LOGLEVEL_WARNING, /* This really shouldn't happen, but hmm */
   LOGLEVEL_FOOBAR  /* So serious that before bugger() was written, such
                       an error would call crash() and end the game. */
} buglevel_t;

