/* ==== item_t.h ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


/* Where do these get decleared? */
/* Our externs get declared in item_p.h, just because. */


/* itemgroup_t is now in enums.h */

typedef struct {
   int ToHit;
   int ToDam;
   int ToEv; /* How it helps you to Evade blows */
   int ToBl; /* How it helps you to Block blows */
} ww_base_t; /* Doesn't include jewelery any more. */
/* Should base types actually either be weapon or armour instead,
   and only ego items get the alternative stats?  Maybe, maybe not */
/* Hmm, normally weapon types have their base quality expressed as
   dice rolls. Armour normally has AC stats as fixed, absolute numbers...
   Improvements to fighting ability from armour is only normally
   implemented as plusses to your rolls. So I think that yes,
   weapons and armour should be treated separately (at least the
   base types)*/
/* NOTE: this structure was made up whilst I wasn't sure how to do
   equipment. It will likely never be used */

/* Stat boosts given by jewellery can include hit/dam/ev/bl as stats.
   (like int, pow, str,dex, etc). Other types include "stat flags"
   that are status flags such as "invisible", "high metabolism", etc
   and can set or unset those flags. So such jewellery would have
   a flag bitmask and changer values.
   It becomes clear that the bitmask selects where the bits come
   from: 0 in bitmask selects the bits from the character, 1 in
   bitmask selects the bits from the jewellery.
   Question arises: How do cumulative effects of stat flag
   modifications work? If taken literally, the final effect
   would depend on the order of evaluation. */
typedef struct {
   int Stat_id; /* What stat is modified */
   /* Possible stats: tohit, todam, toev, tobl,
   speed, int, dex, con, str, pow, and others. NOT Hp, Mp, exp, etc.
   Values of Stat_id should be made an enum. */
   int Stat_mod; /* How is it modified? Vector */
} stat_mod_t; /* For jewellery, stat_mod should be 0? Or maximum value? */

typedef struct {
   int Status_bm; /* A bitmask: Which bits are changed */
   int Status_ch; /* What the bits are changed to */
} status_mod_t;

typedef struct {
   int Opens; /* Can you open/close it? */
   int Wlim; /* Max weight you can get into these containers */
   itemclass_t AcceptsClass; /* Superclass of items you can fit in it*/
   itemgroup_t AcceptsSpec; /* Specific group of items you can (also) fit in*/
} container_t;

typedef struct {
   int Dmg; /* Typical damage caused by weapon */
   int Tht; /* Boost to the to-hit factor. In range 0-100.
    dexterity and level contribute a large proportion of this. */
} weapondata_t;

typedef struct {
   int Cov; /* Coverage percentage item provides
   (really these should be percentage of body-part area, and be changed
   to an overall percentage automatically. But for the moment, just have
   the overall coverage. */
   int Prot; /* How many points of damage this item blocks */
   /* Other stuff should be ww_base_t or something? */
} armourdata_t;

typedef struct {
   char *Dname; /* This is the description, which gets shuffled at init */
   void (*Effect)(potionuse_t utype,void *tgt); /* utype describes how
               it's used: applied, drunk,thrown,dipped, also whether effect
               is on player or monster.; tgt is pointer to target,
               which could be monster, item, or player. Er... how do we
               target player? Simple, some values of utype indicate that.
               We could say NULL pointer, but wouldn't
               we want that for targetting nothing?? Perhaps have other
               usage types for one or the other.
               Also, want ways of defining who threw the potion if it's
               thrown, in case target is killed? Someone'll get EXP. */
/*  int Known; -* 1 if you know what it actually is; will either be
              set on init, based on Prerec probability, or when you
              find out what it is for yourself. If 1 on init, the alt
              name (Dname) gets shuffled amongst all potions, and then
              this potion has its Dname set to Name; if 0, but Prerec
              isn't 0, the Dname is fixed and not shuffled at all. */
   int Prerec; /* Fraction of 10 probability of being already known as
             itself; if not known, has a fixed identity; if 0, identity is
             unfixed. If 10, is already described as its reality; when
             this happens, the alt. description is available for others */

            /* To clarify, the Dname will be fixed to one of two values
            if Prerec>0, but which of those is chosen depends on how it
            sets Known, as does whether the Dname is initially shuffled
            with the other potions' Dnames. */
/* int Phony;  1 if potion is never created (just a way of supplying
             extra names... do I want that?? ...NO I understand how
             to deal with the extra values, this isnt needed at all. */
   int CanApply; /* 1 if you can apply it; otherwise, apply doesn't work.
                  Have to know what potion actually is to apply it- to
                  know how it'd be applied. Would work for ink. Dunno
                  about other things */
} potiondata_t;

typedef struct {
   char *Dname; /* Discriptive name, see potiondata_t->Dname */
   void (*Effect) (void *Reader); /* Effect of scroll (if read
                   by player, Reader==NULL */
} scrolldata_t;

typedef struct {
   char *Dname; /* Descriptive name, see potiondata_t->Dname */
} wanddata_t;

typedef struct {
   char *Dname; /* Descriptive name, see potiondata_t->Dname */
   void (*Effect) (void *Wielder); /* Effect of staff (if wielder
                    is player, Wielder==NULL */
} staffdata_t; /* differs from wanddata_t as staves are untargeted */

typedef struct {
   char *Dname; /* Descriptive name, see potiondata_t->Dname */
   unsigned int RechargeTime; /* When zapped, Delay==RechargeTime */
} roddata_t; /* differes from wanddata_t as rods are self-charging */

typedef struct {
   toolgroup_t Group;
   int Subtype; /* key number,
                       receptacle type, "tool" tool type, etc */
} tooldata_t;

union item_typedata_t {
   container_t Container; /* Container type info */
   ww_base_t Base; /* base stats for a wieldable/wearable- OBSOLETE??? */
   weapondata_t Weapon; /* Base stats for weapon */
   armourdata_t Arm; /* Base stats for item of armour */
   potiondata_t Potion; /* *name* pointers get shuffled at init */
/* void (*Effect)(void);  For potions, scrolls, gems, condiments */
   scrolldata_t Scroll;
   wanddata_t Wand;
   staffdata_t Staff;
   roddata_t Rod;
   tooldata_t Tool;
   struct item_t *DummyInst; /* the only instance of item, for dummy items
         (eg: closed doors are represented by these, with one instance
         for all doors) */
};

typedef struct { /* ego item base types */
/* int Stat_id;  What stats are improved */
/* int Stat_mod;  How do we modify it? */
   stat_mod_t Statmod; /* As above. */
/* int Statflag_bm;  Stat flag bitmask */
/* int Statflag_chl  What the changed flags change to */
   status_mod_t Statusmod; /* As above */
   int Is_artifact; /* 1 if this is actually an artifact (different)*/
} ego_t;

typedef struct {
   int ToHit;  /* Enchantment */
   int ToDam;
   int ToEv;
   int ToBl;
/* int something;   Other stat modifications would come from Ego type */
   ego_t *Ego; /* Pointer if any to ego item type */
} ww_extras_t;

typedef struct {
   struct item_t *Contents;
   int Currentweight; /* How much is in it currently */
   int Open; /* whether or not it's currently open. Unused? */
} containerinst_t;

typedef struct {
   unsigned int Charges: 15; /* how many remaining */
   unsigned int Known: 1; /* Does user know remaining charges? */
} wandinst_t; /* also for staves */

typedef struct {
   unsigned int Delay: 12; /* time (in rod-specific units) till zappable */
   unsigned int Rate: 3; /* every 50 ish ticks, rod loses Rate+1 from
                          its Delay count (until Delay==0) */
   unsigned int Known: 1; /* Does the user know rod's regen rate? */
} rodinst_t;

union item_inst_data_t {
   containerinst_t Container;
   /*int Charges; -* of a wand/rod... NO. */
   wandinst_t Wand; /* also for staves */
   rodinst_t Rod;
   ww_extras_t Extras; /* extra item specific data. Includes ptrs to ego items*/
};

