/* ==== mapgen_types.h ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */

/* Structures used in map generation (also used by AI) */


/* Needs to be changed for region code. Took tunnel data out? */
struct area_t {
   int Firstroom; /* Because we'll only store a bounded number of
      rooms, which we'll stick in an array. */
/* struct tunnellist_t *Firsttunnel; No, no longer. */
   int Size; /* Number of rooms in the area (starts as 1)*/
   struct area_t *Nextarea;/*Because we want a list of em */
   /* With the region code, we still want a list of areas.
     Would we also want a list of regions?? YES! One list per area. */
   struct region_t *Firstregion; /* Must start as NULL. */
};
typedef struct area_t area_t;

/* Needs to be changed for region code??? */
struct tunnellist_t {
   int Xstart,Ystart;
   int Xend,Yend;
/* area_t *Area; We don't use that any more. Get it from region. */
   struct region_t *Region; /* That's all we added! */
   struct tunnellist_t *Nexttunnel;
};
typedef struct tunnellist_t tunnellist_t;

/* Now in enums.h-  typedef enum {
} direction_t; Should be defined elsewhere? Yup, it is now. */


typedef struct {
   int Xcentre,Ycentre; /*Hey, seems we *can* say it this way.*/
   int Xstart,Ystart; /*Just so we don't have to keep calculating it*/
   int Width, Height;
   struct entrancelist_t *Firstentrance; /*Was already sort of there before*/
   area_t *Area;
   int Nextroom;
} roomdata_t;

/* FINALLY! The region type: */
struct region_t {
   area_t *Area;
   tunnellist_t *Firsttunnel;
   struct entrancelist_t *Firstentrance;
   /* Note that though an entrance has a region *and* a room, regions and
   rooms have different lists of the same entrance data (if that makes sense)*/
   /* Max entrances must be numtunnels+1 */
   struct intersection_t *Intersections; /* List of all (?) intersections
    in region- Should EXCLUDE entrances!
    Build_door() should scan region for intersections coinciding with
    entrance's coords, and remove them (there will be some) */
   /* Now, we're going to have LISTS of regions!!! */
   struct region_t *Nextregion;
};
typedef struct region_t region_t;

/* Entrance data is now implemented as separate lists of separate entrances. */

typedef struct {
   int Roomno; /* The room this entrance is on. There's only one! */
   region_t *Region; /* The region this entrance is in. Again, only one! */
   int Xcoord,Ycoord;
   direction_t Outfacing; /* the direction you face if you face this
                              door from inside the room XP */
   char Open; /* 1 if open, 0 if closed. Used when regenerating map, first
      set at map generation, and later changed when doors opened/closed.
      Might add locked status later. Hidden doors too? For display and
      blockage testing, the tiletype is used instead, as searching for
      entrance structures is too time-consuming to do often. */
} entrance_t;

struct entrancelist_t {
   entrance_t *Entrance;
   struct entrancelist_t *Next;
};
typedef struct entrancelist_t entrancelist_t;

/* Intersection data */
struct intersection_t {
   int Xcoord,Ycoord; /* Just coords info? */
   struct intersection_t *Next;
};
typedef struct intersection_t intersection_t;

typedef struct {
   int Xcoord,Ycoord; /* Where the feature is */
   int Room; /* Number of room the feature's in (feature *must* be in room!)*/
   tile_t Feature; /* WHAT it is */
   featuredata_t Extra; /* type specific info */
} feature_t;

/* These 2 types define a level (NOW to be more than 2) */
typedef struct {
   area_t *Firstarea;
   roomdata_t Room[217];
   int Totalrooms; /* How many of those rooms are real */
   feature_t Features[10]; /* The (limited number of) features here */
   int Nfeatures; /* How many there so far */
   unsigned int Nmobs; /* How many mobs we have in this level */
   unsigned int Mobticker; /* Counts up to N and then another mob is spawned */
   item_t *Itemlist; /* ALL items in the level */
   monster_t *Moblist; /* ALL mobs in the level */
/* int Dlev;  level of dungeon (also in levelmap_t, but that's
    different) for determining experience level.
    EXCEPT that this should be stored in area_t, due to eventually
    having separate areas reachable from different levels.
    STOPGAP implementation only. */
} levelstructs_t;
/* TO BE CHANGED! to dlevelstructs? or dun0levelstructs? 
 Perhaps if several level types (would be *Structured* level types)
 use this format, can rename it with #defines??? */

typedef struct {
    int Seed; /* random seed to generate scattered rocks with
     (just a stopgap measure, till something more interesting goes in) */
} wildlevelstructs_t;

typedef union {
    wildlevelstructs_t Wilderness;
    /*dun0*/levelstructs_t Dungeon0;
} structtypes_t;

struct levelmap_t {
   int Depth; /* How deep into the dungeons it is */
   /* How do we represent the details of where the exits lead?
   I think it'll just be in the exit squares. */
   leveltype_t Type; /* What actual sort of level is this?
    Determines structures used, generation methods, AI methods,
    numerous other things. This element gets loaded into a
    global variable for use elsewhere */
   int Archived; /* 1 if level has been compressed, 0 if not */
   levelstructs_t *Unpacked; /* The unpacked data, if any */

   /* structtypes_t Unpacked */
   /* NOTE: should be a UNION Unpacked... or just call it Structures? */

   char *Packed; /* The packed form, if any */
   struct levelmap_t *Nextlevel;
};
typedef struct levelmap_t levelmap_t;

extern levelmap_t *firstlevel;
/* The first in the linked list */
extern levelmap_t *thislevel; /* Where we are now! */
extern levelstructs_t *tls; /* equiv to thislevel->Unpacked */
extern levelmap_t *lastlevel; /* When going to a new level, the
   map generator has to make a way back to where you were */
   /* should perhaps be renamed "prevlevel"?*/
extern leveltype_t thislevtype; /* equiv to thislevel->Type, duh */
