/* ==== structures.h ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#define MAPWID 140
#define MAPHGT 140

/* tile_t is now in enums.h */

typedef struct {
    short Int;
    short Dex;
    short Con;
    short Str;
    short Agi;
    short Exprate; /* obsolete? */
} fixed_stats_t;

typedef struct {
    unsigned short Dex;
    unsigned short Con;
    unsigned short Str;
    unsigned short Agi;
    /* Skills will also go here! */
} training_t;

typedef struct {
    /* Derived stats, etc */
    int Hp; /* Hit points */
    int MaxHp;
    int Mp; /* Mana */
    int MaxMp;
    int Satiation; /* only applies to player, not monsters...
       satiation scale: 0-30 starving, may die; (clipped at 0)
           31-100 fainting (not as badly as nethack??);
           101-500 weak (drastically lowered strength);
           501-900 very hungry (somewhat lowered dexterity)
           901-2000 hungry (no particular effect);
           3500-5000 satiated (somewhat lowered agility)
           5001-6000 bloated (drastically lowered agility);
           6001+ bursting (negligible agility, may get injured or sick etc);*/

    int Ev; /* Evasion score, however that's expressed */
    int Cov; /* Total coverage of armour */
    int Pravg; /* Average protection of all armour (incl bare skin!!) */

    /* Not derived stats, but variable! */
    uint64_t Exp; /* Total experience points */
    int Level;
} var_stats_t;

typedef struct {
    int Mapno; /* Should be a pointer to map, but we aint got none yet! */
    struct levelmap_t *Map; /*There we are! Do we have a use for the int now?*/
    int X;
    int Y;
} locid_t;


struct itemtype_t{
    char Symbol;
    /* char Col; -* color of item- could be bitfield? */
    pcolour_t Col; /* Nope, just an enumeration now. It's not like we'd have
                   THAT many itemtype_ts for God's sake!!!! OTOH they'd be
                   accessed rather often, so extra overhead in extracting
                   the values would be sucky */
    itemgroup_t Type; /* WAS int for some reason :P */
    char *Name;
    unsigned int Weight; /* Not sure if this is right place...
                         yeah, it is, unless we have variant items
                         with *different* weights, in which case it
                         could get messy :P */
    unsigned int Probability : 8; /*how *unrare* these are within the group*/
    unsigned int Known : 1; /* Have these been identified?
                               NB: This was stored in specific type's
                               Typedata previously, but would be messy
                               to deal with that way */
    union item_typedata_t Typedata;  /*Data specific to this type */
    struct itemtype_t *Next; /* for the current implementation, we've
       too many types to count and then stick in an array */
};
typedef struct itemtype_t itemtype_t;

struct item_t {
    itemtype_t *Itemtype; /* Exactly what type of object? */
    unsigned int Number; /* How many of it? */
    char Invletter; /* If you've carried this before, this stores
       what letter it used in your inventory. */
    struct item_t *Next;  /* Next item in a list (if any) */
    struct item_t *Next2; /* Next item in mapwide list. This is
     needed so that when we leave the level, we can collect up
     the items in it, record them somewhere, and free them.
     We can then recreate them on returning-  but that
     requires the coords of the items...*/
    int X,Y; /* Needed for replacing items on map */
    int Cursed; /* If it's cursed. Dunno about blessed items,
                  we might not have those */
    union item_inst_data_t Instdata; /* data specific to this instance */
};
typedef struct item_t item_t;


struct mobtype_t{
    char Symbol; /* What letter is it? */
    /* int Colour; -* Should be changed to whatever type curses uses */
    pcolour_t Colour; /* is this even *used* yet?! */
    char *Name;
    char *Desc; /* A description of this mob type */
    itemtype_t *Corpsetype; /* Pointer to the type for this mob's corpse */

  /*fixed_stats_t Fixed_stats;   -* Global stats for this type, eg INT */
    fixed_stats_t Stat_max; /* Global stat maxes for this type */

    unsigned int BaseLevel; /* when monster created, is this level */
    struct mobtype_t *Next; /* Next in global list */
};
typedef struct mobtype_t mobtype_t;

struct monster_t {
/*    locid_t Pos; */
    int X,Y;
    mobtype_t *Mobtype; /* Exactly what type of monster */
    int Ticker; /* incremented when monster acts, decremented as time
       passes. When 0 or lower, monster can act again */
    fixed_stats_t Fixed_stats; /* individually calculated STR, CON, etc,
               based on stat max at start of game */
    var_stats_t Var_stats; /* Stats that change, eg HP, Mana*/
    item_t *Inventlist; /* List of any items carried */
/*  entrance_t *Target;  Target entrance */
    int Annoyance; /* Normally 0, but increases when monster's
      path is blocked. Possibly also associated with bad health? */
    int Wx,Wy; /* Coords of a waypoint (used in regions) */
    int Tx,Ty; /* Coords of target entrance (must *pass*!) */
    int Ttype; /* The space type to aim for to hit target */
    struct aicache_memory_t *Memory; /* Used by the AI. */
    struct monster_t *Next; /* Next monster in level list */
};
typedef struct monster_t monster_t;

typedef union {
    locid_t *Destination; /* For things like stairs, portals, etc */
    deity_t Deity; /* For altars */
    /* Might eventually put status of doors in here too...
       when I work out how I'll implement that! */
} featuredata_t;
/* Extra data used for various tiles. */

typedef struct {
    tile_t Tiletype; /* Could be walls, floors, etc*/
    monster_t *Mptr; /* Any monster that's here*/
    item_t *Itemlist; /* A list of any items here */
    char Seen; /*This tile has been seen at some point. Seen acts as a bool*/
    itemtype_t *Remembered; /*Pointer to item-type last seen here */
    /*
    locid_t *Dest; Pointer to any destination structure */
/*  featuredata_t Extra;  Replaces Dest element, to be usable by VARIOUS
       feature tiles. */
    unsigned char Featureidx; /* index into feature array for this level.
                  Beacuse having pointers to stuff stored in the maploc_t
                  is STUPID. ...changed to *unsigned* char for portability
                  and sanity!!*/
} maploc_t;

