/* ==== item.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */



#include <stdlib.h>
#include "syscurses.h" /* shouldn't be here */
#include "C-foogrid.h"
#include "porrog_defs.h"
#include "memutils.h"
/* #include "maparray.h" Don't think we use this one any more.*/
#include "item_t.h"
#include "structures.h" /*Hers's where that stuff is now*/
#include "mapgen_types.h" /* Needed for levelmap type */
#include "maparray.h" /* Declares map */
#include "player_t.h"
#include "dXrng.h"/* for p_inv_init... should it be done within player.c? */

/* protos */
#include "item_p.h"
#include "player_p.h"
#include "crash.h"
#include "ui.h"


/* We now need some structure that keeps track of which inventory
   letters have and haven't been used. 52 possible inventory letters
   means we could have an array of 13 elements each describing
   ranges of letters: 4 each. When you want to get a new inventory
   letter, you search through the array from the start till you
   find one element where it's indicated that *at least* one letter
   is available.
   The array doesn't *necessarily* have to indicate *which* of those
   4 letters are available, as simply knowing that there are letters
   within some range allows you to search through the letters of the
   inventory and have some clue of which ones to look for.*/

/* Ah. This search, whilst more efficient than a blind search, is
   actually a bit involved. It's actually a lot easier to just
   have a bit-array... As this can be done with just 52 bits, it's
   hardly memory intensive! It could be done with 8 chars, but
   I'm using 13 to avoid having one half-filled. */

/* char invletter_array[13]; Now in globals.c */
extern char invletter_array[]; /* This's the only file to use this, but...*/

void p_init_inv()
{
   int counter,quantity;
   player.Inventlist=NULL;
   for (counter=0;counter<13;counter++) invletter_array[counter]=15;
   /* All free as unused */
   /* Now create some initial inventory items?? */
   switch(roll_dX(d4set[0])) {
      case 0:
        quantity=1+(roll_dX(d4set[0])+roll_dX(d4set[1])+roll_dX(d4set[2]));
        p_get_item(newItem(getItemtypeByName("pinstripe suit",IG_SUIT),1));
        p_get_item(newItem(getItemtypeByName("bowler hat",IG_HEADGEAR),1));
        p_get_item(newItem(getItemtypeByName("shoes",IG_FOOTWEAR),1));
        p_get_item(newItem(getItemtypeByName("tea",IG_POTION),1));
        p_get_item(newItem(getItemtypeByName("pen",IG_TOOL),1));
        p_get_item(newItem(getItemtypeByName("gold coin",IG_COINAGE),quantity));
        /* should have 4 or 5? Just as spare change... */
        break;
      case 1:
        quantity=1+roll_dX(d8set[0]);
        p_get_item(newItem(getItemtypeByName("whip",IG_WEAPON),1));
        p_get_item(newItem(getItemtypeByName("gimp suit",IG_SUIT),1));
        p_get_item(newItem(getItemtypeByName("waders",IG_FOOTWEAR),1));
        p_get_item(newItem(getItemtypeByName("soylent brown",IG_FOOD),1));
        p_get_item(newItem(getItemtypeByName("paper bag",IG_CONTAINER),1));
        p_get_item(newItem(getItemtypeByName("thrupenny bit",IG_COINAGE),quantity));
        break;
      case 2:
        quantity=(3*(1+roll_dX(d8set[0])))+(12*(2+roll_dX(d4set[0])));
        p_get_item(newItem(getItemtypeByName("sword",IG_WEAPON),1));
        p_get_item(newItem(getItemtypeByName("leather armour",IG_SUIT),1));
        p_get_item(newItem(getItemtypeByName("leg of lamb",IG_FOOD),1));
        p_get_item(newItem(getItemtypeByName("brass coin",IG_COINAGE),quantity));
        /* Should have quite a few? 50? */
        break;
      case 3:
        quantity=1+roll_dX(d4set[0]);
        p_get_item(newItem(getItemtypeByName("penknife",IG_WEAPON),1));
        p_get_item(newItem(getItemtypeByName("paper crown",IG_HEADGEAR),1));
        p_get_item(newItem(getItemtypeByName("fingerless gloves",IG_GLOVES),1));
        p_get_item(newItem(getItemtypeByName("stockings",IG_FOOTWEAR),1));
        p_get_item(newItem(getItemtypeByName("necklace",IG_NECKWEAR),1));
        p_get_item(newItem(getItemtypeByName("doner kebab",IG_FOOD),1));
        p_get_item(newItem(getItemtypeByName("booze",IG_POTION),quantity));
        p_get_item(newItem(getItemtypeByName("carrier bag",IG_CONTAINER),1));
        break;
   }
}

char firstUnusedInvletter()
{
   int counter;
   char bitmap;
   int letter;
   for(counter=0;counter<13;counter++)
   {
      bitmap=invletter_array[counter];
      if (bitmap) break;
   }
   if (13==counter) {/* There wasn't one!!*/ return(' ');}
   letter=counter*4;
   for(;bitmap<8;letter++,bitmap*=2)
      ; /* Only the 4 LSB bits used.
    It's fairly important the 4 MSB bits are clear, but if they
    aren't, the first letter will appear to be free, which would
    initially be true... When stating that a letter is being used,
    the top bits get cleared in the process.*/
   /* We now have a letter number */
   if (letter<26)
   { /* Lower case letter */
      return('a'+letter);
   } else { /* Upper case */
      return('A'+(letter-26));
   }
}

char isInvletterAvailable(char letter) {
   /* newly added, to allow p_get_item() to use the old letters for
   some previously held items-- Mar 2004 */
   int index;
   int exponent;
   int normalised;
   if(letter>='a') normalised=letter-'a';
   else normalised=(letter-'A')+26;
   index=(normalised/4);
   exponent=normalised-(index*4);
   /* a set bit in the appropriate place indicates *freeness* */
   return(invletter_array[index] &(8>>exponent));
   /* bit order is silly but I'm not changing it now */
}

void tagInvletterUsage(char letter,int discarding)
{ /* discarding   means you're freeing that letter.*/
   int index;
   int exponent;
   int normalised;
   if(letter>='a')
   {
      normalised=letter-'a';
   } else {
      normalised=(letter-'A')+26;
   }
   index=(normalised/4);
   exponent=normalised-(index*4);
   if (discarding)
   {   /* We're freeing this letter, set its bit */
      invletter_array[index] |= (8 >> exponent);
   } else { /* We're using this letter, so clear its bit */
      invletter_array[index] &= (15-(8 >> exponent));
   }
}

item_t *getItemByInvletter(char letter) {
   item_t *itemscan;
   if(isInvletterAvailable(letter)) return(NULL); /* NOT HELD! */

   for(itemscan=player.Inventlist;itemscan!=NULL;itemscan=itemscan->Next) {
       if(itemscan->Invletter==letter) return(itemscan);
   }
   /* We SHOULD NOT reach here */
   bugger(REPTYPE_ITEMS|REPTYPE_CODE,LOGLEVEL_FOOBAR,"getItemByInvletter() passed inventory letter for item not found in inventory!");
   crash(INCONSISTENT);
   return(NULL); /* shtup */
}
/* Simple function that was in prompt_for_inv_item() but will also be
   used by displayHeapV2(), so put here for clarity */

/* creates new item and returns pointer to it */
item_t *newItem(itemtype_t *otype,unsigned int quantity)
{
   item_t *newitem;
   if(otype==NULL) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Tried to create new item with NULL type");
     crash(INCONSISTENT);
   }
   if(quantity==0) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Tried to create a stack of 0 items");
     crash(INCONSISTENT);
   }
   newitem=(item_t *)counting_malloc(MEM_ITEM,sizeof(item_t));
   newitem->Itemtype=otype;
   newitem->Invletter=' ';
   newitem->Number=quantity;
   newitem->Cursed=0; /* you'd only sometimes want to set it */
   /* initialise instance data */
   switch(otype->Type){
      case IG_CONTAINER:
        newitem->Instdata.Container.Contents=NULL; /* had segfaults without */
        break;
      case IG_WAND:
        newitem->Instdata.Wand.Known=0;
        /* leave Charges for caller to set? */
        break;
      case IG_ROD:
        /* leave Rate for caller to set? */
        newitem->Instdata.Rod.Delay=0;/*ready to go-should it be otherwise?*/
        newitem->Instdata.Rod.Known=0;
        break;
      default:;
   }
   /* newitem->Next=NULL; -AFAICT this will get set when needed */
   /* newitem->Next2=NULL; -Ditto */
   return(newitem);
}

/* useful because of containers. DO NOT use on phonyitem! */
void freeItem(item_t *item)
{
   item_t *contents;
   item_t *next;
   if(item->Itemtype->Type==IG_CONTAINER) {
      for(contents=item->Instdata.Container.Contents;contents!=NULL;contents=next) {
         next=contents->Next;
         freeItem(contents);
      }
   }
   counting_free(MEM_ITEM,item);
}

/* DITCHED THIS FUNCTION!  NO LONGER USED ANYWHERE! *-
void add_item(levelstructs_t *level,int x, int y,itemtype_t *otype)
{
   item_t *newitem;
   long offset=FOOGRID_IDX(map,y,x);
   maploc_t *mapstart=map->Data; -*Got access to coords*-

   newitem=newItem(otype,1);
   newitem->Next=(mapstart+offset)->Itemlist;
   (mapstart+offset)->Itemlist=newitem;
   newitem->X=x; newitem->Y=y; -* Remember where it is *-
   -* Now, we put the item in the level-wide item list *-
   newitem->Next2=level->Itemlist;
   level->Itemlist=newitem;
   -*Its on top of the stack there. We don't return anything *-
}
-* Places a new instance of otype at the position given. Could
 this now use place_item()?  YES IT COULD. FINALLY REMOVED*/

void newItemtype(itemtype_t *space,char symbol,pcolour_t col,char *name,itemgroup_t type, unsigned int weight, unsigned int probability)
{
    space->Symbol=symbol;
    space->Col=col;
    space->Name=name; /*You'd better not free name later...*/
    space->Type=type;
    space->Weight=weight;
    space->Probability=probability;
    /* Hmm, is that all?*/
}

itemtype_t *push_itype(itemgroup_t group,char *name, unsigned int weight, unsigned int probability)
{
    itemtype_t *newit;
    pcolour_t col=PCOL_DBLUE; /* seemed previously to default to 8? */
    char symbol;
    newit=(itemtype_t *)counting_malloc(MEM_ITYPE,sizeof(itemtype_t));
    /* newit->Next=first_it;  Changed to: */
    newit->Next=first_its[group];
    /* first_it=newit; Changed to: */
    first_its[group]=newit;
    it_nums[group]++; /* Update the count for this type */
    if(it_freqtot[group]+probability<it_freqtot[group]) {
       bugger(REPTYPE_ITEMS|REPTYPE_CODE,LOGLEVEL_FOOBAR, \
         "Item frequency counts overflowed!");
       crash(OUT_OF_BOUNDS);
    }
    it_freqtot[group]+=probability; /* keep running total of that too */
    switch(group) {
       case IG_WEAPON:
         col=PCOL_DCYAN; /* COLO:R_CYAN+1 */
         symbol=')';
         break;
       case IG_RANGED:
         col=PCOL_YELLOW; /* COLO:R_YELLOW+9 */
         symbol='}';
         break;
       case IG_MISSILE:
         col=PCOL_DCYAN; /* COLO:R_CYAN+1 */
         symbol='-';
         break;
       case IG_SUIT:
         col=PCOL_BWHITE; /* 8!!!*/
         symbol='[';
/*       newit->Typedata.Arm.Cov=40;
         newit->Typedata.Arm.Prot=1; */
         break;
       case IG_SHIELD:
         col=PCOL_GREY; /* COLO:R_BLACK+9 */
         symbol='(';
/*       newit->Typedata.Arm.Cov=15;
         newit->Typedata.Arm.Prot=3; */
         break;
       case IG_HEADGEAR:
         col=PCOL_YELLOW; /* COLO:R_YELLOW+9 */
         symbol='{';
/*       newit->Typedata.Arm.Cov=15; */
         break;
       case IG_GLOVES:
         col=PCOL_BRED; /* COLO:R_RED+9 */
         symbol='{';
/*       newit->Typedata.Arm.Cov=8; */
         break;
       case IG_FOOTWEAR:
         col=PCOL_GREY; /* unsure. COLO:R_BLACK+9 */
         symbol='{';
/*       newit->Typedata.Arm.Cov=20; */
         break;
       case IG_RING:
         col=PCOL_YELLOW; /* COLO:R_YELLOW+9 */
         symbol='=';
         break;
       case IG_NECKWEAR:
         col=PCOL_YELLOW; /* COLO:R_YELLOW+9 */
         symbol='='; /* As other jewelery?! */
/*       newit->Typedata.Arm.Cov=2;  */
         break;
       case IG_SCROLL:
         col=PCOL_BWHITE; /* 8! */
         symbol='?';
         break;
       case IG_BOOK:
         col=PCOL_BWHITE; /* 8! */
         symbol='+';
         break;
       case IG_FOOD:
         col=PCOL_BROWN; /* COLO:R_YELLOW+1 */
         symbol='%';
         break;
       case IG_CORPSE:
         col=PCOL_DRED; /* COLO:R_RED+1 */
         symbol='%';
         break;
       case IG_CONDIMENT:
         col=PCOL_BROWN; /* COLO:R_YELLOW+1 */
         symbol='"'; /* Check. */
         break;
       case IG_POTION:
         col=PCOL_PINK; /* COLO:R_MAGENTA+9 */
         symbol='!';
         break;
       case IG_WAND:
         col=PCOL_BROWN; /* not too sure... COLO:R_YELLOW+1*/
         /* col=COLO:R_BLACK+9; */
         symbol='/';
         break;
       case IG_STAFF:
         col=PCOL_BROWN; /* COLO:R_YELLOW+1 */
         symbol='|';
         break;
       case IG_ROD:
         col=PCOL_BCYAN; /* COLO:R_CYAN+9 */
         symbol='/'; /* was |, but now matches wands */
         break;
       case IG_TOOL:
         col=PCOL_DWHITE; /* COLO:R_WHITE+1, which was wrong! ;)*/
         symbol='&'; /* \ is another alternative...*/
         break;
       case IG_CONTAINER:
         /* don't know what colour */
         symbol=']'; /* Could also be as tools. */
         break;
       case IG_STONE:
         col=PCOL_GREY; /* COLO:R_BLACK+9 */
         symbol='*';
         break;
       case IG_COINAGE:
         col=PCOL_YELLOW; /* COLO:R_YELLOW+9 */
         symbol='$';
         break;
       case IG_DUMMY:
         col=PCOL_INV; /* nope COLO:R_BLACK+9*/
         symbol='?'; /* nope. Just stopgap. */
         break;
       case IG_LASTGROUP:
         bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"push_itype():Item put in meaningless group!");
         crash(OUT_OF_BOUNDS);
         break;
    /* default: If we add new types, we should get
       a warning if we leave out a default */
    }
    newItemtype(newit,symbol,col,name, group, weight,probability);
    return(newit);
}

int sortInventoryGroup(itemgroup_t grp1,itemgroup_t grp2) {
    /* Same semantics as Perl's cmp and <=> operators:
       -1: grp1 comes before grp2
        0: same grouping
        1: grp2 comes before grp1 */
    /* This function could alternatively be done as a macro for better
    speed, or even implemented with a great big 2d table */
    switch(grp1) {
       case IG_WEAPON: /* weapons */
       case IG_RANGED:
       case IG_MISSILE:
          if(grp2>IG_MISSILE) return (-1);
          break;
       case IG_SUIT: /* main armour */
       case IG_SHIELD:
          if(grp2>IG_SHIELD) return(-1);
          if(grp2<IG_SUIT) return(1);
          break;
       case IG_HEADGEAR: /* other wearables */
       case IG_GLOVES:
       case IG_FOOTWEAR:
       case IG_RING: /* should we put these under jewelery instead? */
       case IG_NECKWEAR:
          if(grp2>IG_NECKWEAR) return(-1);
          if(grp2<IG_HEADGEAR) return(1);
          break;

       case IG_SCROLL: /* readables */
       case IG_BOOK:
          if(grp2>IG_BOOK) return(-1);
          if(grp2<IG_SCROLL) return(1);
          break;
       case IG_POTION: /* potables */
          if(grp2<IG_POTION) return(1);
          if(grp2>IG_POTION) return(-1);
          break;
       case IG_FOOD: /* comestibles */
       case IG_CORPSE:
       case IG_CONDIMENT:
          if(grp2<IG_FOOD) return(1);
          if(grp2>IG_CONDIMENT) return(-1);
          break;

       case IG_WAND: /* zappables */
       case IG_STAFF:
       case IG_ROD:
          if(grp2<IG_WAND) return(1);
          if(grp2>IG_ROD) return(-1);
          break;

       case IG_TOOL: /* misc tools */
       case IG_CONTAINER:
          if(grp2<IG_TOOL) return(1);
          if(grp2>IG_CONTAINER) return(-1);
          break;
       case IG_STONE:
          if(grp2<IG_STONE) return(1);
          if(grp2>IG_STONE) return(-1);
          break;
       case IG_COINAGE:
          if(grp2<IG_COINAGE) return(1);
          if(grp2>IG_COINAGE) return(-1); /* like *what*? */
          break;
       default: return(0); /* bugger!*/
    }
    /* fall through cases would be same inventory group */
    return(0);
}

void itemGroupHeading(char *space,itemgroup_t group) {
    switch(group) {
        case IG_WEAPON:
        case IG_RANGED:
        case IG_MISSILE:
           sprintf(space,"Weapons");
           break;
        case IG_SUIT:
        case IG_SHIELD:
           sprintf(space,"Main Armour");
           break;
        case IG_HEADGEAR:
        case IG_GLOVES:
        case IG_FOOTWEAR:
        case IG_RING:
        case IG_NECKWEAR:
           sprintf(space,"Other Wearables");
           break;
        case IG_SCROLL:
        case IG_BOOK:
           sprintf(space,"Reading Material");
           break;
        case IG_POTION:
           sprintf(space,"Potables");
           break;
        case IG_FOOD:
        case IG_CORPSE:
        case IG_CONDIMENT:
           sprintf(space,"Comestibles");
           break;
        case IG_WAND:
        case IG_STAFF:
        case IG_ROD:
           sprintf(space,"Zappables");
           break;
        case IG_TOOL:
        case IG_CONTAINER:
           sprintf(space,"Miscellaneous Tools");
           break;
        case IG_STONE:
           sprintf(space,"Stones");
           break;
        case IG_COINAGE:
           sprintf(space,"Money");
           break;
        default:
           sprintf(space,"Items from UNKNOWN group, a bug!");
           break;
    }
}

itemtype_t *getItemtypeByProbability(itemgroup_t grp,unsigned int roll) {
    unsigned int count=0;
    itemtype_t *picktype;
    /* assert that 0 <= roll < (it_freqtot[grp]) */
    if(it_nums[grp]==0) {
       bugger(REPTYPE_CODE|REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"getItemtypeByProbability(): Asked to pick an itemtype from empty list");
       crash(INCONSISTENT);
    }
    if(it_freqtot[grp]<=roll) {
       bugger(REPTYPE_CODE|REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"getItemtypeByProbability(): Roll given outside acceptable bounds");
       crash(INCONSISTENT);
    }
    for(picktype=first_its[grp];
       /* could do with checking picktype!=NULL! */
       count+=picktype->Probability, count<=roll; /*when count>roll, get this*/
       picktype=picktype->Next)
       ;
    if (picktype==NULL) {
       bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"getItemByProbability(): Somehow picked a NULL for an itemtype");
       crash(FUNCTION_ERROR);
    }
    return(picktype);
}

/* It shouldn't be slow enough to need optimising, but I wouldn't
want to be using it too often...  The expected use is when setting
up the players inventory, or creating monsters. Or possibly wishes???
But this can't (yet) deal with plurals, etc. */
itemtype_t *getItemtypeByName(char *name,itemgroup_t grp)
{
    itemtype_t *scan;
    if (grp!=IG_LASTGROUP) {
      for(scan=first_its[grp];scan!=NULL;scan=scan->Next) {
         if(strcmp(scan->Name,name)==0) {
           bugger(REPTYPE_ITEMS,LOGLEVEL_COMMENTARY,"getItemtypeByName found item OK");
           return(scan);
         }
      }
    } else { /* search them all, return the first */
      for(grp=IG_WEAPON;grp<IG_LASTGROUP;grp++) {
         for(scan=first_its[grp];scan!=NULL;scan=scan->Next) {
            if(strcmp(scan->Name,name)==0) {
              bugger(REPTYPE_ITEMS,LOGLEVEL_COMMENTARY,"getItemtypeByName found item OK");
              return(scan);
            }
         }
      }
    }
    bugger(REPTYPE_ITEMS,LOGLEVEL_DEBUG,"getItemtypeByName couldn't find named type");
    return(NULL);
}

/* Removes top item from a heap on the floor, returns it (with
   that item individualised)*/
/* CHANGING to remove a SPECIFIC item from the heap, returning
   nothing. 
item_t *skim_item(foogrid *map,int x,int y) */
void skimItem(levelstructs_t *level, int x,int y, item_t *item)
{
   long offset=FOOGRID_IDX(map,y,x);
   maploc_t *mapstart=map->Data;
   item_t *heap;
   item_t *next; /* Next2, the next in level-wide itemlist */
   item_t *prev; /* item before item in heap */
   if(NULL==item)
   { /* Silly thing to do */
      bugger(REPTYPE_CODE|REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"skimItem() called for NULL item");
      crash(INCONSISTENT);
   }
   heap=(mapstart+offset)->Itemlist;
   if (NULL==heap)
   {
      /* skimItem now expects to remove a given item from
      the heap. If there's no heap, the item cant be there!!!*/
      bugger(REPTYPE_CODE|REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"skimItem() called for NULL heap");
      crash(INCONSISTENT);
   }
   /* Old behaviour: Take away the TOP item.
   (mapstart+offset)->Itemlist=item->Next;
   item->Next=NULL;
   return(item);
   */
   if (heap==item)
   { /* Special case */
      (mapstart+offset)->Itemlist = item->Next;
   } else { /* item must be somewhere else */
      for(prev=heap; prev->Next!=item&&prev->Next!=NULL; prev=prev->Next);
      if(prev->Next==NULL) {
         bugger(REPTYPE_CODE,REPTYPE_ITEMS,"skimItem(): item not in heap");
         crash(INCONSISTENT); /* Not found!! */
      }
      prev->Next = item->Next;
   }
   /* item is no longer on the floor. Hurrah! Caller MUST have either
      put it somewhere else (eg inventory) or freed it. */
   /* NOW also take it off the level-wide item list! */
   next=item->Next2;
   if (level->Itemlist==item) {
      level->Itemlist=next;
   } else {
      for(prev=level->Itemlist;prev->Next2!=item&&prev->Next2!=NULL;prev=prev->Next2);
      if(prev->Next2==NULL) {
         bugger(REPTYPE_CODE,REPTYPE_ITEMS,"skimItem(): item not in level-wide items list!");
         crash(INCONSISTENT); /*Not found! */
      }
      prev->Next2=next;
   }
}

/* Puts an item on top of heap on floor. Assumes item is alone.*/
void placeItem(levelstructs_t *level,int x,int y,item_t *item)
{
   item_t *oldtop;
   item_t *cstack;
   long offset=FOOGRID_IDX(map,y,x);
   maploc_t *mapstart=map->Data;
   if (NULL==item) return;

/* OLD code:
   item->Next=(mapstart+offset)->Itemlist;
   (mapstart+offset)->Itemlist=item;
   item->X=x; item->Y=y;
   -* Remember where it is *-

   -* Now put the item back into the level-wide item list *-
   item->Next2=level->Itemlist;
   level->Itemlist=item;
   END OLD CODE */
   oldtop=(mapstart+offset)->Itemlist;
   /* NOW, yet another change; must use "combinableItem()",
   with its behaviour, doing any combining if necessary. */

   /* OLD CODE 2 !!!
   top=combine_items(item,oldtop);
   if(top!=oldtop) { -* item didn't merge, so
       we have to do other stuff *-
       (mapstart+offset)->Itemlist=top; -* is item, really *-
       top->X=x;top->Y=y; -* Could do that to item, should be same*-
       item->Next2=level->Itemlist;
       level->Itemlist=item;
   } END OLD CODE 2!!! */
   cstack=combinableItem(item,oldtop);
   if(NULL==cstack) { /* Couldn't merge, so place on top */
       item->Next=oldtop;
       (mapstart+offset)->Itemlist=item;
       item->X=x;item->Y=y;
       item->Next2=level->Itemlist;
       level->Itemlist=item;
   } else { /* We could merge something, so lets */
       cstack->Number+=item->Number; /* Increase stack size */
       freeItem(item);
   }
}

item_t *nextItemByGroupThenList(item_t *head,item_t *current) {
   int first=(current==NULL); /* pass NULL to find first item */
   itemgroup_t currentgroup=(first? 0 : current->Itemtype->Type);
   itemgroup_t stype; /* temp var */
   item_t *next=NULL;
   item_t *scan=NULL;
   itemgroup_t best=IG_LASTGROUP; /* try to minimize this */
   /* FIRST search for any items after current that are within the
   same grouping (according to the inventory listing. If we find
   one, that's our next item */
   if(!first) {
     for(next=current->Next;
       next!=NULL
       && 0!=sortInventoryGroup(next->Itemtype->Type,currentgroup);
       /* That said "0!=" before, didn't seem to work and looked WRONG */
       /* maybe I misunderstood it... I think I did. */
       next=next->Next)
       ;
    }
    if(next!=NULL) return(next);
    /* NOW, if next==NULL, we must have searched the whole list for
    one and drawn a blank. In which case, search *from head* to find
    the first item with a later group */
    for(scan=head; scan!=NULL ; scan=scan->Next) {
       stype=scan->Itemtype->Type;
       if(first||1==sortInventoryGroup(stype,currentgroup))
           /* item is later than current (or is first) */
           if(-1==sortInventoryGroup(stype,best)) {
               best=stype; /* best item has minimum valid group */
               next=scan;
           }
    }
   return(next); /* Next?! */
}

void describe_item(char *space, item_t *item)
{
   int quantity,hasvowel,pluralunit;
   int known=1;
   itemtype_t *it;
   itemgroup_t group;
   char schar; /* first character of name */
   char qdesc[10];
   if (item==NULL) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Trying to describe NULL item!");
     crash(INCONSISTENT);
   }
   if (space==NULL) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Trying to describe an item into unallocated space!");
     crash(INCONSISTENT);
   }
   /* we print into space. The caller must allocate it, because we
   don't want to... */
   quantity=item->Number;
   if(quantity==0) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Trying to describe stack of 0 items!");
     crash(INCONSISTENT);
   }
   it=item->Itemtype;
   group=it->Type;
   /* Mar 2004- changed to reflect new location of Known attribute */
   if(group==IG_POTION) known=it->Known; /* was in typedata */
   /* get first char */
   schar=it->Name[0];/* default */
   if (!known){
      switch(group) {
        case IG_POTION:
         schar=(it->Typedata.Potion.Dname)[0];
         break;
        case IG_WAND:
        case IG_ROD:
        case IG_SCROLL:
        case IG_BOOK:
         bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Non-potion item had \"unknown\" status when describing it...");
         crash(INCONSISTENT);
         break;
        default:
         bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Non-flavoured item had unknown type when describing it...");
         crash(INCONSISTENT);
         break;
      }
   }
   pluralunit=(group==IG_GLOVES||group==IG_FOOTWEAR); /* Other itemtypes too*/
   if (quantity==1) { /* should we use a or an? */
      /*schar=it->Name[0];  already got */
      if(schar<='Z') schar+=('a'-'A');
      hasvowel=(schar=='a'||schar=='e'||schar=='i'||schar=='o'||schar=='u');
      /* Perhaps this could be stored with the itemtype itself?? */
      sprintf(qdesc,(item->Cursed||!hasvowel)?"a ":"an ");
   }
   else sprintf(qdesc,"%d ",quantity);
   if(pluralunit) sprintempty(qdesc); /* don't use any qualifier */
   if(!known) {
     sprintf(space,"%s%s%s%s",qdesc,item->Cursed?"cursed ":"",it->Typedata.Potion.Dname,(quantity>1)?"s":"");
     return;
   }
   sprintf(space,"%s%s%s%s",qdesc,item->Cursed?"cursed ":"",it->Name,(quantity>1)?"s":"");
   /* It's going to need a lot of working on */ 
}

void item_basename(char *space,item_t *item /* ,int oneof*/)
{
   /* print a (s) at the end, but no a/an/number, and no description */
   itemtype_t *it;
   int quantity/* ,hasvowel*/;
   int known;
   if (item==NULL) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Trying to give name of NULL item!");
     crash(INCONSISTENT);
   }
   if (space==NULL) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Trying to describe an item into unallocated space!");
     crash(INCONSISTENT);
   }
   quantity=item->Number;
   if(quantity==0) {
     bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"Trying to describe stack of 0 items!");
     crash(INCONSISTENT);
   }
   it=item->Itemtype;
   /* Should also print alternative name depending on
   whether it's known */
   if(it->Type==IG_POTION) { /* Mar 2004- changed for new location of Known*/
      known=it->Known; /* was in typedata */
      sprintf(space,"%s%s",known?it->Name:it->Typedata.Potion.Dname,(quantity>1)?"s":"");
      return;
   }
   sprintf(space,"%s%s",it->Name,(quantity>1)?"s":"");
}

/* Had ONCE been "combine_tems()", which searched through "group" and
combined "item" with any appropriate stack found. HOWEVER this idea was
soon seen to be deeply flawed. NOW, the function is differently named
to reflect the new, more general functionality of simply returning a
stack from "group" that COULD be combined with "item". It is then up
to the caller to do any combining- it might not want to. */
item_t *combinableItem(item_t *item, item_t *group) {
   itemtype_t *match=item->Itemtype;
   int mcursed=item->Cursed;
   item_t *scan;
   switch (match->Type) {
      case IG_MISSILE:
      case IG_SCROLL:
      case IG_FOOD:
      case IG_CORPSE:
      case IG_CONDIMENT:
      case IG_POTION:
      case IG_TOOL:
      case IG_STONE:
      case IG_COINAGE:
        break; /* because these are the ones that stack */
      default:
        return(NULL); /* the rest don't stack */
   }
   for(scan=group;scan!=NULL;scan=scan->Next)
   {
      if(scan->Itemtype==match &&scan->Cursed==mcursed) /* SHOULD be more sophisticated */
      {
         return(scan); /* That's all we do now */
      }
   }
   return(NULL); /* That's all we do here now, too */
}

int isinclass(itemtype_t *type,itemclass_t cl)
{
   itemgroup_t gr=type->Type;
   switch (cl) {
      case IC_WIELDABLES:
        return(gr==IG_WEAPON||gr==IG_RANGED); /* I think this included others*/
      case IC_WEARABLES:
        return(gr==IG_SUIT||gr==IG_SHIELD||gr==IG_HEADGEAR||gr==IG_GLOVES||gr==IG_FOOTWEAR||gr==IG_RING||gr==IG_NECKWEAR);
      case IC_WEAPONS:
        return(gr==IG_WEAPON); /* Um... why? */
      case IC_ARMOURS:
        return(gr==IG_SUIT||gr==IG_HEADGEAR||gr==IG_GLOVES||gr==IG_FOOTWEAR);
      case IC_JEWELLERY:
        return(gr==IG_RING||gr==IG_NECKWEAR);
      case IC_READABLES:
        return(gr==IG_SCROLL||gr==IG_BOOK);
      case IC_COMESTIBLES:
        return(gr==IG_FOOD||gr==IG_CORPSE||gr==IG_CONDIMENT||gr==IG_POTION);
      case IC_ZAPPABLES:
        return(gr==IG_WAND||gr==IG_ROD);
      case IC_VALUABLES:
        return(gr==IG_STONE||gr==IG_COINAGE); /* not all stones though! */
      case IC_SMALL:
        return(gr==IG_COINAGE||gr==IG_RING||gr==IG_NECKWEAR); /* Others too*/
      case IC_ALLCLASSES:
        return(1);
      case IC_NOCLASSES:
        return(0);
   }
   bugger(REPTYPE_ITEMS|REPTYPE_CODE,LOGLEVEL_WARNING,"isinclass() screwed up somehow!");
   return(0);
}

/* --- end of item.c --- */

