/* ==== los.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h" /*Do we?*/

/* protos */
#include "makemap_p.h"
#include "crash.h"

/* I spent hours trying to dig out my old 6th form project (I
   wanted to connect an exercise bike to a computer and have a
   "virtual reality" type game controlled by it. It sort of
   worked but the dongle never got fixed to the bike. And it
   wasn't VR as anyone else'd recognise it), as I remembered
   it had an effective LOS algorithm for the 2d "indoors"
   routine. After I eventually deciphered my 8 years old
   quickbasic spaghetti code, I sadly realised that whilst
   the movement was cell-based, the LOS routine was not, and
   worked by simply drawing trapezoidal blocks over ares hidden
   behind walls. IIRC it wasn't actually noticably slow, even
   on the old 486s, and it was pretty neat having cells only
   partially obscured.
   Anyway, I realise that I can use a variation of what I had
   used with that to set bits of a bitmap array every turn,
   and use that array to determine visibility when drawing
   the dungeon. */

/* char los_vistab[21*21]; Now in globals.c */
extern char los_vistab[];

/* char los_blocktab[21*21]; Now in globals.c */
extern char los_blocktab[];
/* Used in updating LOS each turn. Each element
  gives indices of squares that would be obscured by a block being there.
  Obviously needs either a linked list of such indices, or several tables*/

/* Only for our rogue style LOS */
/* extern roomdata_t room[]; BEING REPLACED by tls->Room[] */

/* This isn't what is wanted*/

int in_los(int px,int py,int x,int y)
{
   /* Returns 1 if from (cx,xy), a point at (cx+dx,cy+dy) is visible*/
/*   int count,ratio or something; */
   /* Getting the code from VREX (my 6th form project ;)*/
   /* For using calc_los, just read the table. */
   int tx,ty;
   ty=(y-py)+10;tx=(x-px)+10;
#ifdef SEE_ALL
 return (1);  /* For debugging!*/
#endif
   if(ty<0||ty>20||tx<0||tx>20) return(0); /* Outside range.*/
   return (los_vistab[tx+(ty*21)]);
}

void calc_los(foogrid *map, int cx,int cy)
{
   /* Sets or clears the elements of los_vistab[] depending on
   current visibility*/
   /* For the moment, will just use the old LOS of the original rogue!
    In other words, when you're inside a room, you can see all of the
    room. When you're in a tunnel you can see adjoining squares.
    When you're on a door, you can see adjoining squares, plus the
    room.*/
    int dy,dx,offset;
    int x,y; /* calculate the real coords */
    int r,rx1,ry1, rx2,ry2;
    maploc_t *mapstart=map->Data;
/*  return;  whilst debugging!! */
    switch((mapstart+FOOGRID_IDX(map,cy,cx))->Tiletype)
    {
       case PTL_PWALL:
       case PTL_ROCK:
       case PTL_HWALL:
       case PTL_VWALL:
       case PTL_CORNER:
       case PTL_DOOR:
          /* This should never happen. But,
             it could if you make_map() into a wall.*/
          for(dy=0;dy<21;dy++)
          {
             offset=dy*21;
             for(dx=0;dx<21;dx++)
             {
                if(dy>8&&dy<12 && dx>8&&dx<12) los_vistab[offset+dx]=1;
                else los_vistab[offset+dx]=0;
             }
          }
          break;
       case PTL_ODOOR:
          r=find_room(cx,cy);
          rx1=tls->Room[r].Xstart;ry1=tls->Room[r].Ystart;
          rx2=rx1+tls->Room[r].Width-1;
          ry2=ry1+tls->Room[r].Height-1;
          for(dy=0;dy<21;dy++)
          {
             offset=dy*21;
             for(dx=0;dx<21;dx++)
             {
#ifdef ROGUE_TUNNELS

                if(dy>8&&dy<12 && dx>8&&dx<12) {
                   x=dx+cx-10;y=dy+cy-10;
                   r=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
                   los_vistab[offset+dx]=(r!=PTL_ROCK&& r!=PTL_PWALL);
                   /* anything else -that INCLUDES walls of room is in los */
                }
#else
                if(dy>8&&dy<12 && dx>8&&dx<12) los_vistab[offset+dx]=1;
#endif
                else
                {
                   x=dx+cx-10; y=dy+cy-10;
                   if(rx1<=x&&rx2>=x && ry1<=y&&ry2>=y)
                      los_vistab[offset+dx]=1; /* In the room */
                   else los_vistab[offset+dx]=0;
                }
             }
          }
          break;
       case PTL_TUNNEL:
          for(dy=0;dy<21;dy++)
          {
             offset=dy*21;
             for(dx=0;dx<21;dx++)
             {
#ifdef ROGUE_TUNNELS
                /* Only show cells that are NOT wall...*/
                if(dy>8&&dy<12&&dx>8&&dx<12) {
                   x=dx+cx-10;y=dy+cy-10;
                   r=(mapstart+FOOGRID_IDX(map,y,x))->Tiletype;
                   los_vistab[offset+dx]=(r==PTL_TUNNEL||r==PTL_DOOR||r==PTL_ODOOR);
                }
#else
                /* Original style */
                if(dy>8&&dy<12&&dx>8&&dx<12) los_vistab[offset+dx]=1;
#endif
                else los_vistab[offset+dx]=0;
             }
          }
          break;
       case PTL_STAIRSUP:
       case PTL_STAIRSDOWN:
       case PTL_ALTAR: /* And prolly other features too */
       case PTL_FLOOR:
       case PTL_SHOP: /* Don't know where this'll be tho */
       case PTL_RMID:
          r=find_room(cx,cy);
          rx1=tls->Room[r].Xstart;ry1=tls->Room[r].Ystart;
          rx2=rx1+tls->Room[r].Width-1;
          ry2=ry1+tls->Room[r].Height-1;
          for(dy=0;dy<21;dy++)
          {
             offset=dy*21;
             for(dx=0;dx<21;dx++)
             {
                x=dx+cx-10; y=dy+cy-10;
                if(rx1<=x&&rx2>=x && ry1<=y&&ry2>=y)
                   los_vistab[offset+dx]=1; /* In the room */
                else los_vistab[offset+dx]=0;
             }
          }
          break;
    }
    return;
}

void init_los()
{
   /* Sets up los_blocktab[], etc. Only needs to be called once.*/
}
