/* ==== monster.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include <stdlib.h>
/* #include <math.h>  for calculating HP rates, etc */
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h"
#include "maparray.h"
#include "player_t.h"
#include "ai_t.h"

/* protos */
#include "ai_p.h"
#include "dXrng.h"
#include "monster.h"
#include "los.h"
#include "ui.h"
#include "item_p.h" /* For the corpses */
#include "mg_utils.h"
#include "memutils.h"
#include "combat.h"
#include "race_p.h"
#include "crash.h"
#include "dungeon_p.h"

/* foogrid *map SHOULD be declared global in maparray.h anyway */
monster_t *add_mob(levelstructs_t *level,int x,int y,mobtype_t *mtype)
{
   monster_t *newmob;
   aicache_memory_t *mmem;
   unsigned int elev,str,dex,agi,con, tr; /* training would strictly be
     very high max, but we won't get near from here */
   long offset=FOOGRID_IDX(map,y,x);
   maploc_t *mapstart=map->Data;
   newmob=(monster_t *)counting_malloc(MEM_MONSTER,sizeof(monster_t));
   if (newmob==NULL) crash(NO_MEM);

   newmob->Mobtype=mtype;
   newmob->Inventlist=NULL; /*Carrying nothing.*/
   newmob->X=x;newmob->Y=y; /*Monster knows where it is*/

   newmob->Ticker=25+roll_dX(d20set[0]); /* monster nearly ready to move*/

#define MON_INIT_SCALE() (19+(10*roll_dX(d8set[0]))+roll_dX(d10set[0]))
/* Derived from PLAYER_INIT_SCALE in player.c */

/*
   newmob->Var_stats.MaxHp=10;  Uh... */
   elev=mtype->BaseLevel;
   tr=MON_INIT_SCALE();
/* newmob->Training.Str=tr; */str=global_scale_stat(mtype->Stat_max.Str,tr);
   tr=MON_INIT_SCALE();
/* newmob->Training.Dex=tr; */dex=global_scale_stat(mtype->Stat_max.Dex,tr);
   tr=MON_INIT_SCALE();
/* newmob->Training.Agi=tr; */agi=global_scale_stat(mtype->Stat_max.Agi,tr);
   tr=MON_INIT_SCALE();
/* newmob->Training.Con=tr; */con=global_scale_stat(mtype->Stat_max.Con,tr);

   newmob->Var_stats.Level=elev;
   if(elev==0) elev=1; /* Not really good, but... */

   newmob->Fixed_stats.Str=str; newmob->Fixed_stats.Dex=dex;
   newmob->Fixed_stats.Agi=agi; newmob->Fixed_stats.Con=con;

   /* N^0.667= 1.5root of N */
/* newmob->Var_stats.MaxHp=(pow(con,0.66667))*(log(1+elev)); Now done in */
   newmob->Var_stats.MaxHp=global_scale_hp(elev,con);
   newmob->Var_stats.Hp = newmob->Var_stats.MaxHp;

   /* AI stuff */
   mmem=(aicache_memory_t *)counting_malloc(MEM_MMEM,sizeof(aicache_memory_t));
   if (mmem==NULL) crash(NO_MEM);
   newmob->Memory=mmem;
   mmem->Roomcache=NULL;
   mmem->Tunnelcache=NULL;
   /* Now buffer entries: Don't need setting up, just creating. */
   mmem->Roombuffer=(aicache_rooment_t *)counting_malloc(MEM_ROOMMEM,sizeof(aicache_rooment_t));
   if (mmem->Roombuffer==NULL) crash(NO_MEM);
   mmem->Roombuffer->Paths=NULL;
   mmem->Roombuffer->Next=NULL; /* To simplify freeing */
   mmem->Roombuf_flag=0; /* Invalid buffer */
   mmem->Tunnelbuffer=(aicache_tunnelent_t *)counting_malloc(MEM_TUNMEM,sizeof(aicache_tunnelent_t));
   if (mmem->Tunnelbuffer==NULL) crash(NO_MEM);
   mmem->Tunnelbuffer->Paths=NULL;
   mmem->Tunnelbuffer->Next=NULL;
   mmem->Tunnelbuf_flag=0;
   /* Other AI stuff */
   newmob->Ttype=1; /* Lets the AI acquire new target */
   newmob->Annoyance=0; /* Feeling easygoing! */
   newmob->Next=level->Moblist;
   level->Moblist=newmob;
   (mapstart+offset)->Mptr=newmob; /*Map knows where it is*/
   level->Nmobs++; /* Increase the count */
   return(newmob);
}

void new_mobtype(mobtype_t *space, char sym, char *name,char *desc)
{
    int l;
    char *cname;
    space->Symbol=sym;
    space->Colour=PCOL_DWHITE; /* STOPGAP DEFAULT! */
    space->Name=name;
    space->Desc=desc;
    l=1+sprintf(strbuf,"%s corpse",name);/* Ah. */
    cname=(char *)counting_malloc(MEM_STRING,l*sizeof(char));
    sprintf(cname,"%s corpse",name);
    space->Corpsetype=push_itype(IG_CORPSE,cname,10000,1); /* does allocation anyways*/
    /* corpses ought to have some small nonzero chance of being randomly
    generated... but meh, perhaps that could be handled by weaker ones
    getting slaughtered by stronger ones!! It'd be handy to be able to
    specify weight/nutrition of corpse types. */

}

mobtype_t *push_mtype(char sym,char *name,pcolour_t col)
{
    mobtype_t *newmt;
    newmt=(mobtype_t *)counting_malloc(MEM_MTYPE,sizeof(mobtype_t));
    newmt->Next=first_mt; /* Should have an array of mobtypes
       one for each symbol, like push_itype does */
    first_mt=newmt;
    mt_nums++;
    new_mobtype(newmt,sym,name,"A monster with no description");
    newmt->Colour=col;
    return(newmt);
}

/* Was (foogrid *map,dx,dy,mob) */
void move_mob(int dx, int dy, monster_t *mob)
{
    monster_t *opp;
    maploc_t *mapstart=map->Data;
    int x,y,tx,ty;
    long offset;
    tile_t tile;
    x=mob->X;y=mob->Y;
    tx=x+dx;ty=y+dy;
    offset=FOOGRID_IDX(map,ty,tx);
    tile=(mapstart+offset)->Tiletype;
    /* Commented whilst debugging map!!!  */
    if(tile==PTL_ROCK||tile==PTL_VWALL||tile==PTL_HWALL||tile==PTL_PWALL||tile==PTL_DOOR||tile==PTL_CORNER)
    {
       if (in_los(player.X,player.Y,x,y)) {
         sprintf(strbuf,"The %s walks into the %s!",mob->Mobtype->Name,tile==PTL_DOOR?"door":"wall");
         show_messages(strbuf);
       }
       return;
    } /**/
    opp=(mapstart+offset)->Mptr;
    if(opp!=NULL) { /* Unusual... */
#ifdef CALM
       sprintf(strbuf,"Monster at %d,%d attacks something, despite CALM option!",x,y);
       bugger(REPTYPE_AI|REPTYPE_COMBAT,LOGLEVEL_WARNING,strbuf);
#endif
/*     if (in_los(player.X,player.Y,x,y)) {   THIS is handled in m_hit_m().
         sprintf(strbuf,"The %s hits the %s!",mob->Mobtype->Name,opp->Mobtype->Name);
         show_messages(strbuf);
       } */
       m_hit_m(mob,opp);
       return; /* We won't have monsters fight each other this way... Will we?
        Maybe we will change later if we can make sure they normally walk
        around each other if they don't mean to fight */
    }
    if (player.X==tx&&player.Y==ty) {
       m_hit_p(mob); /* Fight!! */
       return; /* No, we don't keep walking... */
    }
    (mapstart+FOOGRID_IDX(map,y,x))->Mptr=NULL;
    /*x+=dx;y+=dy; */
    mob->X=tx;mob->Y=ty;
    (mapstart+FOOGRID_IDX(map,ty,tx))->Mptr=mob;
}

/* Was (foogrid *map,monster_t *mob) and opened any doors.
 Now opens one specific door. */
void m_open_door(monster_t *mob, int dx,int dy)
{
    int x=mob->X,y=mob->Y;
    /* int done=0; no longer used?? */
    /* entrance_t *door; no longer used?? */
    maploc_t *mapstart=map->Data;
    dx+=x, dy+=y;
    if((mapstart+FOOGRID_IDX(map,dy,dx))->Tiletype==PTL_DOOR)
    {
       switch(isUnopenable(dx,dy)) {
         case 0:
           open_door(dx,dy);
           if(in_los(player.X,player.Y,dx,dy)) { /* can see door */
              if(in_los(player.X,player.Y,x,y)) { /* and see monster */
                 sprintf(strbuf,"The %s opens a door",mob->Mobtype->Name);
              } else {
                 sprintf(strbuf,"Something opens a door");
              }
              show_messages(strbuf);
           }
           return;
         case 1:
         case 2:
           sprintf(strbuf,"Monster at %d,%d tries to open a door at %d,%d "
              "where there isn't one!!",x,y,dx,dy);
           bugger(REPTYPE_AI|REPTYPE_MONSTERS|REPTYPE_MAPGEN,LOGLEVEL_WARNING,strbuf);
           return;
         case 3:
         case 4:
           if(in_los(player.X,player.Y,dx,dy)) { /* can see door */
              if(in_los(player.X,player.Y,x,y)) { /* and monster */
                 sprintf(strbuf,"The %s struggles with the door",mob->Mobtype->Name);
              } else { /* can't see monster */
                 sprintf(strbuf,"The door rattles");
              }
              show_messages(strbuf);
           } else { /* can't see door */
              if(in_los(player.X,player.Y,x,y)) {
                 sprintf(strbuf,"The %s struggles with something",mob->Mobtype->Name);
                 show_messages(strbuf);
              }
           }
           return;
         default:
           bugger(REPTYPE_CODE|REPTYPE_MAPGEN,LOGLEVEL_WARNING,"Strange return value from isUnopenable()!");
       }
    }
    /* should no longer reach here! */
}

void m_get_item(monster_t *mob,item_t *item)
{
    /* Assumes item is individual: item->Next MUST be NULL*/
    if (item==NULL) return; /*Play it safe*/
    item->Next=mob->Inventlist;
    mob->Inventlist=item;
}
item_t *m_discard_item(monster_t *mob)
{
    /* Removes top item from monster's inventory. Returns it
      as an individual item.*/
    item_t *item=mob->Inventlist;
    if (item==NULL) return(NULL); /*Plausible*/
    mob->Inventlist=item->Next;
    item->Next=NULL;
    return(item);
}

void m_gainexp(monster_t *mon,int e)
{
    unsigned int nlv=mon->Var_stats.Level+1;
    uint64_t targe=global_scale_expreq(nlv);
    mon->Var_stats.Exp+=e;
    if(mon->Var_stats.Exp>=targe) {
#ifndef AUTOPILOT
       if(in_los(player.X,player.Y,mon->X,mon->Y)) {
#else
       if(in_los(tls->Moblist->X,tls->Moblist->Y,mon->X,mon->Y)) {
#endif
           sprintf(strbuf,"The %s looks more experienced!",mon->Mobtype->Name);
           show_messages(strbuf);
       }
       mon->Var_stats.Level++;
       mon->Var_stats.MaxHp=global_scale_hp(nlv,mon->Fixed_stats.Con);
       /* don't affect clev as p_gainexp does, of course... */
       /* perhaps tell player mon looks more powerful, if it's in LOS */
    }
}

void m_hurt(monster_t *victim,int dmg)
{
    victim->Var_stats.Hp-=dmg;
    /* if monster in LOS, show it's hurt? */
    if(victim->Var_stats.Hp<0) {
#ifndef AUTOPILOT
       if(in_los(player.X,player.Y,victim->X,victim->Y)) {
#else
       if(in_los(tls->Moblist->X,tls->Moblist->Y,victim->X,victim->Y)) {
#endif
          sprintf(strbuf,"The %s is killed by... something or other.",victim->Mobtype->Name);
          show_messages(strbuf);
       }
       m_die(tls,victim);
    }
}

/* Should already have map declared. */
void m_die(levelstructs_t *level, monster_t *victim)
{
    monster_t *next;
    monster_t *scan;
    aicache_memory_t *mmem=victim->Memory;
    maploc_t *mapstart=map->Data;
    int x=victim->X, y=victim->Y;
    placeItem(level,x,y,newItem(victim->Mobtype->Corpsetype,1));
    (mapstart+FOOGRID_IDX(map,y,x))->Mptr=NULL; /* KABOOM! */
    /* We should free this, yes? But if in a
    list of all monsters, need to free it from there too */
    /* Yup. Doing that now. */
    next=victim->Next;
    if(level->Moblist==victim) {
       level->Moblist=next; /* The exception */
    } else {
       for (scan=level->Moblist;scan->Next!=victim;scan=scan->Next);
       scan->Next=next;
    } /* Should now have removed victim from list. */
    level->Nmobs--;
    /* Now remove AI cache contents!!! */
    aicache_freerooments(mmem->Roombuffer);
    aicache_freetunnelents(mmem->Tunnelbuffer);
    aicache_freerooments(mmem->Roomcache);
    aicache_freetunnelents(mmem->Tunnelcache);
    counting_free(MEM_MMEM,mmem);
    counting_free(MEM_MONSTER,victim);
}

int m_whatamion(monster_t *mob)
{
    maploc_t *mapstart=map->Data;
    tile_t floor;
    floor=(mapstart+FOOGRID_IDX(map,mob->Y,mob->X))->Tiletype;
    switch (floor) {
       case PTL_PWALL:
       case PTL_ROCK:
       case PTL_HWALL:
       case PTL_VWALL:
       case PTL_CORNER:
          bugger(REPTYPE_MONSTERS|REPTYPE_AI,LOGLEVEL_FOOBAR,"Monster in a wall??");
          crash(INCONSISTENT);
          break; /*Not really needed*/
       case PTL_TUNNEL:
          return(0);
          break; /*Not really needed*/
       case PTL_DOOR:
       case PTL_ODOOR:
          return(2);
          break;
       case PTL_FLOOR:
       default: /* Includes features, etc */
          return(1);
          break;
    }
    return(3); /* To shut up gcc... */
}

int m_respect(monster_t *observer,monster_t *observed)
{
    /* How patient is observer with observed? */
    return(20);
    /* When the proper game mechanics are implemented, it'll
    work out something like (10*observed->Level)/oberserver->Level
    or something very similar. */
}
