/* ==== player.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */


#include <stdlib.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "item_t.h"
#include "structures.h"
#include "mapgen_types.h" /* For levelmap type */
#include "maparray.h"
#include "dXrng.h"
#include "player_t.h"

/* protos */
#include "item_p.h" /* Used by p_pickup, which may be moved */
#include "makemap_p.h" /* Used for changing level */
#include "mg_utils.h"
#include "player_p.h"
#include "ui.h"
#include "crash.h"
#include "race_p.h"
#include "equip.h"
#include "combat.h"
#include "dungeon_p.h"

/* player_t player; The player character's data
 Now in globals.c */

void init_player(levelmap_t *map,int x,int y)
{
   int i,con;
   player.Inventlist=NULL;
   player.Map=map;
   player.Symbol='@'; /* That's you! */
   player.X=x;player.Y=y;
   player.Ticker=0; /* Already ready to move... but it
      depends what point we update everything in the ctrl loop */

   /* Base stats: These are all in a global scale of 0->1000;
   Each race has its own percentage scale that maps onto parts
   of that global scale. Increasing stats gets progressively
   harder, so you can't reach 100%. */
   /*
   player.Fixed_stats.Str=(75*(1+roll_dX(d20set[0])))/20;
   player.Fixed_stats.Dex=(120*(1+roll_dX(d20set[1])))/20;
   player.Fixed_stats.Agi=(50*(1+roll_dX(d20set[0])))/20;
   player.Fixed_stats.Con=(250*(1+roll_dX(d20set[0])))/20;
   */
#define PLAYER_INIT_SCALE() (21+(10*roll_dX(d8set[0]))+roll_dX(d10set[0]))
   /* Actually gives 21 -> 100, which gives %ages of 30 ->66, so it could
   start off pretty low */
   player.Training.Str=PLAYER_INIT_SCALE();
   player.Training.Dex=PLAYER_INIT_SCALE();
   player.Training.Agi=PLAYER_INIT_SCALE();
   player.Training.Con=PLAYER_INIT_SCALE();
   /*Work out the globally scaled stats */
   player.Fixed_stats.Str=global_scale_stat(75,player.Training.Str);
   player.Fixed_stats.Dex=global_scale_stat(120,player.Training.Dex);
   player.Fixed_stats.Agi=global_scale_stat(50,player.Training.Agi);
   con=global_scale_stat(250,player.Training.Con);
   player.Fixed_stats.Con=con;
   player.Var_stats.Ev=10; /* Or something.*/
/*   player.Var_stats.MaxHp=15; Nope. */
   player.Var_stats.MaxHp=global_scale_hp(1/*baselevel*/,con);
   player.Var_stats.Hp=player.Var_stats.MaxHp;
   player.Var_stats.Satiation=2700; /* content */
   player.SatiationStatus=SAT_NORMAL; /* or derive by categorizeSatiation()*/
   player.Var_stats.Exp=0; /* Yup */
   player.Var_stats.Level=1;/* not 0, no. */
   for(i=0;i<10;i++)player.Equipment[i]=NULL; /* Nothing wielded/worn */
}


/* Adapted from move_mob */
void p_move(int dx, int dy)
{
    maploc_t *mapstart=map->Data;
    maploc_t *elem;
    int x,y;
    char fstring[80];
    monster_t *mptr;
    tile_t tile;
    x=player.X;y=player.Y;
    elem=(mapstart+FOOGRID_IDX(map,y+dy,x+dx));
    mptr=elem->Mptr;
    if(NULL!=mptr)
    {
      p_hit_m(mptr);
      return;
    }
    tile=elem->Tiletype;
    if(tile==PTL_ROCK||tile==PTL_VWALL||tile==PTL_HWALL||tile==PTL_PWALL||tile==PTL_DOOR||tile==PTL_CORNER)
    {
       sprintf(strbuf,"You walk into the %s!",tile==PTL_DOOR?"door":"wall");
       show_messages(strbuf);
       return;
    }
    x+=dx;y+=dy;
    player.X=x;player.Y=y;
    if(isfeature(x,y)) {
       describe_feature(fstring,x,y);
       sprintf(strbuf,"There %s %s here.",describe_feature(fstring,x,y)?"are":"is",fstring);
       show_messages(strbuf);
    }
}


void p_open_door() /* Now fixed to ask which door to open :) */
{
    int x=player.X,y=player.Y;
    int dx,dy; /* vector to door */

    direction_t doordir=checkForTile(x,y,PTL_DOOR,0);
    if(doordir==FACE_NONE) {
       show_messages("There's no door to open!");
       return;
    }
    if(doordir==FACE_ERROR) {
       doordir=userSelectDir("Open which door",0,0);
       if(doordir==FACE_NONE) {
          show_messages("Cancelled.");
          return;
       }
    }
    dx=FACING_TO_X(doordir);
    dy=FACING_TO_Y(doordir);
    x+=dx,y+=dy;
    switch(isUnopenable(x,y)) {
       case 0: /* OK */
         open_door(x,y);
         show_messages("You open the door.");
         break;
       case 1: /* not door */
         show_messages("There's no door there!");
         break;
       case 2: /* open */
         show_messages("It's already open!");
         break;
       case 3: /* locked */
         show_messages("It seems to be locked?! Somehow!");
         break;
       case 4: /* stuck*/
         show_messages("The door is stuck!");
         break;
       default: bugger(REPTYPE_MAPGEN|REPTYPE_CODE,LOGLEVEL_WARNING,"Strange value returned from isUnopenable()");
         show_messages("Whoah, strange door, dude!");
    }
}

void p_close_door() /* Now changed to ask which door to close */
{
    int x=player.X,y=player.Y;
    int dx,dy; /* vector to door */
    direction_t doordir=checkForTile(x,y,PTL_ODOOR,0);
    if(doordir==FACE_NONE) {
       show_messages("There's no door to close!");
       return;
    }
    if(doordir==FACE_ERROR) {
       doordir=userSelectDir("Close which door",0,0);
       if(doordir==FACE_NONE) {
          show_messages("Cancelled.");
          return;
       }
    }
    dx=FACING_TO_X(doordir);
    dy=FACING_TO_Y(doordir);
    x+=dx,y+=dy;
    switch(isUncloseable(x,y)) {
        case 0: /* OK */
          close_door(x,y);
          show_messages("You close the door.");
          break;
        case 1: /* None */
          show_messages("There's no door there!");
          break;
        case 2: /* closed */
          show_messages("It's already closed!");
          break; 
        case 3: /* blockage */
          show_messages("The door is blocked!");
          break;
        case 4:
          show_messages("The door is stuck!");
          break;
        default: bugger(REPTYPE_MAPGEN|REPTYPE_CODE,LOGLEVEL_WARNING,"Strange value returned from isCloseable()");
          show_messages("Whoah, strange door, dude!");
    }
}

/* Change to put items into player inventory, merge items,
   and *set inventory letters* if appropriate */
void p_get_item(item_t *item) /* Needs new name */
{
    /* Assumes item is individual: item->Next MUST be NULL*/
    char letter;
    item_t *cstack; /* a stack of items that item could merge with */
    item_t *oldtop;
    item_t *oldend;
    if (NULL==item) return; /*Play it safe*/

    /* OLD code 
    letter=first_unused_invletter();
    if(letter==' ') {
        -* Your inv is full!*-
        crash(INCONSISTENT);
        -* we shouldn't have been called *-
    }
    item->Next=player.Inventlist;
    item->Invletter=letter;
    tag_invletter_usage(letter,0);
    player.Inventlist=item; END OLD CODe */

    /* OLD CODE 2!!!
    oldtop=player.Inventlist;
    top=combine_items(item,oldtop);
    if (top!=oldtop) {  -* Items weren't combined, so muck about *-
       letter=first_unused_invletter();
       if(letter==' ') {
          -* Shouldn't have been called.
          BUT, how would caller know if we would have
          combined stacks without calling combine_stacks?
          Duh. We need to have a different function,
          that searches the heap for an item that COULD
          be combined with, and whose caller does the
          combining :P *-
          crash(INCONSISTENT);
       }
       top->Next=oldtop;
       player.Inventlist=top;
       item->Invletter=letter;
       tag_invletter_usage(letter,0);
    } END OF OLD CODE 2!!! */
    oldtop=player.Inventlist;
    oldend=player.Inventend; /* Now, we append to the end! */
    cstack=combinableItem(item,oldtop);
    if(NULL==cstack) { /* couldn't combine anything */
       letter=firstUnusedInvletter();
       if(letter==' ') {
          crash(INCONSISTENT); /* caller should have checked */
       }
#ifdef OLDINVENT
       /* WAS placing at inventory top. */
       item->Next=oldtop;
       player.Inventlist=item;
#else
       /* Now, place at end of inventory */
       if(oldtop==NULL) { /*The exception case */
          item->Next=oldtop;
          player.Inventlist=item;
          player.Inventend=item;
       } else { /* Normal, tag onto end */
          oldend->Next=item;
          player.Inventend=item;
          item->Next=NULL;
       }
#endif

       /* Mar 2004: Changed to check if item had a previous letter
       that could be used instead */
       if(item->Invletter!=' ' /* previously held */
        && isInvletterAvailable(item->Invletter)) letter=item->Invletter;
       else item->Invletter=letter;
       tagInvletterUsage(letter,0);
    } else { /* COULD combine stacks. So do so! */
       cstack->Number+=item->Number; /* Stack size increased */
       freeItem(item); /* Um */
       /* THAT wasn't too hard, was it?
       The tough bit should be working out WHAT you can merge */
    }

}

/* Now tries to discard named item, if it can't,
will throw error */
void p_discard_item(item_t *item)
{
    /* Removes given item from player's inventory. That's it. */
 /* item_t *item; */
    item_t *prev;
/*  item=prompt_for_inv_item(); Now caller does that. */
/*  if (NULL==item) return(NULL); Plausible- No, now stupid*/
    if (NULL==item) {
       bugger(REPTYPE_ITEMS|REPTYPE_CODE,LOGLEVEL_FOOBAR,"p_discard_item() called for NULL item");
       crash(OUT_OF_BOUNDS); /* So there. */
    }
    /* Now need some function that takes *item* from
       list (removing it) and passes it back. But if we
       take item from the head of the list, the pointer
       to the list has changed!!! So we check the
       head of list first.*/
    if (p_has_equipped(item)) {/* caller should have checked */
       bugger(REPTYPE_ITEMS|REPTYPE_EQUIP,LOGLEVEL_FOOBAR,"p_discard_item() called for equipped item.");
       crash(FUNCTION_ERROR); /* Not really,
      but the caller should have checked it */
    }
    /* Now, we do the removing */

    if (player.Inventlist==item)
    { /* The item is the head of list. */
       player.Inventlist=item->Next;
#ifndef OLDINVENT
       if(item==player.Inventend) player.Inventend=NULL; /* I think! */
#endif
    } else { /* Search for it. */
       for(prev=player.Inventlist;prev->Next!=item&&prev->Next!=NULL;prev=prev->Next);
       if(prev->Next==NULL) {
          bugger(REPTYPE_ITEMS,LOGLEVEL_FOOBAR,"p_discard_item() couldn't find the item to discard in player inventory");
          crash(INCONSISTENT);
       }
       prev->Next=item->Next;/* Snip item out */
#ifndef OLDINVENT
       if(item==player.Inventend) player.Inventend=prev;
#endif

    }
    item->Next=NULL;
    tagInvletterUsage(item->Invletter,1);
}

void p_pickup() /* A combination of p_get_item,prompt_for_heap_item,skimItem*/
{
    maploc_t *mapstart=map->Data;
    char nstring[80];
    item_t *item;
    item_t *heap=(mapstart+FOOGRID_IDX(map,player.Y,player.X))->Itemlist;
    if(NULL==heap){
       show_messages("There's nothing to pick up here!");
       return;
    }
    /* At this point, we ought to check that we've got space, no? */

    if(heap->Next==NULL)
    { /* Only one item. Get it w/out question */
       /* NO, work out if it'll fit in inventory first ... */
       if(' '==firstUnusedInvletter() && NULL==combinableItem(heap,player.Inventlist)) {
          show_messages("Your inventory's full already!");
          return;
       }
       /* Also check if item->Weight+inv->Weight>player.Wcapacity */
       item_basename(nstring,heap);
       skimItem(tls,player.X,player.Y,heap); /* Removes that item from floor*/
       p_get_item(heap); /* Gives it to you! */
       item=heap;
    } else { /* More than one item. */
       item=prompt_for_heap_item(heap);
       if(NULL==item) return; /* You decided not to */
       if(' '==firstUnusedInvletter() && NULL==combinableItem(item,player.Inventlist)) {
          show_messages("Your inventory's full already!");
          return;
       }
       /* Also check if item->Weight+inv->Weight>player.Wcapacity */
       item_basename(nstring,item);
       skimItem(tls,player.X,player.Y,item); /* Removes that item from heap*/
       p_get_item(item);
    }
    sprintf(strbuf,"You pick up the %s.",nstring);
    show_messages(strbuf);
}
void p_drop()
{
/*  maploc_t *mapstart=map->Data; */
    char nstring[80];
    item_t *item;
/*  item_t *heap=(mapstart+FOOGRID_IDX(map,player.Y,player.X))->Itemlist; */
    item=prompt_for_inv_item(0);
    if(NULL==item) return;
    if (p_has_equipped(item)) {
       show_messages("You can't drop things you've equipped!");
       return;
    }
    item_basename(nstring,item);
    p_discard_item(item);
    /*p_discard_item() *used* to prompt for the
    item to drop, remove it, and pass it back. Now, it just removes
    a given item. MUCH more useful. */
    placeItem(tls,player.X,player.Y,item);
    sprintf(strbuf,"You drop the %s.",nstring);
    show_messages(strbuf);
}

void p_try_upstairs()
{
    maploc_t *mapstart=map->Data;
    maploc_t *this=mapstart+FOOGRID_IDX(map,player.Y,player.X);
    locid_t *dest;
    levelmap_t *new;
    if (PTL_STAIRSUP==this->Tiletype)
    {
       /* Well, these are stairs up, so lets try going up... */
       show_messages("With great trepidation, you go up the stairs...");
       /* Um...*/
       dest=tls->Features[this->Featureidx].Extra.Destination;
       if (dest->Map==NULL) {
           /* New level. */
           lastlevel=thislevel;
           new=level_skel();
           dest->Map=new;
           tls=new->Unpacked;
           make_dmap(new,0);
           thislevel=new;
           /* This works because make_dmap() moves the player to the stiars*/
           dest->X=player.X; dest->Y=player.Y;
       } else {
           /* An old level. */
           lastlevel=thislevel; /* Do we even need this here? */
           thislevel=dest->Map;
           player.X=dest->X;
           player.Y=dest->Y;
           tls=thislevel->Unpacked;
           remake_dmap(thislevel);
       }
       /* We should now be on another level */
    } else {
       show_messages("There is no way up here.");
    }
}

void p_try_downstairs()
{
    maploc_t *mapstart=map->Data;
    maploc_t *this=mapstart+FOOGRID_IDX(map,player.Y,player.X);
    locid_t *dest;
    levelmap_t *new;
    if (PTL_STAIRSDOWN==this->Tiletype)
    {
       /* Well, these are stairs down, so lets try going down... */
       show_messages("With great trepidation, you go down the stairs...");
      /* Um...*/
      dest=tls->Features[this->Featureidx].Extra.Destination;
      if (dest->Map==NULL) {
          /* New level. */
          lastlevel=thislevel;
          new=level_skel();
          dest->Map=new;
          tls=new->Unpacked;
          make_dmap(new,1);
          thislevel=new;
          dest->X=player.X; dest->Y=player.Y;
      } else {
          /* An old level. */
          lastlevel=thislevel;
          thislevel=dest->Map;
          player.X=dest->X;
          player.Y=dest->Y;
          tls=thislevel->Unpacked;
          remake_dmap(thislevel);
      }
    } else {
       show_messages("There is no way down here.");
    }
}

satiation_t categorizeSatiation(int satiation) {
    if(satiation<2001) { /* hungry */
       if(satiation<501) { /* SERIOUS */
          if(satiation>100) return SAT_WEAK;
          if(satiation>30) return SAT_FAINTING;
          return SAT_STARVING;
       } else { /* less hungry */
          if(satiation<901) return SAT_VERYHUNGRY;
          return SAT_HUNGRY;
       }
    } else { /* not hungry */
       if(satiation>5000) { /* way too full */
          if(satiation>6000) return SAT_BURSTING;
          return SAT_BLOATED;
       } else {
          if(satiation>3500) return SAT_SATIATED;
          return SAT_NORMAL;
       }
    }
}

char *descSatiation(satiation_t state,int banner) {
    char *satDesc[]={
       "starving",
       "fainting",
       "weak",
       "very hungry",
       "hungry",
       "",
       "satiated",
       "bloated",
       "bursting"
    };
    static char localbuf[15]; /* few too many. CAN we pass this back?? */
    if(banner) { /* for draw_status */
       strcpy(localbuf,satDesc[state]);
       if(state==SAT_STARVING||state==SAT_BURSTING) stoupper(localbuf);
       if(state<SAT_VERYHUNGRY||state>SAT_SATIATED) strcat(localbuf,"!");
       return(localbuf);
    }
    return(satDesc[state]);
}

void p_incSatiation(unsigned int satDelta) {
    satiation_t newStatus,oldStatus=player.SatiationStatus;
    newStatus=categorizeSatiation(player.Var_stats.Satiation+=satDelta);
    player.SatiationStatus=newStatus;
    if(newStatus!=oldStatus) { /* describe the change */
       if(oldStatus<SAT_VERYHUNGRY) {
          switch(newStatus) {
            case SAT_FAINTING:
            case SAT_WEAK:
               sprintf(strbuf,"You're no longer %s, but you're still %s!",descSatiation(oldStatus,0),descSatiation(newStatus,0));
               break;
            case SAT_VERYHUNGRY:
            case SAT_HUNGRY:
               sprintf(strbuf,"You're no longer %s, merely %s.",descSatiation(oldStatus,0),descSatiation(newStatus,0));
               break;
            case SAT_NORMAL:
               sprintf(strbuf,"You're no longer %s- you feel fine now!",descSatiation(oldStatus,0));
               break;
            case SAT_SATIATED:
            case SAT_BLOATED:
            case SAT_BURSTING:
               sprintf(strbuf,"You're no longer %s- in fact now you're %s!!",descSatiation(oldStatus,0),descSatiation(newStatus,0));
               break;
            case SAT_STARVING:
               bugger(REPTYPE_CODE,LOGLEVEL_FOOBAR,"p_incSatiation() found change in satiation that left satiation the same?!");
               return;
               break;
          }
          show_messages(strbuf);
          return;
       }
       if(oldStatus<SAT_NORMAL) { /* but would be >SAT_WEAK */
          switch(newStatus) {
             case SAT_HUNGRY:
               show_messages("You're no longer so hungry.");
               return;
             case SAT_NORMAL:
               show_messages("You're no longer hungry.");
               return;
             case SAT_SATIATED:
             case SAT_BLOATED:
             case SAT_BURSTING:
               sprintf(strbuf,"You're no longer hungry- in fact, now you're %s!",descSatiation(newStatus,0));
               show_messages(strbuf);
               return;
             case SAT_STARVING:
             case SAT_FAINTING:
             case SAT_WEAK:
             case SAT_VERYHUNGRY:
               bugger(REPTYPE_CODE,LOGLEVEL_FOOBAR,"p_incSatiation() found impossible satiation transition!");
               return;
          }
       }
       sprintf(strbuf,"You're %s!",descSatiation(newStatus,0));
       show_messages(strbuf);
       return;
    }
    if(newStatus<SAT_NORMAL) {
       sprintf(strbuf,"You're still %s%c",descSatiation(newStatus,0),newStatus<SAT_VERYHUNGRY?'!':'.');
       show_messages(strbuf);
    }
}

void p_decSatiation(unsigned int satDelta) {
    satiation_t newStatus,oldStatus=player.SatiationStatus;
    newStatus=categorizeSatiation(player.Var_stats.Satiation-=satDelta);
    player.SatiationStatus=newStatus;
    if(newStatus!=oldStatus) { /* describe the change */
       if(oldStatus>SAT_NORMAL) {
          switch(newStatus) {
             case SAT_BLOATED:
             case SAT_SATIATED:
               sprintf(strbuf,"You're no longer %s, but you're still %s%c",descSatiation(oldStatus,0),descSatiation(newStatus,0),newStatus==SAT_BLOATED?'!':'.');
               break;
             case SAT_NORMAL:
               sprintf(strbuf,"You're no longer %s. You feel comfortable.",descSatiation(oldStatus,0));
               break;
             default:
               sprintf(strbuf,"You're no longer %s. In fact, you're %s!",descSatiation(oldStatus,0),descSatiation(newStatus,0));
               break;
          }
          show_messages(strbuf);
          return;
       }
       sprintf(strbuf,"You're %s%c",descSatiation(newStatus,0),newStatus<SAT_VERYHUNGRY?'!':'.');
       show_messages(strbuf);
    }
    /* otherwise print nothing, because satiation is decremented
    every move, and you'd go NUTS! */
}

void p_setSatiation(unsigned int newSatiation) {
    int oldSatiation=player.Var_stats.Satiation;
    /* it's easier IMO to split the state change messages into
    increased satiation and decreased satiation. So inc and dec don't
    call set, but the exact opposite! */
    if(newSatiation>oldSatiation) {
      p_incSatiation(newSatiation-oldSatiation);
      return;
    }
    if(newSatiation<oldSatiation)
      p_decSatiation(oldSatiation-newSatiation);
    /* if it was the same, we do nothing :) */
}

int p_whatamion()
{
    maploc_t *mapstart=map->Data;
    tile_t floor;
    floor=(mapstart+FOOGRID_IDX(map,player.Y,player.X))->Tiletype;
    switch (floor) {
       case PTL_PWALL:
       case PTL_ROCK:
       case PTL_HWALL:
       case PTL_VWALL:
       case PTL_CORNER:
          show_messages("Player in a wall??");
          crash(INCONSISTENT);
          break; /*Not really needed*/
       case PTL_TUNNEL:
          return(0);
          break; /*Not really needed*/
       case PTL_DOOR:
       case PTL_ODOOR:
          return(2);
          break;
       case PTL_FLOOR:
       default: /* Includes features, etc */
          return(1);
          break;
    }
    return(3); /* To shut up gcc */
}

int p_eat(item_t *item) {
    sprintf(strbuf,"You eat the %s.",item->Itemtype->Name);
    show_messages(strbuf);
    /* player.Var_stats.Satiation+=700; -* STOPGAP!! */
    p_incSatiation(700); /* STILL a stopgap!*/
    /* item decrement code copied from quaff_main() */
    if(item->Number==1) {
       p_discard_item(item); /* So we don't screw up things! */
       freeItem(item); /* So we don't leak */
    } else (item->Number)--; /* simple, eh? */
    return(3000); /* I don't know how long it should take...
              ...would probably depend on food type. Weight? */
}

int p_tryEat() {
    /* prompt for an item, see if it's edible, if so, eat it */
    item_t *item=prompt_for_inv_item(0);
    itemgroup_t group;
    if(item==NULL) {
      show_messages("cancelled"); return 0;
    }
    group=item->Itemtype->Type;
    if(group==IG_FOOD||group==IG_CORPSE||group==IG_CONDIMENT) {
       return p_eat(item);
    } else {
       show_messages("You can't eat that!!");
       return 0;
    }
}

void p_trywield()
{
    /* Prompt for an item, see if its a wieldable, try equipping it. */
    item_t *item=prompt_for_inv_item(1); /* 1 to allow unwielding */
    char nstring[80];
    equipspace_t space;
    if(item==NULL) return;
    if(item==phonyitem) { /* player chose '-'. Try unwielding. */
       space=phonyitem->Number;
       item=player.Equipment[space];
       if(item==NULL) { /* Nothing equipped there anyways */
          sprintf(strbuf,"You continue to %s nothing.",space==IG_WEAPON?"wield":"ready");
          show_messages(strbuf);
       } else { /* unwield/unready */
          item_basename(nstring,item); /* or use describe, but dont say "the"*/
          sprintf(strbuf,"You %s the %s.",space==IG_WEAPON?"unwield":"unready",nstring);
          show_messages(strbuf);
          player.Equipment[space]=NULL;
       }
       return;
    }
    space=classify_equipspace(item);
    if(space>EQUIP_RANGED) {
       item_basename(nstring,item);
       sprintf(strbuf,"You can't wield a %s!",nstring);
       show_messages(strbuf);
       return;
    }
    p_equip(item); /* We could mebbe do this ourselves... */
}

void p_trywear()
{
    /* Prompt for an item, see if its a wearable, try equipping it. */
    item_t *item=prompt_for_inv_item(0);
    char nstring[80];
    equipspace_t space;
    if(item==NULL) return;
    item_basename(nstring,item);
    space=classify_equipspace(item);
    if(space<EQUIP_SUIT||space>EQUIP_NECKWEAR) {
       sprintf(strbuf,"You can't wear a %s!",nstring);
       show_messages(strbuf);
       return;
    }
    if(p_has_equipped(item)) {
      show_messages("You're already wearing that!");
      return;
    }
    /* Doh, this is a muck up */
    sprintf(strbuf,"You put the %s on.",nstring);
    show_messages(strbuf);
    if(item->Cursed) {
      show_messages("You feel uneasy."); /* so you should... */
    }
    p_equip(item); /*Except for rings, these are all the same */
}

void p_tryunwear()
{
    /* Prelim version: prompt for an item, see if its equipped AND
    an item you'd wear. If so, do p_unequip() or whatever. */
    int cov;
    char nstring[80];
    item_t *item=prompt_for_inv_item(0);
    equipspace_t space;
    if(item==NULL) return;
    item_basename(nstring,item);
    space=classify_equipspace(item);
    if(space<EQUIP_SUIT||space==EQUIP_NONE) {
       sprintf(strbuf,"Now why would you be wearing a %s?", nstring);
       show_messages(strbuf);
       return;
    }
    /* Now see if it's equipped.*/
    if(player.Equipment[space]==item|| (space==EQUIP_RING1&&player.Equipment[space=EQUIP_RING2]==item)) {
       /* Actually, don't bother with p_unequip.  We've done
       enough here already that we can do the rest ourselves */
       if(item->Cursed) { /* Can't take it off! */
          sprintf(strbuf,"You can't remove the %s- it seems to be cursed!",nstring);
          show_messages(strbuf);
          return;
       }
       player.Equipment[space]=NULL;
       sprintf(strbuf,"You take the %s off.",nstring);
       show_messages(strbuf);
       cov=item->Itemtype->Typedata.Arm.Cov;
       player.Var_stats.Cov-=cov;
       player.Var_stats.Pravg-=(item->Itemtype->Typedata.Arm.Prot*cov);
    } else {
       sprintf(strbuf,"But you're not wearing the %s!",nstring);
       show_messages(strbuf);
    }
    return;
}

void p_gainexp(int e)
{
    unsigned int nlv=player.Var_stats.Level+1;
    /*unsigned long olde=player.Var_stats.Exp; */
    uint64_t targe=global_scale_expreq(nlv);
    player.Var_stats.Exp+=e;
    if(player.Var_stats.Exp>=targe) {
       player.Var_stats.Level++;
       player.Var_stats.MaxHp=global_scale_hp(nlv,player.Fixed_stats.Con);
       clev=(dlev+nlv)/2;
       sprintf(strbuf,"Welcome to experience level %d!",nlv);
       show_messages(strbuf);
       /* dlev would be dungeon level, but that's a stopgap variable */
    }
}

void p_hurt(int dmg)
{
#ifdef AUTOPILOT
      int r;
#endif
    player.Var_stats.Hp-=dmg;
    show_messages("Ow! That hurt!"); /* Um... */
    if (player.Var_stats.Hp<0) {
#ifndef AUTOPILOT
       show_messages("Something unnamed kills you!");/*Should be passed as arg*/
       quit=2;
#else
       /* Just put the player somewhere else instead */
       player.Var_stats.Hp=player.Var_stats.MaxHp;
       r=pick_room(tls->Totalrooms);
       player.X=tls->Room[r].Xcentre;
       player.Y=tls->Room[r].Ycentre;
#endif
    }
}

/* --- end of file --- */
