/* ==== race.c ==== */
/* START BOILERPLATE
   (=== a copyright notice to include everywhere, now by "boil.pl" ===)
   Part of the source of the program "Porrog", a roguelike game.
   Copyright (C) 2002-2004  Tom Barnes-Lawrence
   Current web address: www.angelfire.com/super2/duologue/software/porrog.html
   Current email address: tomble@usermail.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-- END BOILERPLATE */

#include <stdlib.h>
#include <math.h>
#include "porrog_defs.h"
#include "item_t.h"
#include "structures.h"

extern unsigned long ex_nx[];
extern unsigned long ex_tot[];

/* There's no different races yet. The player currently has just a
 single hard-coded race, but these functions are used to map the
 details of that race into the global stat scale. */

unsigned short global_scale_stat(unsigned short racialmax,unsigned short training)
{
   /* This function makes the output stat asymptotically approach racialmax,
   and starts off with output at 0 when training is 0. */
   /*With the constant at 50, the output is at 33% when training is 25,
   at 50% when training is 50, at 66% when training is 100,
   at 75% when training is 150, doesnt reach 90% till training is 450...
   When constant is lower, advancement is faster.
   In fact, looking further, SLOPECONSTANT is always the value of "training"
   at which the output is 50%! */
#define SLOPECONSTANT 50
   long numer=racialmax*SLOPECONSTANT;
   return(racialmax-(numer/(training+SLOPECONSTANT)));
}

unsigned short global_scale_hp(unsigned short elev,unsigned short con)
{
   /* Never been able to thoroughly decide on this */
   unsigned short conroot=pow(con,0.66667); /* root 1.5 of con */
   return(conroot*log(1+elev));
}

/* this prolly isn't a good way to do this */
int global_scale_weapdmg(item_t *weap,unsigned int str,unsigned int elev)
{
   int wgt;
   double eff; /* efficiency- ability to wield something so heavy */
   if(weap==NULL) { /* unarmed */
      return(sqrt(str)*0.25*log(M_E+(elev/4))); /* stupid */
   } else { /* take weapon into account */
      wgt=(weap->Itemtype->Weight)/4;
      if(str<wgt) {
         if(str<(wgt/2)) eff=0;
         else eff=(str-(wgt/2))/wgt;
      } else eff=1;
      if(weap->Itemtype->Type==IG_STONE) {
         return((eff*wgt)/4); /* ick */
      }
      return(eff*(weap->Itemtype->Typedata.Weapon.Dmg)); /* yuck */
   } 
}

int global_scale_weaptht(item_t *weap,unsigned int dex,unsigned int elev)
{
   int tht=pow(dex,0.66667);;
   if(weap!=NULL)tht+=weap->Itemtype->Typedata.Weapon.Dmg;
   if(tht<0)tht=0; /* should that be 1? */
   if(tht>100)tht=100; /* should that be 99? */
   return(tht);
}

/* returns some argument in range 0->100.
Or should that be 1->100, or 0->99? */
int global_scale_evasion(unsigned int agi, unsigned int elev)
{
   double lboostf=(.25-((.24*20)/(elev+20))); /* level based evasion boost */
   /* at l 0, boosts to avg of 12.5%.
     at l 20, boosts to avg of 18.75%.
     At l 40, boosts to avg of 20.83%
     It could do with tweaking! */
   double agiroot=pow(agi,0.66667);
   return( ((1-lboostf)*agiroot)+(100*lboostf));

}


void init_expscale()
{
   int l;
   double decay;
   ex_nx[1]=0;
   ex_nx[2]=10;
   for(l=3;l<27;l++) {
      decay=((double)l/25);
      ex_nx[l]=ex_nx[l-1]*(2.04-decay);
   }
   /* from the values of ex_nx[l], we can work out ex_tot[l].
   But for l<21, it's best to precalculate them (to avoid doing
   multiple sums). */
   ex_tot[1]=0;
   for(l=2;l<27;l++) {
      ex_tot[l]=ex_tot[l-1]+ex_nx[l];
      /* ex_tot[l] is total experience required to reach level l */
   }
   /* This function isn't remotely intensive, but you wouldn't want to
   be doing it all the time. */
}

/* returns total exp required to reach level */
uint64_t global_scale_expreq(unsigned int level)
{
   if(level<27) return(ex_tot[level]);
   else return(ex_tot[26]+((level-26)*ex_nx[26]));
}

